(function () {
	'use strict';

	class Dictionary {
	    constructor() {
	        this._keys = [];
	        this._values = [];
	    }
	    get values() {
	        return this._values;
	    }
	    get keys() {
	        return this._keys;
	    }
	    set(key, value) {
	        var index = this.indexOf(key);
	        if (index >= 0) {
	            this._values[index] = value;
	            return;
	        }
	        this._keys.push(key);
	        this._values.push(value);
	    }
	    indexOf(key) {
	        var index = this._keys.indexOf(key);
	        return index;
	    }
	    get(key) {
	        var index = this.indexOf(key);
	        return index < 0 ? null : this._values[index];
	    }
	    remove(key) {
	        var index = this.indexOf(key);
	        if (index >= 0) {
	            this._keys.splice(index, 1);
	            this._values.splice(index, 1);
	            return true;
	        }
	        return false;
	    }
	    clear() {
	        this._values.length = 0;
	        this._keys.length = 0;
	    }
	}

	class Sprite extends Laya.Sprite {
	    constructor() {
	        super();
	        this._touchable = false;
	        this.isDestroyed = false;
	        this.isChanged = true;
	    }
	    get touchable() { return this._touchable; }
	    set touchable(can) {
	        this._touchable = can;
	        this.mouseEnabled = can;
	    }
	    removeFromParent(dispose = false) {
	        this.removeSelf();
	    }
	    set color(col) {
	    }
	    get width() {
	        var rec = this.getBounds();
	        return rec.width;
	    }
	    set width(value) {
	        var w0 = this.width / this.scaleX;
	        this.scaleX = value / w0;
	    }
	    get height() {
	        var rec = this.getBounds();
	        return rec.height;
	    }
	    set height(value) {
	        var w0 = this.height / this.scaleY;
	        this.scaleY = value / w0;
	    }
	    addChildWithChild(child, tarName, xy, scale = false) {
	        let tar = this.getChildByName(tarName);
	        if (tar == null) {
	            this.addChild(child);
	        }
	        else {
	            this.addChildAt(child, this.getChildIndex(tar) + 1);
	            if (xy) {
	                child.x = tar.x;
	                child.y = tar.y;
	            }
	            if (scale) {
	                child.scaleX *= tar.scaleX;
	                child.scaleY *= tar.scaleY;
	            }
	        }
	    }
	    setSpriteData(data) {
	        this.spriteData = data;
	        if (this.spriteData && (this.spriteData[0] instanceof Array) == false && this.spriteData.length > 0 && this.spriteData[0]["url"] == null) {
	            this.setMetaData(this.spriteData[0]);
	        }
	    }
	    setMetaData(data) {
	        this.metaData = data;
	        if (this.metaData["鼠标区域"]) {
	            if (this.metaData["鼠标区域"]["形状"] == "圆") {
	                this.hit_半径 = this.metaData["鼠标区域"]["半径"];
	                this.hit_半径half = this.hit_半径 / 2;
	                this.hit_半径2 = this.hit_半径 * this.hit_半径;
	            }
	        }
	    }
	    clearChild(dispose = false) {
	        this.removeChildren();
	    }
	    setDisplayColor(display, color) {
	        display.color = color;
	    }
	    getBounds() {
	        if (this.isChanged == false && this.bonds != null && this.lastX == this.x && this.lastY == this.y && this.lastRo == this.rotation && this.lastSx == this.scaleX
	            && this.lastSy == this.scaleY && this.lastSkewx == this.skewX && this.lastSkewy == this.skewY) {
	            return this.bonds;
	        }
	        this.bonds = super.getBounds();
	        this.lastX = this.x;
	        this.lastY = this.y;
	        this.lastRo = this.rotation;
	        this.lastSx = this.scaleX;
	        this.lastSy = this.scaleY;
	        this.lastSkewx = this.skewX;
	        this.lastSkewy = this.skewY;
	        this.isChanged = false;
	        if (this.bonds.width == 0 || this.bonds.height == 0)
	            this.isChanged = true;
	        return this.bonds;
	    }
	    addChild(node) {
	        this.isChanged = true;
	        return super.addChild(node);
	    }
	    addChildAt(node, index) {
	        this.isChanged = true;
	        return super.addChildAt(node, index);
	    }
	    removeChild(node) {
	        this.isChanged = true;
	        return super.removeChild(node);
	    }
	    removeChildAt(index) {
	        this.isChanged = true;
	        return super.removeChildAt(index);
	    }
	    removeChildByName(name) {
	        this.isChanged = true;
	        return super.removeChildByName(name);
	    }
	    removeChildren(beginIndex, endIndex) {
	        this.isChanged = true;
	        return super.removeChildren(beginIndex, endIndex);
	    }
	    destroyF() {
	        this.isDestroyed = true;
	        this.removeChildren();
	        this.removeFromParent(true);
	    }
	}

	class Tool_StringBuild {
	    static replaceSTR(str, str1, str2) {
	        var out = "";
	        var arr = str.split(str1);
	        for (var i = 0; i < arr.length; i++) {
	            if (i > 0)
	                out += str2;
	            out += arr[i];
	        }
	        return out;
	    }
	    static onRemoveEnterSpace(str) {
	        return str.trim();
	    }
	    static rebuild_by_length(str, len, _chinese) {
	        var out = "";
	        if (_chinese == false) {
	            out = str.substr(0, len);
	        }
	        else {
	            var nowL = 0;
	            var count = 0;
	            while (count < str.length) {
	                if (nowL >= len)
	                    break;
	                var char = str.charAt(count);
	                var code = str.charCodeAt(count);
	                out += char;
	                if (code >= 10000)
	                    nowL += 2;
	                else
	                    nowL += 1;
	                count++;
	            }
	        }
	        return out;
	    }
	    static getNewSTRBySlip(str1, str2) {
	        var arr = str1.split(str2);
	        str1 = "";
	        while (arr.length > 0) {
	            str1 += arr[0];
	            arr.shift();
	        }
	        return str1;
	    }
	    static getArr_By_split(str1, str2, _changeInstance = null) {
	        var arr = str1.split(str2);
	        if (_changeInstance) {
	            var i;
	            var l = arr.length;
	            for (i = 0; i < l; i++) {
	                if (_changeInstance == "int")
	                    arr[i] = Tool_Function.onChangeInstance(arr[i]);
	                else if (_changeInstance == "Number")
	                    arr[i] = Tool_Function.onChangeInstance(arr[i], "num");
	            }
	        }
	        return arr;
	    }
	    static onInsertF(str, index, strTar) {
	        var str0 = str.slice(0, index);
	        var str1 = str.slice(index);
	        return str0 + strTar + str1;
	    }
	    static removeStrFromStr(str, index, length, insertStr) {
	        let str0 = str.substr(0, index);
	        let str1 = str.substr(index + length + 1);
	        if (insertStr)
	            return str0 + insertStr + str1;
	        return str0 + str1;
	    }
	    static fixNumberTOString(val, num, fillStr, isFront = true) {
	        let str = val + "";
	        while (str.length < num) {
	            if (isFront == true)
	                str = fillStr + str;
	            else
	                str += fillStr;
	        }
	        return str;
	    }
	    static getLength(str) {
	        var out = str.length;
	        var count = 0;
	        while (count < str.length) {
	            var code = str.charCodeAt(count);
	            if (code >= 10000)
	                out++;
	            count++;
	        }
	        return out;
	    }
	}

	class MyByteArray {
	    constructor(cmd = -1) {
	        this.CMD = cmd;
	    }
	    writeF(val) {
	        if (this.WriteB == null) {
	            this.WriteB = new Laya.Byte();
	            this.WriteB.endian = MyByteArray.ENDIAN;
	        }
	        if (val instanceof Array) {
	            for (var i = 0; i < val.length; i++) {
	                this.write(val[i]);
	            }
	        }
	        else
	            this.write(val);
	    }
	    write(val) {
	        if (typeof val == "number") {
	            if (val != Tool_Function.onForceConvertType(val)) {
	                this.WriteB.writeByte(5);
	                this.WriteB.writeFloat64(val);
	                return;
	            }
	            var n = Math.abs(val);
	            if (n < 127) {
	                this.WriteB.writeByte(1);
	                this.WriteB.writeByte(val);
	            }
	            else if (n < 32767) {
	                this.WriteB.writeByte(2);
	                this.WriteB.writeInt16(val);
	            }
	            else if (n < 2147483647) {
	                this.WriteB.writeByte(3);
	                this.WriteB.writeInt32(val);
	            }
	            else {
	                this.WriteB.writeByte(5);
	                this.WriteB.writeFloat64(val);
	            }
	        }
	        else if (typeof val === "string") {
	            this.WriteB.writeByte(4);
	            this.WriteB.writeUTFString(val);
	        }
	        else if (val == null || val == undefined) {
	            this.WriteB.writeByte(8);
	        }
	        else if (typeof val == "boolean") {
	            this.WriteB.writeByte(7);
	            if (val == true)
	                this.WriteB.writeByte(1);
	            else
	                this.WriteB.writeByte(0);
	        }
	        else if (val instanceof Array) {
	            this.WriteB.writeByte(6);
	            this.write(val.length);
	            for (let i = 0; i < val.length; i++) {
	                this.write(val[i]);
	            }
	            return;
	        }
	        else if (val instanceof Dictionary) {
	            var dic = val;
	            var arr_key = dic.keys;
	            var arr_val = dic.values;
	            this.WriteB.writeByte(9);
	            this.write(arr_key.length);
	            for (let i2 = 0; i2 < arr_key.length; i2++) {
	                this.write(arr_key[i2]);
	                this.write(arr_val[i2]);
	            }
	        }
	        else {
	            arr_key = [];
	            arr_val = [];
	            for (var key in val) {
	                arr_key.push(key);
	                arr_val.push(val[key]);
	            }
	            this.WriteB.writeByte(9);
	            this.write(arr_key.length);
	            for (let i2 = 0; i2 < arr_key.length; i2++) {
	                this.write(arr_key[i2]);
	                this.write(arr_val[i2]);
	            }
	        }
	    }
	    readF(buf) {
	        this.Arr_val = [];
	        while (buf.bytesAvailable > 0) {
	            this.Arr_val.push(readNext());
	        }
	        function readNext() {
	            var val;
	            var n = buf.readByte();
	            if (n == 1)
	                val = buf.readByte();
	            else if (n == 2)
	                val = buf.readInt16();
	            else if (n == 3)
	                val = buf.readInt32();
	            else if (n == 4) {
	                val = buf.readUTFString();
	                if (val.indexOf("\\n") != -1) {
	                    val = Tool_StringBuild.replaceSTR(val, "\\n", "\n");
	                }
	            }
	            else if (n == 5)
	                val = buf.readFloat64();
	            else if (n == 6) {
	                var lenType = buf.readByte();
	                var len;
	                if (lenType == 1)
	                    len = buf.readByte();
	                else if (lenType == 2)
	                    len = buf.readInt32();
	                var arr = [];
	                for (; len > 0; len--) {
	                    arr.push(readNext());
	                }
	                return arr;
	            }
	            else if (n == 7) {
	                val = Tool_Function.onForceConvertType(buf.readByte());
	                if (val == 0)
	                    return false;
	                else
	                    return true;
	            }
	            else if (n == 8) {
	                return null;
	            }
	            else if (n == 9) {
	                var dic = {};
	                lenType = buf.readByte();
	                if (lenType == 1)
	                    len = buf.readByte();
	                else if (lenType == 2)
	                    len = buf.readInt16();
	                var key;
	                for (; len > 0; len--) {
	                    key = readNext();
	                    dic[key] = readNext();
	                }
	                return dic;
	            }
	            else {
	                Config.LogF("Unkownd Type In MyByteArray's readF!CMD==" + this.CMD + ",n==" + n);
	                return null;
	            }
	            return val;
	        }
	    }
	}
	MyByteArray.ENDIAN = "BIG_ENDIAN";

	class MBase64 {
	    static encode(data) {
	        var bytes = new Laya.Byte();
	        bytes.writeUTFBytes(data);
	        return this.encodeByteArray(bytes);
	    }
	    static encodeByteArray(data) {
	        var output = "";
	        var dataBuffer;
	        var outputBuffer = new ArrayBuffer(4);
	        data.pos = 0;
	        while (data.bytesAvailable > 0) {
	            dataBuffer = [];
	            for (let i = 0; i < 3 && data.bytesAvailable > 0; i++) {
	                dataBuffer[i] = data.readUint8();
	            }
	            outputBuffer[0] = (dataBuffer[0] & 0xfc) >> 2;
	            outputBuffer[1] = ((dataBuffer[0] & 0x03) << 4) | ((dataBuffer[1]) >> 4);
	            outputBuffer[2] = ((dataBuffer[1] & 0x0f) << 2) | ((dataBuffer[2]) >> 6);
	            outputBuffer[3] = dataBuffer[2] & 0x3f;
	            for (let j = dataBuffer.length; j < 3; j++) {
	                outputBuffer[j + 1] = 64;
	            }
	            for (let k = 0; k < 4; k++) {
	                output += this.BASE64_CHARS.charAt(outputBuffer[k]);
	            }
	        }
	        return output;
	    }
	    static decode(data) {
	        var bytes = this.decodeToByteArray(data);
	        return bytes.readUTFBytes(bytes.length);
	    }
	    static decodeToByteArray(data) {
	        var output = new Laya.Byte();
	        output.endian = MyByteArray.ENDIAN;
	        var dataBuffer = new ArrayBuffer(4);
	        var outputBuffer = new ArrayBuffer(3);
	        for (let i = 0; i < data.length; i += 4) {
	            for (let j = 0; j < 4 && i + j < data.length; j++) {
	                dataBuffer[j] = this.BASE64_CHARS.indexOf(data.charAt(i + j));
	            }
	            outputBuffer[0] = (dataBuffer[0] << 2) + ((dataBuffer[1] & 0x30) >> 4);
	            outputBuffer[1] = ((dataBuffer[1] & 0x0f) << 4) + ((dataBuffer[2] & 0x3c) >> 2);
	            outputBuffer[2] = ((dataBuffer[2] & 0x03) << 6) + dataBuffer[3];
	            for (let k = 0; k < 3; k++) {
	                if (dataBuffer[k + 1] == 64)
	                    break;
	                output.writeByte(outputBuffer[k]);
	            }
	        }
	        output.pos = 0;
	        return output;
	    }
	}
	MBase64.BASE64_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
	MBase64.version = "1.0.0";

	class Tool_Function {
	    static onRunFunction(f, ...arg) {
	        if (f == null) {
	            return;
	        }
	        if (f instanceof Function) {
	            return f.apply(null, arg);
	        }
	        else {
	            if (f.caller instanceof Sprite && f.caller.isDestroyed == true) {
	                console.error("f.call已清理，不执行");
	                return null;
	            }
	            return f.runWith(arg);
	        }
	    }
	    static setHandlerOnce(f, once) {
	        if (f instanceof Laya.Handler) {
	            f.once = once;
	        }
	        else if (f instanceof Array) {
	            for (let i = 0; i < f.length; i++) {
	                this.setHandlerOnce(f[i], once);
	            }
	        }
	    }
	    static compareHandlers(f, fun) {
	        if (f == null && fun == null)
	            return true;
	        if (f instanceof Laya.Handler) {
	            if (fun == null || (fun instanceof Laya.Handler && fun == f)) {
	                return true;
	            }
	            else if (fun == null || (fun instanceof Function && f.method == fun)) {
	                return true;
	            }
	        }
	        else if (f instanceof Function) {
	            if (fun == null || (fun instanceof Function && fun == f)) {
	                return true;
	            }
	            else if (fun == null || (fun instanceof Laya.Handler && fun.method == f)) {
	                return true;
	            }
	        }
	        return false;
	    }
	    static onTouchable(tar, needParent = true, needChild = true) {
	        if (tar == null)
	            return;
	        tar.touchable = true;
	    }
	    static onTouchThrough(tar, needParent = true, needChild = true) {
	        if (tar == null)
	            return;
	        tar.mouseThrough = true;
	        if (needChild) {
	            for (let i = 0; i < tar.numChildren; i++) {
	                let ont = tar.getChildAt(i);
	                if (ont instanceof Sprite)
	                    this.onTouchThrough(ont, false, needChild);
	            }
	        }
	    }
	    static getLastClassName(tar) {
	        var className = tar.constructor.toString();
	        if (className.indexOf("(") != -1) {
	            className = className.slice(0, className.indexOf("("));
	            className = className.substr(className.indexOf(" ") + 1);
	        }
	        return className;
	    }
	    static isTypeOf(obj, type) {
	        if (obj == null)
	            return false;
	        if (type == "number" || type == "num") {
	            if (typeof obj === "number")
	                return true;
	            return false;
	        }
	        if (type == "int") {
	            if (typeof obj === "number" && Tool_Function.onForceConvertType(obj) == obj) {
	                return true;
	            }
	            return false;
	        }
	        if (type == "string" || type == "str") {
	            if (typeof obj === "string")
	                return true;
	            return false;
	        }
	        if (type == "obj") {
	            if (obj instanceof Object)
	                return true;
	            return false;
	        }
	        if (type == "arr") {
	            if (obj instanceof Array)
	                return true;
	            return false;
	        }
	        return false;
	    }
	    static deg2rad(deg) {
	        return deg / 180.0 * Math.PI;
	    }
	    static rad2deg(rad) {
	        return rad * 180 / Math.PI;
	    }
	    static onForceConvertType(tar, type = "int") {
	        return Tool_Function.onChangeInstance(tar, type);
	    }
	    static onForceConvertType_string(tar) {
	        return Tool_Function.onChangeInstance(tar, "str");
	    }
	    static onGetLengthBetweenPoint(point1, point2) {
	        var lx = point1.x - point2.x;
	        var ly = point1.y - point2.y;
	        var l = lx * lx + ly * ly;
	        return Math.sqrt(l);
	    }
	    static onGetXLengthBetweenPoint(point1, point2) {
	        let l = Tool_Function.onGetLengthBetweenPoint(point1, point2);
	        if (l == 0) {
	            return 0;
	        }
	        let ang = Tool_Function.onGetAngleBetweenPoint(point1, point2);
	        let sx;
	        let sy;
	        if (ang <= 90) {
	            sx = l * Math.sin(Tool_Function.deg2rad(ang));
	        }
	        else if (ang <= 180) {
	            sx = l * Math.cos(Tool_Function.deg2rad(ang - 90));
	        }
	        else if (ang <= 270) {
	            sx = -l * Math.sin(Tool_Function.deg2rad(ang - 180));
	        }
	        else {
	            sx = -l * Math.sin(Tool_Function.deg2rad(360 - ang));
	        }
	        if (sx)
	            sy = Math.sqrt(l * l - sx * sx);
	        else if (sy)
	            sx = Math.sqrt(l * l - sy * sy);
	        if (point1.y > point2.y)
	            sy = -sy;
	        return sx;
	    }
	    static onGetYLengthBetweenPoint(point1, point2) {
	        let l = Tool_Function.onGetLengthBetweenPoint(point1, point2);
	        if (l == 0) {
	            return 0;
	        }
	        let ang = Tool_Function.onGetAngleBetweenPoint(point1, point2);
	        let sx;
	        let sy;
	        if (ang <= 90) {
	            sx = l * Math.sin(Tool_Function.deg2rad(ang));
	        }
	        else if (ang <= 180) {
	            sx = l * Math.cos(Tool_Function.deg2rad(ang - 90));
	        }
	        else if (ang <= 270) {
	            sx = -l * Math.sin(Tool_Function.deg2rad(ang - 180));
	        }
	        else {
	            sx = -l * Math.sin(Tool_Function.deg2rad(360 - ang));
	        }
	        if (sx)
	            sy = Math.sqrt(l * l - sx * sx);
	        else if (sy)
	            sx = Math.sqrt(l * l - sy * sy);
	        if (point1.y > point2.y)
	            sy = -sy;
	        return sy;
	    }
	    static onGetAngleBetweenPoint(point1, point2) {
	        var ang = -1;
	        var lx = point2.x - point1.x;
	        var ly = point2.y - point1.y;
	        if (lx == 0 && ly == 0) {
	            return -1;
	        }
	        if (ly == 0) {
	            if (lx >= 0) {
	                ang = 90;
	            }
	            else {
	                ang = 270;
	            }
	        }
	        else if (ly < 0) {
	            if (lx >= 0) {
	                ang = this.rad2deg(Math.atan(lx / -ly));
	            }
	            else {
	                ang = 360 - this.rad2deg(Math.atan(-lx / -ly));
	            }
	        }
	        else {
	            if (lx >= 0) {
	                ang = 90 + 90 - this.rad2deg(Math.atan(lx / ly));
	            }
	            else {
	                ang = 270 - (90 - this.rad2deg(Math.atan(-lx / ly)));
	            }
	        }
	        return ang;
	    }
	    static onGetSpdXY_bySpd_and_Ang(spd, ang) {
	        var spdx;
	        var spdy;
	        if (ang == -1) {
	            spdx = spdy = spd;
	        }
	        else if (ang == 0) {
	            spdx = 0;
	            spdy = -spd;
	        }
	        else if (ang < 90) {
	            spdx = spd * Math.tan(this.deg2rad(ang));
	            spdy = -Math.sqrt(spd * spd - spdx * spdx);
	        }
	        else if (ang == 90) {
	            spdx = spd;
	            spdy = 0;
	        }
	        else if (ang < 180) {
	            spdy = spd * Math.tan(this.deg2rad(ang - 90));
	            spdx = Math.sqrt(spd * spd - spdy * spdy);
	        }
	        else if (ang == 180) {
	            spdx = 0;
	            spdy = spd;
	        }
	        else if (ang < 270) {
	            spdx = -spd * Math.tan(this.deg2rad(ang - 180));
	            spdy = Math.sqrt(spd * spd - spdx * spdx);
	        }
	        else if (ang == 270) {
	            spdx = -spd;
	            spdy = 0;
	        }
	        else {
	            spdx = -spd * Math.tan(this.deg2rad(360 - ang));
	            spdy = Math.sqrt(spd * spd - spdx * spdx);
	        }
	        return { "x": spdx, "y": spdy };
	    }
	    static onChangeInstance(tar, type = "int") {
	        var tarType = typeof (tar);
	        if (type == "int") {
	            if (tar == null || isNaN(tar))
	                return 0;
	            if (tarType == "number") {
	                return parseInt(tar + "");
	            }
	            else if (tarType === "string") {
	                if (tar == "true")
	                    return 1;
	                if (tar == "false")
	                    return 0;
	                return parseInt(tar);
	            }
	            else {
	                return 0;
	            }
	        }
	        else if (type == "num") {
	            if (tar == null || isNaN(tar))
	                return 0;
	            if (tarType == "number") {
	                return tar;
	            }
	            else if (tarType === "string") {
	                if (tar == "true")
	                    return 1;
	                if (tar == "false")
	                    return 0;
	                return parseFloat(tar);
	            }
	            else {
	                return 0;
	            }
	        }
	        if (type == "str") {
	            return String(tar);
	        }
	        return null;
	    }
	    static validateIdCard(card) {
	        if (card === '') {
	            return false;
	        }
	        if (isCardNo(card) === false) {
	            return false;
	        }
	        if (checkProvince(card) === false) {
	            return false;
	        }
	        if (checkBirthday(card) === false) {
	            return false;
	        }
	        if (checkParity(card) === false) {
	            return false;
	        }
	        return true;
	        function isCardNo(card) {
	            var reg = /(^\d{15}$)|(^\d{17}(\d|X)$)/;
	            if (reg.test(card) === false) {
	                return false;
	            }
	            return true;
	        }
	        function checkProvince(card) {
	            var vcity = { 11: "北京", 12: "天津", 13: "河北", 14: "山西", 15: "内蒙古",
	                21: "辽宁", 22: "吉林", 23: "黑龙江", 31: "上海", 32: "江苏",
	                33: "浙江", 34: "安徽", 35: "福建", 36: "江西", 37: "山东", 41: "河南",
	                42: "湖北", 43: "湖南", 44: "广东", 45: "广西", 46: "海南", 50: "重庆",
	                51: "四川", 52: "贵州", 53: "云南", 54: "西藏", 61: "陕西", 62: "甘肃",
	                63: "青海", 64: "宁夏", 65: "新疆", 71: "台湾", 81: "香港", 82: "澳门", 91: "国外"
	            };
	            var province = card.substr(0, 2);
	            if (vcity[province] == undefined) {
	                return false;
	            }
	            return true;
	        }
	        function checkBirthday(card) {
	            var len = card.length;
	            if (len == '15') {
	                var re_fifteen = /^(\d{6})(\d{2})(\d{2})(\d{2})(\d{3})$/;
	                var arr_data = card.match(re_fifteen);
	                var year = arr_data[2];
	                var month = arr_data[3];
	                var day = arr_data[4];
	                var birthday = new Date('19' + year + '/' + month + '/' + day);
	                return verifyBirthday('19' + year, month, day, birthday);
	            }
	            if (len == '18') {
	                var re_eighteen = /^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X)$/;
	                var arr_data = card.match(re_eighteen);
	                var year = arr_data[2];
	                var month = arr_data[3];
	                var day = arr_data[4];
	                var birthday = new Date(year + '/' + month + '/' + day);
	                return verifyBirthday(year, month, day, birthday);
	            }
	            return false;
	        }
	        function verifyBirthday(year, month, day, birthday) {
	            var now = new Date();
	            var now_year = now.getFullYear();
	            if (birthday.getFullYear() == year && (birthday.getMonth() + 1) == month && birthday.getDate() == day) {
	                var time = now_year - year;
	                if (time >= 3 && time <= 100) {
	                    return true;
	                }
	                return false;
	            }
	            return false;
	        }
	        function checkParity(card) {
	            card = changeFivteenToEighteen(card);
	            var len = card.length;
	            if (len == '18') {
	                var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
	                var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
	                var cardTemp = 0, i, valnum;
	                for (i = 0; i < 17; i++) {
	                    cardTemp += card.substr(i, 1) * arrInt[i];
	                }
	                valnum = arrCh[cardTemp % 11];
	                if (valnum == card.substr(17, 1)) {
	                    return true;
	                }
	                return false;
	            }
	            return false;
	        }
	        function changeFivteenToEighteen(card) {
	            if (card.length == '15') {
	                var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
	                var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
	                var cardTemp = 0, i;
	                card = card.substr(0, 6) + '19' + card.substr(6, card.length - 6);
	                for (i = 0; i < 17; i++) {
	                    cardTemp += card.substr(i, 1) * arrInt[i];
	                }
	                card += arrCh[cardTemp % 11];
	                return card;
	            }
	            return card;
	        }
	    }
	    static onDate_int_to_Date(ms, sec) {
	        var d = new Date(sec > 0 ? sec * 1000 : ms);
	        console.log(this.onDate_Data_to_String(d, true));
	        return d;
	    }
	    static onDate_String_to_Date(str) {
	        if (str == null)
	            return null;
	        var year;
	        var month;
	        var day;
	        var hour = null;
	        var minute = null;
	        var second = null;
	        var typeMiddle = -1;
	        var index;
	        var index2 = -1;
	        index = str.indexOf("年");
	        if (index > 0) {
	            typeMiddle = 0;
	        }
	        else {
	            index = str.indexOf("/");
	            if (index > 0) {
	                typeMiddle = 1;
	            }
	            else {
	                index = str.indexOf(".");
	                if (index > 0) {
	                    typeMiddle = 2;
	                }
	                else {
	                    index = str.indexOf("-");
	                    if (index > 0) {
	                        typeMiddle = 3;
	                    }
	                }
	            }
	        }
	        if (typeMiddle == -1)
	            return null;
	        year = this.onForceConvertType(str.slice(0, index));
	        if (typeMiddle == 0)
	            index2 = str.indexOf("月");
	        else if (typeMiddle == 1)
	            index2 = str.indexOf("/", index + 1);
	        else if (typeMiddle == 2)
	            index2 = str.indexOf(".", index + 1);
	        else if (typeMiddle == 3)
	            index2 = str.indexOf("-", index + 1);
	        if (index2 == -1)
	            return null;
	        month = this.onForceConvertType(str.slice(index + 1, index2));
	        index = index2;
	        index2 = -1;
	        if (typeMiddle == 0)
	            index2 = str.indexOf("日");
	        else
	            index2 = str.indexOf(" ", index + 1);
	        if (index2 == -1)
	            return null;
	        day = this.onForceConvertType(str.slice(index + 1, index2));
	        index = index2;
	        index2 = -1;
	        index2 = str.indexOf("时");
	        if (index2 > 0) {
	            hour = this.onForceConvertType(str.slice(index + 1, index2));
	            index = index2;
	            index2 = -1;
	            index2 = str.indexOf("分");
	            if (index2 == -1)
	                return null;
	            minute = this.onForceConvertType(str.slice(index + 1, index2));
	            index = index2;
	            index2 = -1;
	            index2 = str.indexOf("秒");
	            if (index2 != -1) {
	                second = this.onForceConvertType(str.slice(index + 1, index2));
	            }
	        }
	        else {
	            var str2 = str.substr(index + 1);
	            var arr = str2.split(":");
	            if (arr.length >= 2) {
	                hour = this.onForceConvertType(arr[0]);
	                minute = this.onForceConvertType(arr[1]);
	                if (arr.length == 3)
	                    second = this.onForceConvertType(arr[2]);
	            }
	        }
	        var d = new Date(year, month - 1, day, hour, minute, second);
	        return d;
	    }
	    static onDate_Data_to_String(d, needUTC = false, useChinese = true) {
	        if (d == null)
	            return null;
	        var str;
	        if (needUTC)
	            str = d.getUTCFullYear() + "年" + (d.getUTCMonth() + 1) + "月" + d.getUTCDate() + "日" + d.getUTCHours() + ":" + d.getUTCMinutes() + ":" + d.getUTCSeconds();
	        str = d.getFullYear() + "年" + (d.getMonth() + 1) + "月" + d.getDate() + "日" + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds();
	        if (useChinese == false) {
	            str = str.replace("年", "-");
	            str = str.replace("月", "-");
	            str = str.replace("日", "");
	        }
	        return str;
	    }
	    static onDate_getBetween(d1, d2, timeName = "时") {
	        var day = this.getBetweenDays(d2, d1);
	        if (timeName == "天") {
	            return day;
	        }
	        var min = Tool_Function.onForceConvertType(d2.getMinutes() - d1.getMinutes() + (day * 24 + d2.getHours() - d1.getHours()) * 60);
	        if (timeName == "分") {
	            return min;
	        }
	        if (timeName == "时") {
	            return Tool_Function.onForceConvertType(min / 60);
	        }
	        if (timeName == "秒") {
	            var sec = Tool_Function.onForceConvertType(d2.getSeconds() - d1.getSeconds() + min * 60);
	            return sec;
	        }
	        return 0;
	    }
	    static onDate_getAllDays(d) {
	        var ms = d.getTime();
	        var day = (ms - (-8 * 60 * 60 * 1000)) / (24 * 60 * 60 * 1000);
	        return Tool_Function.onForceConvertType(day);
	    }
	    static getBetweenDays(d1, d2) {
	        return this.onDate_getAllDays(d1) - this.onDate_getAllDays(d2);
	    }
	    static getColorStrFromInt(col) {
	        let str = col.toString(16);
	        while (str.length < 6) {
	            str = "0" + str;
	        }
	        return "#" + str;
	    }
	    static onColor_getRed(color) { return (color >> 16) & 0xff; }
	    static onColor_getGreen(color) { return (color >> 8) & 0xff; }
	    static onColor_getBlue(color) { return color & 0xff; }
	    static onColor_rgb(red, green, blue) {
	        return (red << 16) | (green << 8) | blue;
	    }
	    static onColor_argb(alpha, red, green, blue) {
	        return (alpha << 24) | (red << 16) | (green << 8) | blue;
	    }
	    static stringToByte(str) {
	        var bytes = MBase64.decodeToByteArray(str);
	        return bytes;
	    }
	    static byteToString(arr) {
	        return MBase64.encodeByteArray(arr);
	    }
	    static getPerValueInArray(val, arr) {
	        let per = 0;
	        if (arr == null || arr.length < 4) {
	            Config.LogError("Tool_Function.getPerValueInArray：arr数据格式错误！");
	            return per;
	        }
	        if (val <= arr[0])
	            return arr[1];
	        if (val >= arr[arr.length - 2])
	            return arr[arr.length - 1];
	        for (let i = 0; i < arr.length; i += 2) {
	            if (i == arr.length - 1 || val <= arr[i]) {
	                if (arr[i] - arr[i - 2] == 0)
	                    per = 0;
	                else
	                    per = arr[i - 1] + (val - arr[i - 2]) * (arr[i + 1] - arr[i - 1]) / (arr[i] - arr[i - 2]);
	            }
	        }
	        return per;
	    }
	    static getRandomInt(max, min, seed) {
	        max = max || 1;
	        min = min || 0;
	        seed = (seed * 9301 + 49297) % 233280;
	        let rnd = seed / 233280.0;
	        return parseInt(Math.floor(min + rnd * (max - min)) + "");
	    }
	    static math_limiteNumber(now, min, max) {
	        if (now < min)
	            return min;
	        if (now > max)
	            return max;
	        return now;
	    }
	    static getBezierPoints(anchorpoints, pointsAmount) {
	        var points = [];
	        for (var i = 0; i < pointsAmount; i++) {
	            var point = MultiPointBezier(anchorpoints, i / pointsAmount);
	            points.push(point);
	        }
	        return points;
	        function MultiPointBezier(points, t) {
	            var len = points.length;
	            var x = 0, y = 0;
	            var erxiangshi = function (start, end) {
	                var cs = 1, bcs = 1;
	                while (end > 0) {
	                    cs *= start;
	                    bcs *= end;
	                    start--;
	                    end--;
	                }
	                return (cs / bcs);
	            };
	            for (var i = 0; i < len; i++) {
	                var point = points[i];
	                x += point.x * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
	                y += point.y * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
	            }
	            return { x: x, y: y };
	        }
	    }
	}

	class MyEventManager {
	    constructor() {
	        this.DIC_Listener = {};
	        this.isEventing = false;
	        this.arrWaite = [];
	    }
	    dispatchF(type, val = null) {
	        if (this.isEventing == true) {
	            this.arrWaite.push(["dispatch", type, val]);
	            return;
	        }
	        this.isEventing = true;
	        if (this.DIC_Listener[type] != null) {
	            var dic = this.DIC_Listener[type];
	            for (let i = 0; i < dic.keys.length; i++) {
	                let id = dic.keys[i];
	                var arr2 = dic.get(id);
	                var f = arr2[0];
	                if (val != null && arr2[2] != null) {
	                    Tool_Function.onRunFunction(f, arr2[2], val);
	                }
	                else if (val != null) {
	                    Tool_Function.onRunFunction(f, val);
	                }
	                else if (arr2[2] != null) {
	                    Tool_Function.onRunFunction(f, arr2[2]);
	                }
	                else {
	                    Tool_Function.onRunFunction(f);
	                }
	                if (arr2[1] == true) {
	                    this.isEventing = false;
	                    this.removeListenF(type, f);
	                    this.isEventing = true;
	                }
	            }
	        }
	        this.isEventing = false;
	        while (this.arrWaite.length > 0) {
	            var arr = this.arrWaite.shift();
	            var waiteType = arr.shift();
	            if (waiteType == "add") {
	                this.addListenF(arr[0], arr[1], arr[2], arr[3]);
	            }
	            else if (waiteType == "remove") {
	                this.removeListenF(arr[0], arr[1]);
	            }
	            else if (waiteType == "dispatch") {
	                this.dispatchF(arr[0], arr[1]);
	                break;
	            }
	        }
	    }
	    addListenF(type, fun, val = null, onece = false) {
	        if (this.isEventing == true) {
	            this.arrWaite.push(["add", type, fun, val, onece]);
	            return;
	        }
	        var dic;
	        if (this.DIC_Listener[type] == null) {
	            dic = new Dictionary();
	            dic.set(fun, [fun, onece, val]);
	            this.DIC_Listener[type] = dic;
	        }
	        else {
	            dic = this.DIC_Listener[type];
	            dic.set(fun, [fun, onece, val]);
	        }
	    }
	    removeListenF(type, fun) {
	        if (this.isEventing == true) {
	            this.arrWaite.push(["remove", type, fun]);
	            return;
	        }
	        var dic;
	        if (this.DIC_Listener[type] != null) {
	            if (fun == null) {
	                delete this.DIC_Listener[type];
	            }
	            else {
	                dic = this.DIC_Listener[type];
	                for (let i = 0; i < dic.keys.length; i++) {
	                    let fun2 = dic.keys[i];
	                    if (Tool_Function.compareHandlers(fun, fun2) == true) {
	                        dic.remove(fun2);
	                        break;
	                    }
	                }
	            }
	        }
	    }
	    haveListener(type) {
	        var num = 0;
	        if (this.DIC_Listener[type] != null) {
	            var dic = this.DIC_Listener[type];
	            num = dic.keys.length;
	        }
	        return num;
	    }
	    destroyF() {
	        for (var id in this.DIC_Listener) {
	            this.removeListenF(id, null);
	            delete this.DIC_Listener[id];
	        }
	    }
	}

	class MyTweenerManager {
	    constructor() {
	        this.MM = MainManager.getInstence();
	        this.DIC_Tween = new Dictionary();
	        this.isListener = false;
	        this.Pause = false;
	    }
	    actionF() {
	        if (this.Pause)
	            return;
	        for (let i = 0; i < this.DIC_Tween.keys.length; i++) {
	            let id = this.DIC_Tween.keys[i];
	            if (this.DIC_Tween.get(id) == null) {
	                this.DIC_Tween.remove(id);
	                i--;
	            }
	            else
	                this.DIC_Tween.get(id).moveF();
	        }
	        if (this.DIC_Tween.keys.length == 0) {
	            this.isListener = false;
	            this.MM.removeEnterFrameFun(this.FunActionF);
	            this.FunActionF = null;
	        }
	    }
	    newTween(mc, t) {
	        if (this.DIC_Tween.keys.indexOf(mc) == -1) {
	            if (t == null)
	                return;
	        }
	        if (t == null) {
	            if (this.DIC_Tween.keys.indexOf(mc) != -1) {
	                this.DIC_Tween.remove(mc);
	            }
	        }
	        else {
	            this.DIC_Tween.set(mc, t);
	            if (this.isListener == false) {
	                this.isListener = true;
	                if (this.FunActionF == null) {
	                    this.FunActionF = Laya.Handler.create(this, this.actionF, null, false);
	                    this.MM.addEnterFrameFun(this.FunActionF);
	                }
	            }
	        }
	    }
	    stopAll() {
	        this.DIC_Tween.clear();
	        this.isListener = false;
	        this.MM.removeEnterFrameFun(this.FunActionF);
	        this.FunActionF = null;
	    }
	}

	class MyPools {
	    constructor() {
	        this.Dic = {};
	        this.Dic_Max = {};
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new MyPools();
	        return this.instance;
	    }
	    registF(Name, max = 50) {
	        if (this.Dic[Name] != null) {
	            this.removeF(Name);
	        }
	        this.Dic[Name] = [];
	        this.Dic_Max[Name] = max;
	    }
	    hasPool(Name) {
	        return this.Dic[Name] != null;
	    }
	    getFromPool(Name, ...arg) {
	        if (this.Dic[Name] == null) {
	            return null;
	        }
	        if (this.Dic[Name].length == 0) {
	            return null;
	        }
	        return this.Dic[Name].shift();
	    }
	    returnToPool(Name, one) {
	        if (this.Dic[Name] == null || this.Dic[Name].length >= this.Dic_Max[Name]) {
	            one = Tool_ObjUtils.destroyF_One(one);
	            return;
	        }
	        this.Dic[Name].push(one);
	    }
	    removeF(Name) {
	        if (this.Dic[Name] != null) {
	            Tool_ObjUtils.destroyF_One(this.Dic[Name]);
	            delete this.Dic[Name];
	        }
	    }
	    destroyF() {
	        MyPools.instance = null;
	        if (this.Dic == null) {
	            return;
	        }
	        for (var Name in this.Dic) {
	            this.removeF(Name);
	        }
	        this.Dic = null;
	    }
	}

	class Tool_ArrayUtils {
	    static getNewArrayFromPool(...arg) {
	        var arr;
	        if (MyPools.getInstance().hasPool("Array") == false) {
	            arr = MyPools.getInstance().getFromPool("Array");
	        }
	        if (arr == null) {
	            arr = [];
	        }
	        if (arg.length > 0) {
	            var i;
	            for (i = 0; i < arg.length; i++) {
	                arr[i] = arg[i];
	            }
	        }
	        return arr;
	    }
	    static returnArrayToPool(arr) {
	        if (arr == null) {
	            return;
	        }
	        if (MyPools.getInstance().hasPool("Array") == false) {
	            return;
	        }
	        arr.length = 0;
	        MyPools.getInstance().returnToPool("Array", arr);
	    }
	    static copyArr(arr, onlyLowLevel = false) {
	        if (arr == null)
	            return null;
	        var out = [];
	        for (var i = 0; i < arr.length; i++) {
	            if (onlyLowLevel == false)
	                out[i] = Tool_ObjUtils.CopyF(arr[i]);
	            else
	                out[i] = arr[i];
	        }
	        return out;
	    }
	    static isEqual(arr1, arr2) {
	        if (arr1 == null && arr2 == null)
	            return true;
	        if (arr1 == null || arr2 == null)
	            return false;
	        if (arr1.length != arr2.length)
	            return false;
	        for (var i = 0; i < arr1.length; i++) {
	            if (Tool_ObjUtils.isEqual(arr1[i], arr2[i]) == false)
	                return false;
	        }
	        return true;
	    }
	    static onRemoveLoopNumber(arr) {
	        var dic = {};
	        for (var i = 0; i < arr.length; i++) {
	            if (arr[i] instanceof Number) {
	                if (dic[arr[i]] == true)
	                    arr.splice(i--, 1);
	                else
	                    dic[arr[i]] = true;
	            }
	        }
	    }
	    static getRadomOneFromArray(arr) {
	        let i = Tool_Function.onForceConvertType(Math.random() * arr.length);
	        return arr[i];
	    }
	    static onRadomSort(arr) {
	        var cloneArr = arr.slice();
	        arr.length = 0;
	        var i = cloneArr.length;
	        while (i) {
	            arr.push(cloneArr.splice(Tool_Function.onForceConvertType(Math.random() * i--), 1)[0]);
	        }
	    }
	    static onNumber_SmallToBig(a, b) {
	        if (a < b)
	            return -1;
	        else
	            return 1;
	    }
	    static onNumber_BigToSmall(a, b) {
	        if (a > b)
	            return -1;
	        else
	            return 1;
	    }
	    static onSerchAndRemove(arr, val) {
	        if (arr == null)
	            return false;
	        var po = arr.indexOf(val);
	        if (po == -1)
	            return false;
	        arr.splice(po, 1);
	        return true;
	    }
	    static insertInto(arr, index, val) {
	        if (index == -1 || index >= arr.length)
	            arr.push(val);
	        else {
	            arr.splice(index, 0, val);
	        }
	    }
	}

	class Image extends Sprite {
	    constructor(t) {
	        super();
	        this.sx = 1;
	        this.sy = 1;
	        this.w0 = 0;
	        this.h0 = 0;
	        this.isMipmap = false;
	        this.texture = t;
	    }
	    get texture() {
	        return this.tmpTexture;
	    }
	    set texture(value) {
	        this.tmpTexture = value;
	        if (this.texture) {
	            if (this.tmpTexture) {
	                this.graphics.clear();
	            }
	        }
	        if (this.tmpTexture == null)
	            return;
	        this.graphics.drawImage(this.tmpTexture, 0, 0);
	        this.w0 = this.tmpTexture.width;
	        this.h0 = this.tmpTexture.height;
	        if (this.isMipmap) {
	            this.isMipmap = false;
	            this.setMipMap(true);
	        }
	    }
	    setMipMap(real) {
	        if (this.isMipmap == real)
	            return;
	        this.isMipmap = real;
	        if (this.isMipmap == true) {
	            this.graphics.clear();
	            let mt = new Laya.Matrix();
	            mt.scale(4, 4);
	            this.graphics.drawTexture(this.tmpTexture, 0, 0, this.w0 / 4, this.h0 / 4, mt);
	        }
	        else {
	            this.graphics.clear();
	            this.graphics.drawImage(this.tmpTexture, 0, 0);
	        }
	    }
	    get height() {
	        if (this.h0 == 0) {
	            this.h0 = this.getBounds().height / this.scaleY;
	        }
	        return this.h0 * this.scaleY;
	    }
	    set height(value) {
	        this.scaleY = value / this.h0;
	    }
	    get width() {
	        if (this.w0 == 0) {
	            this.w0 = this.getBounds().width / this.scaleX;
	        }
	        return this.w0 * this.scaleX;
	    }
	    set width(value) {
	        this.scaleX = value / this.w0;
	    }
	    destroy(destroyChild = true) {
	        this.tmpTexture = null;
	        super.destroy(destroyChild);
	    }
	}

	class SwfImage extends Image {
	    constructor(t) {
	        super(t);
	    }
	}

	class Color {
	    static getAlpha(color) { return (color >> 24) & 0xff; }
	    static getRed(color) { return (color >> 16) & 0xff; }
	    static getGreen(color) { return (color >> 8) & 0xff; }
	    static getBlue(color) { return color & 0xff; }
	    static setAlpha(color, alpha) {
	        return (color & 0x00ffffff) | (alpha & 0xff) << 24;
	    }
	    static setRed(color, red) {
	        return (color & 0xff00ffff) | (red & 0xff) << 16;
	    }
	    static setGreen(color, green) {
	        return (color & 0xffff00ff) | (green & 0xff) << 8;
	    }
	    static setBlue(color, blue) {
	        return (color & 0xffffff00) | (blue & 0xff);
	    }
	    static rgb(red, green, blue) {
	        return (red << 16) | (green << 8) | blue;
	    }
	    static argb(alpha, red, green, blue) {
	        return (alpha << 24) | (red << 16) | (green << 8) | blue;
	    }
	    static toVector(color, out = null) {
	        if (out == null)
	            out = new Array();
	        out[0] = ((color >> 16) & 0xff) / 255.0;
	        out[1] = ((color >> 8) & 0xff) / 255.0;
	        out[2] = (color & 0xff) / 255.0;
	        out[3] = ((color >> 24) & 0xff) / 255.0;
	        return out;
	    }
	    static multiply(color, factor) {
	        if (factor == 0.0)
	            return 0x0;
	        var alpha = ((color >> 24) & 0xff) * factor;
	        var red = ((color >> 16) & 0xff) * factor;
	        var green = ((color >> 8) & 0xff) * factor;
	        var blue = (color & 0xff) * factor;
	        if (alpha > 255)
	            alpha = 255;
	        if (red > 255)
	            red = 255;
	        if (green > 255)
	            green = 255;
	        if (blue > 255)
	            blue = 255;
	        return this.argb(alpha, red, green, blue);
	    }
	    static numbererpolate(startColor, endColor, ratio) {
	        var startA = (startColor >> 24) & 0xff;
	        var startR = (startColor >> 16) & 0xff;
	        var startG = (startColor >> 8) & 0xff;
	        var startB = (startColor) & 0xff;
	        var endA = (endColor >> 24) & 0xff;
	        var endR = (endColor >> 16) & 0xff;
	        var endG = (endColor >> 8) & 0xff;
	        var endB = (endColor) & 0xff;
	        var newA = startA + (endA - startA) * ratio;
	        var newR = startR + (endR - startR) * ratio;
	        var newG = startG + (endG - startG) * ratio;
	        var newB = startB + (endB - startB) * ratio;
	        return (newA << 24) | (newR << 16) | (newG << 8) | newB;
	    }
	    Color() { throw new Error(); }
	}
	Color.WHITE = 0xffffff;
	Color.SILVER = 0xc0c0c0;
	Color.GRAY = 0x808080;
	Color.BLACK = 0x000000;
	Color.RED = 0xff0000;
	Color.MAROON = 0x800000;
	Color.YELLOW = 0xffff00;
	Color.OLIVE = 0x808000;
	Color.LIME = 0x00ff00;
	Color.GREEN = 0x008000;
	Color.AQUA = 0x00ffff;
	Color.TEAL = 0x008080;
	Color.BLUE = 0x0000ff;
	Color.NAVY = 0x000080;
	Color.FUCHSIA = 0xff00ff;
	Color.PURPLE = 0x800080;

	class ImageNum extends Sprite {
	    constructor(fname, _swf) {
	        super();
	        this.W = -1;
	        this.Arr_Save = [];
	        this.MidL = 0;
	        this.vAlin = "左";
	        this.MinL = -1;
	        this.Col = -1;
	        this.tmpCol = -1;
	        this.OnlyNumberColor = false;
	        this.neetW_to_10000 = -1;
	        this.Arr数字 = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "+", "-", "."];
	        this.SwfName = _swf;
	        if (this.SwfName == null)
	            this.SwfName = ImageNum.SWFDefault;
	        this.FrontName = fname;
	    }
	    addSource(arr = null) {
	        if (arr == null)
	            arr = [];
	        arr.push([this.SwfName, "swf", "FrontSource/" + this.SwfName]);
	        return arr;
	    }
	    setValue(str, val = null) {
	        switch (str) {
	            case "间隔":
	                this.MidL = val;
	                break;
	            case "对齐":
	                this.vAlin = val;
	                break;
	            case "最少位数":
	                this.MinL = val;
	                break;
	            case "颜色":
	                if (this.Col == val)
	                    return;
	                this.Col = val;
	                for (var i = 0; i < this.Arr_Save.length; i++) {
	                    if (this.OnlyNumberColor && this.Arr_Save[i][2] == true)
	                        continue;
	                    if (this.Arr_Save[i][1])
	                        this.Arr_Save[i][1].color = this.Col;
	                }
	                break;
	            case "局部颜色":
	                tmpCol = val;
	                break;
	            case "颜色仅数字":
	                this.OnlyNumberColor = val;
	                if (this.Col != -1) {
	                    var tmpCol = this.Col;
	                    this.Col = -1;
	                    this.setValue("颜色", tmpCol);
	                }
	                break;
	            case "父对象":
	                if (val instanceof SwfImage) {
	                    val.parent.addChildAt(this, val.parent.getChildIndex(val));
	                    val.parent.removeChild(val);
	                    this.x = val.x;
	                    this.y = val.y;
	                    this.TypeOfParent = "img";
	                }
	                else if (val instanceof Sprite) {
	                    val.removeChildren();
	                    val.addChild(this);
	                    this.TypeOfParent = "spr";
	                }
	                break;
	            case "x":
	                if (this.TypeOfParent == "spr")
	                    this.parent.x = val;
	                else
	                    this.x = val;
	                break;
	            case "y":
	                if (this.TypeOfParent == "spr")
	                    this.parent.y = val;
	                else
	                    this.y = val;
	                break;
	            case "stage监听":
	                this.once(Laya.Event.REMOVED, this, this.onRemoveF);
	                break;
	            case "万":
	                this.neetW_to_10000 = val;
	                break;
	        }
	    }
	    showF(val) {
	        this.L = 0;
	        if (this.Layer == null) {
	            this.Layer = new Sprite();
	            this.addChild(this.Layer);
	            this.tm = MySourceManager.getInstance();
	        }
	        else {
	            this.Layer.removeChildren();
	            this.clearSaveF();
	        }
	        if (val == null)
	            val = "";
	        else if (typeof (val) === "number" && this.neetW_to_10000 > 0 && val >= this.neetW_to_10000) {
	            val = Tool_Function.onChangeInstance((val / 10000).toFixed(1)) + "万";
	        }
	        this.appendShowF(val);
	    }
	    appendShowF(val) {
	        this.NowStr = Tool_Function.onChangeInstance(val, "str");
	        var x0 = this.L == 0 ? 0 : (this.L * this.W + (this.L - 1) * this.MidL);
	        this.L = this.NowStr.length;
	        if (this.L == 0)
	            return;
	        var more0 = -1;
	        if (this.L < this.MinL) {
	            more0 = this.MinL - this.L;
	            this.L = this.MinL;
	        }
	        for (var i = 0; i < this.L; i++) {
	            var char = this.NowStr.charAt(i);
	            if (more0-- >= 0)
	                char = "0";
	            if (char == ".")
	                char = "小数点";
	            var is数字 = true;
	            var img;
	            img = ImageNum.getCache(this.FrontName + char, this.SwfName);
	            if (img == null) {
	                img = this.tm.getImgFromSwf(this.SwfName, "img_" + this.FrontName + "Num_" + char);
	                if (img == null && this.SwfName != ImageNum.SWFDefault)
	                    img = this.tm.getImgFromSwf(ImageNum.SWFDefault, "img_" + this.FrontName + "Num_" + char);
	                if (img == null && char == "-") {
	                    img = ImageNum.getCache(this.FrontName + "减", this.SwfName);
	                    if (img == null && this.SwfName != ImageNum.SWFDefault)
	                        img = this.tm.getImgFromSwf(ImageNum.SWFDefault, "img_" + this.FrontName + "Num_" + "减");
	                }
	                if (img == null)
	                    console.log("找不到数字：" + this.SwfName + "：" + this.FrontName + "：" + char);
	            }
	            if (this.Arr数字.indexOf(char) == -1)
	                is数字 = false;
	            if (is数字 == false)
	                this.Arr_Save.push([char, img]);
	            else
	                this.Arr_Save.push([char, img, true]);
	            if (img) {
	                if (this.W == -1) {
	                    var imgW = this.tm.getImgFromSwf(this.SwfName, "img_" + this.FrontName + "FrontWidth");
	                    if (imgW == null) {
	                        if (img) {
	                            this.W = img.width + img.pivotX;
	                        }
	                    }
	                    else {
	                        this.W = imgW.width;
	                    }
	                }
	                img.x = x0 + i * this.W + this.MidL;
	                if (this.tmpCol != -1) {
	                    if (this.OnlyNumberColor == false || is数字 == true)
	                        img.color = this.tmpCol;
	                }
	                else if (this.Col != -1) {
	                    if (this.OnlyNumberColor == false || is数字 == true)
	                        img.color = this.Col;
	                }
	                this.Layer.addChild(img);
	            }
	        }
	        if (this.vAlin == "左")
	            this.Layer.x = 0;
	        else if (this.vAlin == "中")
	            this.Layer.x = -(this.L * this.W + (this.L - 1) * this.MidL) / 2 + this.W / 2;
	        else if (this.vAlin == "右")
	            this.Layer.x = -(this.L * this.W + (this.L - 1) * this.MidL) + this.W;
	        this.tmpCol = -1;
	    }
	    runF(fend, s, e, t, head, tail) {
	        this.FunRunover = fend;
	        this.NowNum = s;
	        this.EndNum = e;
	        this.RunTime = t;
	        this.STR_head = head;
	        if (this.STR_head == null)
	            this.STR_head = "";
	        this.STR_tail = tail;
	        if (this.STR_tail == null)
	            this.STR_tail = "";
	        this.SpdNum = (this.EndNum - this.NowNum) / t;
	        this.showF(this.STR_head + Tool_Function.onChangeInstance(this.NowNum) + this.STR_tail);
	        this.FunRunCheck = Laya.Handler.create(this, this.runCheck, null, false);
	        MainManager._instence.addEnterFrameFun(this.FunRunCheck);
	    }
	    runCheck() {
	        if (this.NowNum == this.EndNum) {
	            MainManager._instence.removeEnterFrameFun(this.FunRunCheck);
	            if (this.FunRunover)
	                this.FunRunover.run();
	            return;
	        }
	        this.NowNum += this.SpdNum;
	        if (this.SpdNum > 0 && this.NowNum > this.EndNum)
	            this.NowNum = this.EndNum;
	        else if (this.SpdNum < 0 && this.NowNum < this.EndNum)
	            this.NowNum = this.EndNum;
	        if (--this.RunTime <= 0)
	            this.NowNum = this.EndNum;
	        this.showF(this.STR_head + Tool_Function.onChangeInstance(this.NowNum) + this.STR_tail);
	    }
	    getValue(want) {
	        switch (want) {
	            case "右边x":
	                var x0;
	                if (this.vAlin == "左")
	                    x0 = this.x + (this.L * this.W + (this.L - 1) * this.MidL);
	                else if (this.vAlin == "中")
	                    x0 = this.x - this.Layer.x + this.W;
	                else if (this.vAlin == "右")
	                    x0 = this.x;
	                if (this.TypeOfParent == "spr")
	                    x0 += this.parent.x;
	                return x0;
	            default: return this[want];
	        }
	    }
	    onRemoveF(e) {
	        this.destroyF();
	    }
	    destroyF() {
	        if (this.Layer) {
	            this.removeChild(this.Layer);
	            this.Layer.destroyF();
	            this.Layer = null;
	        }
	        this.tm = null;
	        this.clearSaveF();
	        this.removeFromParent(true);
	    }
	    clearSaveF() {
	        for (var i = 0; i < this.Arr_Save.length; i++) {
	            if (this.Arr_Save[i][1]) {
	                if (this.Col != -1)
	                    this.Arr_Save[i][1].color = Color.WHITE;
	                ImageNum.addCache(this.FrontName + this.Arr_Save[i][0], this.SwfName, this.Arr_Save[i][1]);
	            }
	            this.Arr_Save.splice(i--, 1);
	        }
	    }
	    static addNeedCache(fName, real) {
	        this.NeedCache[fName] = real;
	    }
	    static removeNeedCache(fName) {
	        if (fName == null) {
	            this.NeedCache = {};
	            this.clearF();
	        }
	        else
	            this.NeedCache[fName] = false;
	    }
	    static isNeedCache(fName) {
	        if (this.NeedCache[fName] != null)
	            return this.NeedCache[fName];
	        return false;
	    }
	    static getCache(str, swf) {
	        if (this.isNeedCache(str) == false)
	            return null;
	        if (this.DicCache == null)
	            this.DicCache = {};
	        if (this.DicCache[swf] == null)
	            this.DicCache[swf] = {};
	        if (this.DicCache[swf][str] == null)
	            this.DicCache[swf][str] = [];
	        var img;
	        if (this.DicCache[swf][str].length > 0) {
	            img = this.DicCache[swf][str][0];
	            this.DicCache[swf][str].shift();
	        }
	        return img;
	    }
	    static addCache(str, swf, obj) {
	        if (this.isNeedCache(str) == false)
	            return;
	        if (this.DicCache == null)
	            this.DicCache = {};
	        if (this.DicCache[swf] == null)
	            this.DicCache[swf] = {};
	        if (this.DicCache[swf][str] == null)
	            this.DicCache[swf][str] = [];
	        this.DicCache[swf][str].push(obj);
	    }
	    static clearF() {
	        if (this.DicCache == null)
	            return;
	        for (var swf in this.DicCache) {
	            var dic = this.DicCache[swf];
	            for (var key in dic) {
	                var arr = dic[key];
	                for (var i = 0; i < arr.length; i++) {
	                    arr[i].removeFromParent(true);
	                }
	                delete dic[key];
	            }
	        }
	        this.DicCache = null;
	    }
	    static onCacheAndSaveF(fName, swf) {
	        this.addNeedCache(fName, true);
	        for (var i = 0; i < 10; i++) {
	            var count = 5;
	            while (count-- > 0) {
	                this.addCache(fName + i, swf, MySourceManager.instance.getImgFromSwf(swf, "img_" + fName + "Num_" + i));
	            }
	        }
	    }
	}
	ImageNum.SWFDefault = "SWF_Default";
	ImageNum.NeedCache = {};

	var Handler = Laya.Handler;
	class MainManagerOne {
	    constructor() {
	        this.Arr_EnterFrame = [];
	        this.Dic_Delay = new Dictionary();
	        this.MM = MainManager._instence;
	        this.id = MainManagerOne.countID++;
	    }
	    addEnterFrameFun(fun) {
	        if (this.Arr_EnterFrame.indexOf(fun) == -1) {
	            this.Arr_EnterFrame.push(fun);
	            this.MM.addEnterFrameFun(fun);
	        }
	    }
	    removeEnterFrameFun(fun) {
	        if (fun == null) {
	            for (var i = 0; i < this.Arr_EnterFrame.length; i++) {
	                var f = this.Arr_EnterFrame[i];
	                this.Arr_EnterFrame.splice(i--, 1);
	                this.MM.removeEnterFrameFun(f);
	            }
	        }
	        else {
	            var index = -1;
	            for (var i = 0; i < this.Arr_EnterFrame.length; i++) {
	                var f = this.Arr_EnterFrame[i];
	                if (Tool_Function.compareHandlers(f, fun) == true) {
	                    index = i;
	                }
	                if (index != -1) {
	                    break;
	                }
	            }
	            if (index != -1) {
	                this.Arr_EnterFrame.splice(index, 1);
	                this.MM.removeEnterFrameFun(fun);
	            }
	        }
	    }
	    addEnterSecondFun(fun) {
	        if (this.Arr_EnterSecond == null) {
	            this.Arr_EnterSecond = [];
	            this.FunEnter = Handler.create(this, this.enterF, null, false);
	            this.MM.addEnterFrameFun(this.FunEnter);
	        }
	        if (this.hasEnterSecondFunction(fun) == -1) {
	            this.Arr_EnterSecond.push([fun, 0]);
	            console.log("成功添加enterSecond");
	        }
	        else {
	            console.log("重复添加entersecond");
	        }
	    }
	    removeEnterSecondFun(fun) {
	        if (this.Arr_EnterSecond == null) {
	            return;
	        }
	        var i;
	        for (i = 0; i < this.Arr_EnterSecond.length; i++) {
	            if (this.Arr_EnterSecond[i] != null && (this.Arr_EnterSecond[i][0] == fun || this.Arr_EnterSecond[i][0].method == fun)) {
	                this.Arr_EnterSecond[i] = Tool_ObjUtils.destroyF_One(this.Arr_EnterSecond[i]);
	                console.log("移除enterSecond");
	            }
	        }
	    }
	    hasEnterSecondFunction(fun) {
	        if (this.Arr_EnterSecond == null)
	            return -1;
	        var i;
	        var l = this.Arr_EnterSecond.length;
	        for (i = 0; i < l; i++) {
	            if (this.Arr_EnterSecond[i] != null && Tool_Function.compareHandlers(this.Arr_EnterSecond[i][0], fun) == true) {
	                return i;
	            }
	        }
	        return -1;
	    }
	    enterF() {
	        var i;
	        if (this.Arr_EnterSecond) {
	            var count;
	            for (i = 0; i < this.Arr_EnterSecond.length; i++) {
	                if (this.Arr_EnterSecond[i] != null) {
	                    count = this.Arr_EnterSecond[i][1]++;
	                    if (count >= Config.playSpeedTrue) {
	                        this.Arr_EnterSecond[i][1] = 0;
	                        Tool_Function.onRunFunction(this.Arr_EnterSecond[i][0]);
	                    }
	                }
	            }
	            for (i = 0; i < this.Arr_EnterSecond.length; i++) {
	                if (this.Arr_EnterSecond[i] == null) {
	                    this.Arr_EnterSecond.splice(i--, 1);
	                }
	            }
	        }
	        else {
	        }
	    }
	    add_delayFunction(fun, delay, val = null) {
	        this.remove_delayFunction(fun);
	        var h = Handler.create(this, this.tmpFunction, [fun, val]);
	        this.Dic_Delay.set(fun, h);
	        this.MM.add_delayFunction(h, delay);
	    }
	    tmpFunction(fun, val) {
	        if (this.MM == null)
	            return;
	        if (this.Dic_Delay) {
	            for (let i = 0; i < this.Dic_Delay.keys.length; i++) {
	                if (Tool_Function.compareHandlers(fun, this.Dic_Delay.keys[i])) {
	                    this.Dic_Delay.remove(this.Dic_Delay.keys[i]);
	                    break;
	                }
	            }
	        }
	        Tool_Function.onRunFunction(fun, val);
	    }
	    remove_delayFunction(f) {
	        if (this.Dic_Delay == null || this.MM == null)
	            return;
	        for (let i = 0; i < this.Dic_Delay.keys.length; i++) {
	            if (Tool_Function.compareHandlers(f, this.Dic_Delay.keys[i])) {
	                var h = this.Dic_Delay.get(this.Dic_Delay.keys[i]);
	                if (h != null)
	                    this.MM.remove_delayFunction(h);
	                this.Dic_Delay.remove(this.Dic_Delay.keys[i]);
	                break;
	            }
	        }
	    }
	    destroyF() {
	        for (let i = 0; i < this.Arr_EnterFrame.length; i++) {
	            this.removeEnterFrameFun(this.Arr_EnterFrame[i--]);
	        }
	        this.Arr_EnterFrame = null;
	        if (this.Dic_Delay && this.Dic_Delay.keys) {
	            while (this.Dic_Delay.keys.length > 0) {
	                this.remove_delayFunction(this.Dic_Delay.keys[0]);
	            }
	            this.Dic_Delay.clear();
	        }
	        this.Dic_Delay = null;
	        if (this.Arr_EnterSecond) {
	            this.MM.removeEnterFrameFun(this.FunEnter);
	        }
	        this.Arr_EnterSecond = Tool_ObjUtils.destroyF_One(this.Arr_EnterSecond);
	        this.MM = null;
	    }
	}
	MainManagerOne.countID = 0;

	class TextFormat {
	    constructor(t) {
	        this.T = t;
	    }
	    get color() { return Number("0x" + this.T.color.substr(1)); }
	    set color(value) {
	        this._color = value;
	        if (this.T) {
	            this.T.color = "#" + this._color.toString(16);
	        }
	    }
	    get font() { return this.T.font; }
	    set font(value) { this.T.font = value; }
	    get size() { return this.T.fontSize; }
	    set size(value) { this.T.fontSize = value; }
	    get horizontalAlign() { return this.T.align; }
	    set horizontalAlign(value) {
	        if (value == "左" || value == "left")
	            this.T.align = "left";
	        else if (value == "右" || value == "right")
	            this.T.align = "right";
	        else
	            this.T.align = "center";
	    }
	    get italic() { return this.T.italic; }
	    set italic(value) { this.T.italic = value; }
	    get bold() { return this.T.bold; }
	    set bold(value) { this.T.bold = value; }
	}

	class TextField extends Laya.Text {
	    constructor(_w, _h, text, formate = null) {
	        super();
	        this.isCleared = false;
	        this._autoScale = false;
	        this._touchable = false;
	        this.text = text;
	        this.lastStr = text;
	    }
	    get changed() {
	        let bol = this.lastStr != this.text;
	        this.lastStr = this.text;
	        return bol;
	    }
	    get format() {
	        if (this._format == null) {
	            this._format = new TextFormat(this);
	        }
	        else {
	            this._format.T = this;
	        }
	        return this._format;
	    }
	    set format(value) {
	        this._format = value;
	    }
	    get autoScale() { return this._autoScale; }
	    set autoScale(_x) {
	        this._autoScale = _x;
	    }
	    get touchable() { return this._touchable; }
	    set touchable(can) {
	        this._touchable = can;
	        this.mouseEnabled = can;
	    }
	    removeFromParent(dispose = false) {
	        this.isCleared = true;
	        if (this.parent != null) {
	            this.parent.removeChild(this);
	        }
	        if (dispose) {
	            this.destroy();
	        }
	    }
	}

	class Tool_Textfield {
	    static newTextfield(w, h, text, font, size, col, vUD = "中", hLR = "中", italic = false, bold = false, multiline = false) {
	        var t = new TextField(w, h, text);
	        t.font = font;
	        t.fontSize = size;
	        if (Tool_Function.isTypeOf(col, "num"))
	            t.color = Tool_Function.getColorStrFromInt(col);
	        else
	            t.color = col;
	        if (vUD == "上" || vUD == "up")
	            t.valign = "top";
	        else if (vUD == "下" || vUD == "down")
	            t.valign = "bottom";
	        else
	            t.valign = "middle";
	        if (hLR == "左" || hLR == "left")
	            t.align = "left";
	        else if (hLR == "右" || hLR == "right")
	            t.align = "right";
	        else
	            t.align = "center";
	        if (multiline)
	            t.wordWrap = true;
	        t.width = w;
	        t.height = h;
	        t.italic = italic;
	        t.bold = bold;
	        t.text = text;
	        t.autoScale = true;
	        return t;
	    }
	    static changeTextfieldColor(t, col) {
	        if (t == null)
	            return;
	        if (Tool_Function.isTypeOf(col, "num"))
	            t.color = Tool_Function.getColorStrFromInt(col);
	        else
	            t.color = col;
	    }
	}

	var AutoBitmap = Laya.AutoBitmap;
	class SwfS9Image extends Sprite {
	    constructor() {
	        super();
	        var g = new AutoBitmap();
	        this.graphics = g;
	    }
	    get sizeGrid() {
	        if (this.graphics.sizeGrid)
	            return this.graphics.sizeGrid.join(",");
	        return null;
	    }
	    set sizeGrid(value) {
	        this.graphics.sizeGrid = Laya.UIUtils.fillArray(Laya.Styles.defaultSizeGrid, value, Number);
	    }
	    get source() {
	        return this.graphics.source;
	    }
	    set source(value) {
	        if (!this.graphics)
	            return;
	        this.graphics.source = value;
	        super.event(Laya.Event.LOADED);
	        super.repaint();
	    }
	    set width(value) {
	        this.graphics.width = value == 0 ? 0.0000001 : value;
	    }
	    get width() {
	        return this.graphics.width;
	    }
	    set height(value) {
	        this.graphics.height = value == 0 ? 0.0000001 : value;
	    }
	    get height() {
	        return this.graphics.height;
	    }
	    removeFromParent(dispose = false) {
	        if (this.parent != null) {
	            this.parent.removeChild(this);
	        }
	        if (dispose) {
	            this.destroy();
	        }
	    }
	    destroy(destroyChild = true) {
	        if (this.graphics)
	            this.graphics.destroy();
	        super.destroy(true);
	    }
	}

	class SWF {
	    constructor(url_Json, FLoaded) {
	        this.Dic_NameFrame = {};
	        this.URL_Swf = url_Json;
	        if (url_Json.indexOf("/") != -1) {
	            this.SwfName = url_Json.slice(url_Json.lastIndexOf("/") + 1, url_Json.indexOf(".json"));
	        }
	        this.FunLoaded = FLoaded;
	        this.hanProgress = Laya.Handler.create(this, this.onAtlasProgress, null, false);
	        Laya.loader.load(url_Json, Laya.Handler.create(this, this.onLoaded), this.hanProgress, Laya.Loader.ATLAS);
	    }
	    onLoaded() {
	        this.hanProgress.clear();
	        this.hanProgress = null;
	        var atlas = Laya.Loader.getAtlas(this.URL_Swf);
	        var data = Laya.Loader.getRes(this.URL_Swf);
	        if (data == null) {
	            Config.LogF("加载url_Json报错！");
	            this.onLoadEndF(false);
	            return;
	        }
	        var frames = data.frames;
	        var str;
	        var count = 0;
	        for (let name in frames) {
	            var t = Laya.loader.getRes(atlas[count++]);
	            if (name.indexOf(".") >= 0)
	                str = name.slice(0, name.lastIndexOf("."));
	            this.Dic_NameFrame[str] = t;
	        }
	        this.onLoadBytes(this.URL_Swf);
	    }
	    onAtlasProgress(per) {
	        MainManager.getInstence().MEM.dispatchF("加载进度", per * 0.8);
	    }
	    onLoadBytes(url_Json) {
	        var url_bytes = url_Json.split(".")[0] + SWF.dataFileType;
	        var data = Laya.Loader.getRes(url_bytes);
	        if (data) {
	            this.initData(data);
	        }
	        else {
	            this.l = new Laya.Loader();
	            this.l.once(Laya.Event.COMPLETE, this, this.loadCompleteF);
	            this.l.once(Laya.Event.ERROR, this, this.loadErrorF);
	            this.l.on(Laya.Event.PROGRESS, null, function (per) {
	                if (per != 1) {
	                }
	            });
	            this.l.load(url_bytes, Laya.Loader.BUFFER);
	        }
	    }
	    loadErrorF(err) {
	        Config.LogF("加载Bytes报错！");
	        this.loadCompleteF(null);
	    }
	    loadCompleteF(data) {
	        this.l.offAll();
	        if (data == null && SWF.dataFileType != ".txt") {
	            SWF.dataFileType = ".txt";
	            let url_bytes = this.URL_Swf.split(".")[0] + SWF.dataFileType;
	            this.onLoadBytes(this.URL_Swf);
	            return;
	        }
	        this.initData(data);
	    }
	    initData(data) {
	        try {
	            var inflate = new Zlib.Inflate(new Uint8Array(data));
	            var inbuffer = inflate.decompress();
	            var bytes = new Laya.Byte(inbuffer);
	        }
	        catch (e) {
	            Config.LogF("解压报错！" + e.message);
	            return;
	        }
	        this.readUTFAsync(bytes, (str) => {
	            this._swfDatas = JSON.parse(str);
	            if (this._swfDatas["particle"] != null) {
	                var arrAll = [];
	                for (var key in this._swfDatas["particle"]) {
	                    if (this._swfDatas["particle"][key] != null) {
	                        arrAll.push(key);
	                    }
	                }
	                if (arrAll.length > 0) {
	                    this.onLoadParticalSource(arrAll);
	                    return;
	                }
	            }
	            this.onLoadEndF(true);
	        });
	    }
	    readUTFAsync(byte, callback, start = null) {
	        let i = 0;
	        let str = start ? start : "";
	        let len = 100000;
	        if (byte.bytesAvailable > len) {
	            str += byte.readUTFBytes(len);
	        }
	        else {
	            str += byte.readUTFBytes();
	        }
	        if (byte.bytesAvailable == 0) {
	            Tool_Function.onRunFunction(callback, str);
	            return;
	        }
	        MainManager.getInstence().add_delayFunction(() => {
	            this.readUTFAsync(byte, callback, str);
	        }, 1);
	    }
	    onLoadParticalSource(arr) {
	        var key = arr[0];
	        key = key.substr(9);
	        Laya.loader.load("assets/Particles/" + key + ".part", Laya.Handler.create(this, this.onParticleLoadOne, [arr]), null, Laya.Loader.JSON);
	    }
	    onParticleLoadOne(arr, settings) {
	        var key = arr.shift();
	        if (this.Dic_ParticleSetting == null) {
	            this.Dic_ParticleSetting = {};
	        }
	        this.Dic_ParticleSetting[key] = settings;
	        if (arr.length == 0) {
	            this.onLoadEndF(true);
	        }
	        else {
	            this.onLoadParticalSource(arr);
	        }
	    }
	    onLoadEndF(suc) {
	        Tool_Function.onRunFunction(this.FunLoaded, suc == true ? this : null);
	        this.FunLoaded = null;
	    }
	    createFuns(objData) {
	        var strName;
	        if (objData instanceof Array)
	            strName = objData[0];
	        else if (typeof objData === 'string')
	            strName = objData;
	        else
	            strName = objData["url"];
	        if (strName.indexOf(SWF.dataKey_Image) == 0)
	            return this.getImage(strName);
	        if (strName.indexOf(SWF.dataKey_Sprite) == 0)
	            return this.getSprite(strName);
	        if (strName.indexOf(SWF.dataKey_MovieClip) == 0)
	            return this.getMc(strName);
	        if (strName.indexOf(SWF.dataKey_Scale9) == 0)
	            return this.getS9Image(strName);
	        if (strName.indexOf(SWF.dataKey_TextField) == 0)
	            return this.createTextField(objData);
	        if (strName.indexOf(SWF.dataKey_Particle) == 0)
	            return this.creatParticle(objData);
	        return null;
	    }
	    ;
	    getObject(strName) {
	        if (strName.indexOf("img_") == 0)
	            return this.getImage(strName);
	        if (strName.indexOf("spr_") == 0)
	            return this.getSprite(strName);
	        if (strName.indexOf("mc_") == 0)
	            return this.getMc(strName);
	        if (strName.indexOf("s9_") == 0)
	            return this.getS9Image(strName);
	        return null;
	    }
	    getTexture(strName) {
	        if (this.Dic_NameFrame[strName] != null) {
	            var t = this.Dic_NameFrame[strName];
	            return t;
	        }
	        return null;
	    }
	    getImage(strName) {
	        if (this._swfDatas[SWF.dataKey_Image][strName] == null) {
	            return null;
	        }
	        if (this.Dic_NameFrame[strName] != null) {
	            var t = this.Dic_NameFrame[strName];
	            var image = new SwfImage(null);
	            var imageData = this._swfDatas[SWF.dataKey_Image][strName];
	            image.classLink = strName;
	            image.swfName = this.SwfName;
	            image.texture = t;
	            image.pivotX = imageData[imageData.length - 2];
	            image.pivotY = imageData[imageData.length - 1];
	            if (typeof (imageData[0]) != "number") {
	                image.metaData = imageData[0];
	                if (imageData.length >= 5) {
	                }
	            }
	            else {
	                if (imageData.length >= 4) {
	                }
	            }
	            return image;
	        }
	        return null;
	    }
	    getS9Image(strName) {
	        if (this.Dic_NameFrame[strName] != null) {
	            var scale9Data = this._swfDatas[SWF.dataKey_Scale9][strName];
	            var t = this.Dic_NameFrame[strName];
	            var image = new SwfS9Image();
	            image.classLink = strName;
	            image.swfName = this.SwfName;
	            image.source = t;
	            image.sizeGrid = scale9Data["y"] + "," + (t.width - scale9Data["x"] - scale9Data["w"]) + "," + (t.height - scale9Data["y"] - scale9Data["h"]) + "," + scale9Data["x"];
	            return image;
	        }
	        return null;
	    }
	    getSprite(strName) {
	        if (this._swfDatas[SWF.dataKey_Sprite] == null || this._swfDatas[SWF.dataKey_Sprite][strName] == null) {
	            Config.LogF("没有Sprite:" + strName);
	            return null;
	        }
	        var sprData = this._swfDatas[SWF.dataKey_Sprite][strName];
	        var sprite = new Sprite();
	        var length = sprData.length;
	        var objData;
	        var display;
	        var i = 0;
	        if (sprData.length > 0) {
	            if (!(sprData[0] instanceof Array) && sprData[0]["url"] == null)
	                i = 1;
	            for (; i < length; i++) {
	                objData = sprData[i];
	                display = this.getOneChild(objData);
	                if (display == null)
	                    continue;
	                sprite.addChild(display);
	            }
	        }
	        sprite.setSpriteData(sprData);
	        sprite.classLink = strName;
	        sprite.swfName = this.SwfName;
	        return sprite;
	    }
	    getOneChild(objData) {
	        if (objData == null)
	            return null;
	        var display;
	        var i = 0;
	        display = this.createFuns(objData);
	        if (display == null)
	            return null;
	        display.name = objData["name"];
	        display.pos(objData["x"], objData["y"]);
	        if (objData["type"] != SWF.dataKey_Scale9) {
	            display.scaleX = objData["sx"];
	            display.scaleY = objData["sy"];
	            display.sx0 = objData["sx"];
	            display.sy0 = objData["sy"];
	        }
	        else {
	            display.width = objData["w"];
	            display.height = objData["h"];
	        }
	        display.skewX = -objData["skewx"];
	        display.skewY = objData["skewy"];
	        display.alpha = objData["alpha"];
	        return display;
	    }
	    getMc(strName) {
	        if (this._swfDatas[SWF.dataKey_MovieClip] == null || this._swfDatas[SWF.dataKey_MovieClip][strName] == null) {
	            Config.LogF("没有MC:" + strName);
	            return null;
	        }
	        var movieClipData = this._swfDatas[SWF.dataKey_MovieClip][strName];
	        if (movieClipData == null)
	            return null;
	        var objectCountData = movieClipData["objCount"];
	        var displayObjects = {};
	        var displayObjectArray;
	        var type;
	        var count;
	        for (var objName in objectCountData) {
	            type = objectCountData[objName][0];
	            count = objectCountData[objName][1];
	            displayObjectArray = displayObjects[objName] == null ? [] : displayObjects[objName];
	            if (type == "particle") {
	                for (let i = 0; i < count; i++) {
	                    displayObjectArray.push(this.createFuns(objName));
	                }
	            }
	            else if (type == "text") {
	                for (let i = 0; i < count; i++) {
	                    displayObjectArray.push(null);
	                }
	            }
	            else {
	                for (let i = 0; i < count; i++) {
	                    if (i >= SwfMovieClip.CacheNumBegin)
	                        break;
	                    displayObjectArray.push(this.createFuns(objName));
	                }
	            }
	            displayObjects[objName] = displayObjectArray;
	        }
	        var mc = new SwfMovieClip(movieClipData["frames"], movieClipData["labels"], displayObjects, this, movieClipData["frameEvents"]);
	        if (movieClipData["重复"] != null)
	            mc.onCashSameFrameInfo(movieClipData["重复"]);
	        if (movieClipData["meta"] != null)
	            mc.setMetaData(movieClipData["meta"]);
	        mc.loop = movieClipData["loop"];
	        mc.classLink = strName;
	        mc.swfName = this.SwfName;
	        return mc;
	    }
	    createTextField(data) {
	        var textfield;
	        var txt = "";
	        if (data) {
	            if (data["text"] && data["text"] != "\r" && data["text"] != "") {
	                txt = data["text"];
	            }
	            if (data["font"] != null) {
	                if (SWF.DicFont && SWF.DicFont[data["font"]] != null) {
	                    data["font"] = SWF.DicFont[data["font"]];
	                }
	                else if (SWF.DicFont && (data["font"] == "Arial" || data["font"] == "_sans") && SWF.DicFont["default"]) {
	                    data["font"] = SWF.DicFont["default"];
	                }
	            }
	            else {
	                if (SWF.DicFont && SWF.DicFont["default"])
	                    data["font"] = SWF.DicFont["default"];
	            }
	            textfield = Tool_Textfield.newTextfield(data["w"], data["h"], txt, data["font"], data["size"], data["color"], "上", data["align"], data["italic"], data["bold"], data["multiline"]);
	        }
	        return textfield;
	    }
	    creatParticle(_name) {
	        if (this.Dic_ParticleSetting[_name] == null) {
	            return null;
	        }
	        var sp = new Laya.Particle2D(this.Dic_ParticleSetting[_name]);
	        sp.emitter.start();
	        sp.play();
	        return sp;
	    }
	    destroyF() {
	        Laya.Loader.clearRes(this.URL_Swf);
	    }
	    onCheckDataForFilterAndColor(display, data) {
	        if (data == null)
	            return;
	        SWF.setBlendMode(display, data["blend"]);
	        if (data["color"] != null && (display instanceof Sprite) == true) {
	            SWF.changeColor(display, data["color"]);
	        }
	    }
	    static changeColor(tar, color) {
	        if (tar == null) {
	            return;
	        }
	        if (color instanceof Array) {
	            let red = color[0] / color[1];
	            let green = color[2] / color[3];
	            let blue = color[4] / color[5];
	            var matrix = [
	                1, 0, 0, 0, red,
	                0, 1, 0, 0, green,
	                0, 0, 1, 0, blue,
	                0, 0, 0, 1, -color[1] * 255
	            ];
	        }
	        else {
	            matrix = [
	                Color.getRed(color) / 255, 0.6094, 0.0820, 0, 0,
	                0.3086, Color.getGreen(color) / 255, 0.0820, 0, 0,
	                0.3086, 0.6094, Color.getBlue(color) / 255, 0, 0,
	                0, 0, 0, 1, 0
	            ];
	        }
	        var fil = new Laya.ColorFilter(matrix);
	        if (tar.filters == null)
	            tar.filters = [fil];
	        else {
	            for (let i = 0; i < tar.filters.length; i++) {
	                let lastf = tar.filters[i];
	                if (lastf instanceof Laya.ColorFilter) {
	                    tar.filters.splice(i, 1);
	                }
	            }
	            tar.filters.push(fil);
	        }
	    }
	    static setBlendMode(display, blendMode) {
	        if (blendMode == "add") {
	            display.blendMode = "lighter";
	        }
	        else if (blendMode == "erase") {
	        }
	        else {
	        }
	    }
	}
	SWF.dataKey_Sprite = "spr";
	SWF.dataKey_Image = "img";
	SWF.dataKey_MovieClip = "mc";
	SWF.dataKey_TextField = "text";
	SWF.dataKey_Scale9 = "s9";
	SWF.dataKey_Particle = "particle";
	SWF.dataFileType = ".mp3";

	var Handler$1 = Laya.Handler;
	class SwfMovieClip extends Sprite {
	    constructor(frames, labels, displayObjects, ownerSwf, _events) {
	        super();
	        this._isPlay = false;
	        this._loop = true;
	        this._complete = null;
	        this._hasCompleteListener = false;
	        this.numMC = 0;
	        this.needUpdateEveryFrame = false;
	        this.pastTime = 0;
	        if (SwfMovieClip.TimeFrame <= 0)
	            SwfMovieClip.TimeFrame = 1000 / Config.playSpeedTrue;
	        this.touchable = false;
	        this._frames = frames;
	        this._labels = labels;
	        this._frameEvents = _events;
	        this._displayObjects = displayObjects;
	        this._startFrame = 0;
	        this._endFrame = this._frames.length - 1;
	        this._ownerSwf = ownerSwf;
	        this.fps = SwfMovieClip.FPS_default;
	        this.currentFrame = 0;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf(SWF.dataKey_MovieClip) == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    this.numMC++;
	                }
	            }
	        }
	        this.play();
	    }
	    onCashSameFrameInfo(obj) {
	        for (var f in obj) {
	            this._frames[f] = null;
	        }
	    }
	    _frameLoop() {
	        if (!this._isPlay) {
	            if (this.funEnter)
	                MainManager.getInstence().removeEnterFrameFun(this.funEnter);
	            return;
	        }
	        this.pastTime += SwfMovieClip.TimeFrame;
	        var f2 = Tool_Function.onChangeInstance(this.pastTime / this.NeedTime);
	        while (f2 > 0) {
	            if (this._ownerSwf == null) {
	                return;
	            }
	            this.nextFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    nextFrame() {
	        if (this._ownerSwf == null)
	            return;
	        if (this._currentFrame >= this._endFrame) {
	            var isReturn = false;
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                isReturn = true;
	            }
	            if (this._complete) {
	                Tool_Function.onRunFunction(this._complete, this);
	                this._complete = Tool_ObjUtils.destroyF_One(this._complete);
	            }
	            if (isReturn)
	                return;
	            this._currentFrame = this._startFrame;
	        }
	        else {
	            this._currentFrame++;
	        }
	        this.currentFrame = this._currentFrame;
	        if (this._currentFrame >= this._endFrame) {
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                if (this._complete) {
	                    Tool_Function.onRunFunction(this._complete, this);
	                    this._complete = Tool_ObjUtils.destroyF_One(this._complete);
	                }
	            }
	        }
	    }
	    removeChildFromAllFrame(_classLink) {
	        var length = this._frames.length;
	        for (var i = 0; i < length; i++) {
	            var frameInfos2 = this._frames[i];
	            if (frameInfos2 == null)
	                continue;
	            for (var j = 0; j < frameInfos2.length; j++) {
	                this.data = frameInfos2[j];
	                if (this.data instanceof Array == false) {
	                    if (this.data["url"] == _classLink || this.data["name"] == _classLink) {
	                        frameInfos2.splice(j--, 1);
	                    }
	                }
	            }
	        }
	    }
	    getChildByName_onFrame(_name, f) {
	        if (this._frames[f] == null)
	            return null;
	        var frameInfos2 = this._frames[f];
	        for (var j = 0; j < frameInfos2.length; j++) {
	            this.data = frameInfos2[j];
	            if (this.data instanceof Array == false) {
	                if (this.data["name"] == _name) {
	                    var display;
	                    if (this._displayObjects)
	                        display = this._displayObjects[this.data["url"]][this.data["index"]];
	                    return display;
	                }
	            }
	        }
	        return null;
	    }
	    set currentFrame(frame) {
	        this._currentFrame = frame;
	        this.__frameInfos = this._frames[this._currentFrame];
	        if (this.__frameInfos == null || this._ownerSwf == null)
	            return;
	        this.onSetCurrentFrame();
	    }
	    onSetCurrentFrame() {
	        if (this.lastFrame == this._currentFrame)
	            return;
	        this.lastFrame = this._currentFrame;
	        if (this.frameFunction)
	            Tool_Function.onRunFunction(this.frameFunction);
	        if (this._displayObjects)
	            this.clearChild();
	        else
	            this.clearChild(true);
	        var data;
	        var display;
	        var useIndex;
	        var length = this.__frameInfos.length;
	        for (var i = 0; i < length; i++) {
	            data = this.__frameInfos[i];
	            useIndex = data["index"];
	            if (data["type"] == SWF.dataKey_Particle) {
	                var pd = this._displayObjects[data["url"]][useIndex];
	                pd.x = data.x;
	                pd.y = data.y;
	                pd.alpha = data["alpha"];
	                pd.name = data["name"];
	                pd.emitter.emit();
	                this.addChild(pd);
	                continue;
	            }
	            if (this._displayObjects) {
	                display = this._displayObjects[data["url"]][useIndex];
	            }
	            if (display == null) {
	                if (data["type"] == "text")
	                    display = this._ownerSwf.createTextField(data);
	                else
	                    display = this._ownerSwf.getObject(data["url"]);
	                if (display && this._displayObjects) {
	                    this._displayObjects[data["url"]][useIndex] = display;
	                }
	            }
	            if (display == null || (this.dicRemoved != null && this.dicRemoved[display] == true))
	                continue;
	            this._ownerSwf.onCheckDataForFilterAndColor(display, data);
	            display.alpha = data["alpha"];
	            display.name = data["name"];
	            display.x = data["x"];
	            display.y = data["y"];
	            display.skewX = -data["skewx"];
	            display.skewY = data["skewy"];
	            if (this.touchable == true) {
	                display.touchable = true;
	                if (display.mouseEnabled == false) {
	                    display.mouseEnabled = true;
	                }
	            }
	            switch (data["type"]) {
	                case SWF.dataKey_Scale9:
	                    display.width = data["w"];
	                    display.height = data["h"];
	                    SWF.setBlendMode(display, data["blend"]);
	                    if (data["color"] != null) {
	                        SWF.changeColor(display, data["color"]);
	                    }
	                    this.addChild(display);
	                    break;
	                case SWF.dataKey_TextField:
	                    var tx = display;
	                    if (tx.isCleared == true)
	                        break;
	                    tx.width = data["w"];
	                    tx.height = data["h"];
	                    tx.format.font = data["font"];
	                    tx.format.color = data["color"];
	                    tx.format.size = data["size"];
	                    tx.format.horizontalAlign = data["align"];
	                    tx.format.italic = data["italic"];
	                    tx.format.bold = data["bold"];
	                    if (tx.text != data["text"] && tx.text != "\r" && tx.text != "") { }
	                    else if (data["text"] && data["text"] != "\r" && data["text"] != "") {
	                        if (tx.changed == false) {
	                            tx.text = data["text"];
	                            tx.text = data["text"];
	                        }
	                    }
	                    SWF.setBlendMode(display, data["blend"]);
	                    this.addChild(display);
	                    break;
	                case SWF.dataKey_Image:
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    SWF.setBlendMode(display, data["blend"]);
	                    if (data["color"] != null) {
	                        SWF.changeColor(display, data["color"]);
	                    }
	                    this.addChild(display);
	                    break;
	                default:
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    SWF.setBlendMode(display, data["blend"]);
	                    if (data["color"] != null) {
	                        SWF.changeColor(display, data["color"]);
	                    }
	                    this.addChild(display);
	                    break;
	            }
	        }
	        if (this._frameEvents != null && this._frameEvents[this._currentFrame] != null) {
	            Tool_Function.onRunFunction(this.eventFunction, this._frameEvents[this._currentFrame]);
	        }
	    }
	    get currentFrame() {
	        return this._currentFrame;
	    }
	    play(rePlayChildMovie = true) {
	        if (this._isPlay == true)
	            return;
	        this._isPlay = true;
	        if (this._currentFrame >= this._endFrame) {
	            this._currentFrame = this._startFrame;
	        }
	        if (this.funEnter == null || this.funEnter.method == null)
	            this.funEnter = Handler$1.create(this, this._frameLoop, null, false);
	        MainManager.getInstence().addEnterFrameFun(this.funEnter);
	        if (this.numMC == 0 || this._displayObjects == null)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf(SWF.dataKey_MovieClip) == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip) == false)
	                        continue;
	                    if (rePlayChildMovie)
	                        arr[i].currentFrame = 0;
	                    arr[i].play(rePlayChildMovie);
	                }
	            }
	        }
	        this.needUpdateEveryFrame = rePlayChildMovie;
	    }
	    stop(stopChild = false) {
	        this._isPlay = false;
	        this.pastTime = 0;
	        if (this.funEnter)
	            MainManager.getInstence().removeEnterFrameFun(this.funEnter);
	        if (this.numMC == 0)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf(SWF.dataKey_MovieClip) == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip) == false)
	                        continue;
	                    if (stopChild == true)
	                        arr[i].stop(stopChild);
	                    else
	                        arr[i].play();
	                }
	            }
	        }
	        this.needUpdateEveryFrame = !stopChild;
	    }
	    gotoAndStop(frame, stopChild = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.stop(stopChild);
	    }
	    gotoAndPlay(frame, rePlayChildMovie = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.play(rePlayChildMovie);
	    }
	    onCheckLastNotNullFrame() {
	        var count = 0;
	        while (this.__frameInfos == null) {
	            if (this._currentFrame - count < 0) {
	                this.__frameInfos = this._frames[0];
	                break;
	            }
	            this.__frameInfos = this._frames[this._currentFrame - count];
	            count++;
	        }
	        this.onSetCurrentFrame();
	    }
	    goTo(frame) {
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                frame = 0;
	                this._currentFrame = this._startFrame = frame;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._currentFrame = this._startFrame = labelData[1];
	            }
	        }
	        else {
	            if (frame < 0)
	                frame = 0;
	            this._currentFrame = this._startFrame = frame;
	        }
	        this.currentFrame = this._currentFrame;
	    }
	    getLabelData(label) {
	        var length = this._labels.length;
	        var labelData;
	        for (var i = 0; i < length; i++) {
	            labelData = this._labels[i];
	            if (labelData[0] == label) {
	                return labelData;
	            }
	        }
	        return null;
	    }
	    get isPlay() {
	        return this._isPlay;
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(value) {
	        this._loop = value;
	    }
	    set completeFunction(value) {
	        this._complete = value;
	    }
	    get completeFunction() {
	        return this._complete;
	    }
	    get totalFrames() {
	        return this._frames.length;
	    }
	    get currentLabel() {
	        return this._currentLabel;
	    }
	    get labels() {
	        if (this._labelStrings != null)
	            return this._labelStrings;
	        this._labelStrings = [];
	        var length = this._labels.length;
	        for (var i = 0; i < length; i++) {
	            this._labelStrings.push(this._labels[i][0]);
	        }
	        return this._labelStrings;
	    }
	    hasLabel(label) {
	        return !(this.labels.indexOf(label) == -1);
	    }
	    set startFrame(value) {
	        this._startFrame = value < 0 ? 0 : value;
	        this._startFrame = this._startFrame > this._endFrame ? this._endFrame : this._startFrame;
	    }
	    get startFrame() {
	        return this._startFrame;
	    }
	    set endFrame(value) {
	        this._endFrame = value > this._frames.length - 1 ? this._frames.length - 1 : value;
	        this._endFrame = this._endFrame < this._startFrame ? this._startFrame : this._endFrame;
	    }
	    get endFrame() {
	        return this._endFrame;
	    }
	    removeChildF(child, dispose = true) {
	        if (this.dicRemoved == null)
	            this.dicRemoved = {};
	        if (child instanceof String) {
	            child = this.getChildByName(child);
	        }
	        if (child != null) {
	            this.dicRemoved[child] = true;
	            this.removeChild(child);
	        }
	    }
	    destroyF() {
	        this.dicRemoved = null;
	        this.frameFunction = Tool_ObjUtils.destroyF_One(this.frameFunction);
	        this.eventFunction = Tool_ObjUtils.destroyF_One(this.eventFunction);
	        this.completeFunction = Tool_ObjUtils.destroyF_One(this.completeFunction);
	        if (this._displayObjects) {
	            for (var key in this._displayObjects) {
	                let array = this._displayObjects[key];
	                var len = array.length;
	                for (let i = 0; i < len; i++) {
	                    if (array[i]) {
	                        array[i].removeFromParent(true);
	                    }
	                }
	            }
	            this._displayObjects = null;
	        }
	        this._ownerSwf = null;
	        if (this._isPlay) {
	            this._isPlay = false;
	            if (this.funEnter)
	                MainManager.getInstence().removeEnterFrameFun(this.funEnter);
	        }
	    }
	    get fps() { return this._fps; }
	    set fps(value) {
	        this._fps = value;
	        this.NeedTime = 1000 / this._fps;
	    }
	}
	SwfMovieClip.ANGLE_TO_RADIAN = Math.PI / 180;
	SwfMovieClip.FPS_default = 30;
	SwfMovieClip.TimeFrame = 0;
	SwfMovieClip.CacheNumBegin = 9999999;

	var Event = Laya.Event;
	class MyZMovieClip extends Sprite {
	    constructor(url, f) {
	        super();
	        this._loop = true;
	        this.isComplete = false;
	        this.countMoreFrame = 0;
	        MyMC.addF(this);
	        this.URL = url;
	        this.Fun = f;
	        var index = this.URL.lastIndexOf("/");
	        this.Name = this.URL.substr(index + 1);
	        if (MyZMovieClip.dicTemplet == null) {
	            MyZMovieClip.dicTemplet = {};
	        }
	        if (MyZMovieClip.dicTemplet[this.URL] == null) {
	            this.zGameMc = new Laya.Templet();
	            MyZMovieClip.dicTemplet[this.URL] = { "模板": this.zGameMc, "num": 1 };
	            this.zGameMc.on(Event.COMPLETE, this, this.parseComplete);
	            this.zGameMc.on(Event.ERROR, this, this.onError);
	            this.zGameMc.loadAni(url);
	        }
	        else {
	            this.zGameMc = MyZMovieClip.dicTemplet[this.URL]["模板"];
	            MyZMovieClip.dicTemplet[this.URL]["num"]++;
	            Tool_Function.onRunFunction(this.Fun, this);
	            this.Fun = null;
	        }
	    }
	    parseComplete(fac) {
	        this.zGameMc.offAll();
	        if (fac == null) {
	            Tool_Function.onRunFunction(this.Fun, null);
	        }
	        else {
	            Tool_Function.onRunFunction(this.Fun, this);
	        }
	        this.Fun = null;
	    }
	    onError(e) {
	        console.log("加载spine失败：" + URL);
	        this.parseComplete(null);
	    }
	    initF() {
	        this.mArmature = this.zGameMc.buildArmature(0);
	        this.mArmature.offAll();
	        this.addChild(this.mArmature);
	        this.mArmature.playbackRate(30 / Config.playSpeedTrue);
	        this.mArmature.on(Event.STOPPED, this, this.completeHandler);
	        this.mArmature.on(Event.LABEL, this, this.onEvent);
	    }
	    onEvent(e) {
	        var tEventData = e;
	        if ((tEventData.stringValue != null && tEventData.stringValue.length > 0) || (tEventData.intValue > 0 || tEventData.intValue < 0)) {
	            let dic = { "name": tEventData.name, "str": tEventData.stringValue, "int": tEventData.intValue };
	            if (this.FunEvent)
	                Tool_Function.onRunFunction(this.FunEvent, dic);
	        }
	        else {
	            if (this.FunEvent)
	                Tool_Function.onRunFunction(this.FunEvent, tEventData.name);
	        }
	    }
	    completeHandler() {
	        console.info("spine complete");
	        this.isComplete = true;
	        Tool_Function.onRunFunction(this.completeFunction);
	    }
	    play(rePlayChildMovie = true) {
	        if (this.nowLabel == null) {
	            console.error(this.Name + "：spine：play失败，不应该没有label直接播放");
	            return;
	        }
	        this.nowFrame = -1;
	        this.mArmature.resume();
	    }
	    gotoAndPlay(frame, rePlayChildMovie = false) {
	        if (typeof frame == "string") {
	            this.gotoAndPlayLable(frame);
	        }
	        else {
	            if (this.nowLabel == null) {
	                console.error(this.Name + "：spine：play失败，不应该没有label直接播放");
	                return;
	            }
	            try {
	                this.mArmature.index = frame;
	            }
	            catch (e) {
	                console.log(this.Name + "：spine：gotoAndPlay失败：" + e.message);
	                return;
	            }
	            this.play();
	        }
	    }
	    gotoAndPlayLable(label) {
	        if (typeof label == "number") {
	            label = this.mArmature.getAniNameByIndex(label);
	        }
	        var allLabel = this.mArmature.getAnimNum();
	        for (var i = 0; i < allLabel; i++) {
	            if (label == this.mArmature.getAniNameByIndex(i)) {
	                this.nowLabel = label;
	                this.nowFrame = -1;
	                this.mArmature.play(label, this.loop, true);
	                return;
	            }
	        }
	        console.log("spine动画播放label：" + label + "：失败");
	    }
	    gotoAndStop(frame, stopChild = false) {
	        if (this.zGameMc) {
	            this.countMoreFrame = 0;
	            this.mArmature.index = frame;
	            this.nowFrame = frame;
	            while (this.mArmature.index < frame) {
	                this.countMoreFrame++;
	                if (frame + this.countMoreFrame < this.mArmature.total) {
	                    this.mArmature.index = frame + this.countMoreFrame;
	                    if (this.mArmature.index > this.nowFrame) {
	                        this.nowFrame = this.mArmature.index;
	                    }
	                }
	                else {
	                    break;
	                }
	            }
	        }
	    }
	    gotoAndStopLable(label) {
	        this.isComplete = false;
	        this.nowLabel = label;
	        var allLabel = this.mArmature.getAnimNum();
	        for (var i = 0; i < allLabel; i++) {
	            if (label == this.mArmature.getAniNameByIndex(i)) {
	                this.mArmature.play(i, this.loop);
	                this.mArmature.index = 0;
	                this.nowFrame = -1;
	                return;
	            }
	        }
	        console.log("spine动画播放label：" + label + "：失败");
	    }
	    stop(stopChild = false) {
	        if (this.zGameMc) {
	            this.mArmature.stop();
	        }
	    }
	    pauseF() {
	        this.stop();
	    }
	    resumeF() {
	        this.mArmature.resume();
	    }
	    get totalFrames() {
	        if (this.mArmature && this.mArmature.player.currentAnimationClipIndex != -1) {
	            return this.mArmature.total;
	        }
	        return 0;
	    }
	    get currentFrame() {
	        if (this.zGameMc) {
	            if (this.mArmature.index < this.nowFrame && this.nowFrame >= 0) {
	                return this.nowFrame;
	            }
	            return this.mArmature.index;
	        }
	        return 0;
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(real) {
	        this._loop = real;
	    }
	    findBone(bName) {
	        if (this.mArmature == null)
	            return null;
	        return this.zGameMc.boneSlotArray;
	    }
	    destroyF() {
	        MyMC.removeF(this);
	        this.completeFunction = Tool_ObjUtils.destroyF_One(this.completeFunction);
	        if (this.mArmature) {
	            this.mArmature.destroy();
	            this.mArmature = null;
	        }
	        if (this.zGameMc) {
	            if (MyZMovieClip.dicTemplet[this.URL]) {
	                if (--MyZMovieClip.dicTemplet[this.URL]["num"] <= 0) {
	                    this.zGameMc.destroy();
	                    MyZMovieClip.dicTemplet[this.URL] = null;
	                }
	            }
	            this.zGameMc = null;
	        }
	        super.destroyF();
	    }
	}

	class MyMouseManager {
	    constructor() {
	        this.Listeners = [];
	        this.pointIndex = {};
	        Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.mouseHandler);
	        Laya.stage.on(Laya.Event.MOUSE_UP, this, this.mouseHandler);
	        Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.mouseHandler);
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new MyMouseManager();
	        return this.instance;
	    }
	    static destroyF() {
	        if (this.instance) {
	            this.instance.destroyF();
	            this.instance = null;
	        }
	    }
	    destroyF() {
	        Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.mouseHandler);
	        Laya.stage.off(Laya.Event.MOUSE_UP, this, this.mouseHandler);
	        Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.mouseHandler);
	    }
	    addListener(one, index = -1) {
	        Tool_ArrayUtils.insertInto(this.Listeners, index, one);
	    }
	    removeListener(one) {
	        let index = this.Listeners.indexOf(one);
	        if (index >= 0) {
	            this.Listeners.splice(index, 1);
	        }
	        one.destroyF();
	    }
	    mouseHandler(e) {
	        var touches = e.touches;
	        if (touches && touches.length > 0) {
	            for (let i = 0; i < touches.length; i++) {
	                let t = touches[i];
	                let nameIndex = e.touchId + "_" + i;
	                if (this.pointIndex[nameIndex] == null)
	                    this.pointIndex[nameIndex] = {};
	                if (t == null) {
	                    this.upF(nameIndex, this.pointIndex[nameIndex]);
	                    continue;
	                }
	                this.pointIndex[nameIndex]["x"] = t.stageX;
	                this.pointIndex[nameIndex]["y"] = t.stageY;
	                switch (e.type) {
	                    case Laya.Event.MOUSE_DOWN:
	                        this.downF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                    case Laya.Event.MOUSE_UP:
	                        this.upF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                    case Laya.Event.MOUSE_MOVE:
	                        this.moveF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                }
	            }
	        }
	        else {
	            let i = "999";
	            let nameIndex = "_" + i;
	            if (this.pointIndex[nameIndex] == null)
	                this.pointIndex[nameIndex] = {};
	            this.pointIndex[nameIndex]["x"] = Laya.stage.mouseX;
	            this.pointIndex[nameIndex]["y"] = Laya.stage.mouseY;
	            switch (e.type) {
	                case Laya.Event.MOUSE_DOWN:
	                    this.downF(i, this.pointIndex[nameIndex]);
	                    break;
	                case Laya.Event.MOUSE_UP:
	                    this.upF(i, this.pointIndex[nameIndex]);
	                    break;
	                case Laya.Event.MOUSE_MOVE:
	                    this.moveF(i, this.pointIndex[nameIndex]);
	                    break;
	            }
	        }
	    }
	    downF(index, p) {
	        if (this.onlyMMO) {
	            if (this.onlyMMO.isDown == false) {
	                if (this.onlyMMO.onDownF(index, p["x"], p["y"]) == false) {
	                    if (this.onlyMMO.FunNone) {
	                        Tool_Function.onRunFunction(this.onlyMMO.FunNone);
	                    }
	                }
	            }
	            return;
	        }
	        for (let i = this.Listeners.length - 1; i >= 0; i--) {
	            let one = this.Listeners[i];
	            if (one == null) {
	            }
	            else if (one.isDown == false) {
	                if (one.onDownF(index, p["x"], p["y"]) == true && one.canEventMop == false)
	                    break;
	            }
	        }
	    }
	    upF(index, p) {
	        for (let i = this.Listeners.length - 1; i >= 0; i--) {
	            let one = this.Listeners[i];
	            if (one && one.isDown == true && (one.nowTouchIndex == index || index == "999")) {
	                one.onUpF(p["x"], p["y"]);
	            }
	        }
	    }
	    moveF(index, p) {
	        for (let i = this.Listeners.length - 1; i >= 0; i--) {
	            let one = this.Listeners[i];
	            if (one && one.isDown == true && one.nowTouchIndex == index) {
	                let re = one.onMoveF(p["x"], p["y"]);
	                if (re != true)
	                    break;
	            }
	        }
	    }
	}

	class MyMouseManagerOne {
	    constructor(tar, index = -1) {
	        this.isDestroyed = false;
	        this.nowTouchIndex = null;
	        this.isDown = false;
	        this.startXY = {};
	        this.globalXY = {};
	        this.canMopThisTime = false;
	        this.isAct = false;
	        this.moveNeedDownBefore = true;
	        this.Tar = tar;
	        MyMouseManager.getInstance().addListener(this, index);
	    }
	    get Index() { return MyMouseManager.getInstance().Listeners.indexOf(this); }
	    ;
	    get canEventMop() { if (this.canMopThisTime == true) {
	        this.canMopThisTime = false;
	        return true;
	    } return false; }
	    ;
	    checkInArea(gx, gy) {
	        if (this.Tar.parent == null || this.Tar.visible == false) {
	            return false;
	        }
	        if (this.p1 == null)
	            this.p1 = new MyMouseEvent();
	        this.p1.x = gx;
	        this.p1.y = gy;
	        this.p1.canMop = false;
	        this.Tar.parent.globalToLocal(this.p1.p, false);
	        let rec = this.Tar.getBounds();
	        if (this.p1.x >= rec.x && this.p1.x <= rec.right && this.p1.y >= rec.y && this.p1.y <= rec.bottom) {
	            this.p1.x = (this.p1.x - this.Tar.x) / this.Tar.scaleX;
	            this.p1.y = (this.p1.y - this.Tar.y) / this.Tar.scaleY;
	            return true;
	        }
	        return false;
	    }
	    onDownF(index, gx, gy) {
	        if (this.Tar.stage == null || this.Tar.parent == null) {
	            return false;
	        }
	        if (this.checkInArea(gx, gy) == false)
	            return false;
	        this.nowTouchIndex = index;
	        this.isDown = true;
	        this.startXY["x"] = gx;
	        this.startXY["y"] = gy;
	        this.globalXY["x"] = gx;
	        this.globalXY["y"] = gy;
	        this.isAct = false;
	        Tool_Function.onRunFunction(this.FunDown, this.p1);
	        if (this.isAct) {
	            this.isAct = false;
	            return true;
	        }
	        return this.p1.canMop;
	    }
	    onUpF(gx, gy) {
	        if (this.Tar.stage == null || this.Tar.parent == null) {
	            return;
	        }
	        this.nowTouchIndex = null;
	        this.globalXY["x"] = gx;
	        this.globalXY["y"] = gy;
	        if (this.checkInArea(gx, gy) == false) {
	            this.isDown = false;
	            Tool_Function.onRunFunction(this.FunUp, null);
	            return;
	        }
	        if (this.isDown) {
	            Tool_Function.onRunFunction(this.FunClick, this.p1);
	        }
	        this.isDown = false;
	        Tool_Function.onRunFunction(this.FunUp, this.p1);
	    }
	    onMoveF(gx, gy) {
	        if (this.Tar.stage == null || this.Tar.parent == null) {
	            return;
	        }
	        if (this.isDown == false && this.moveNeedDownBefore == false)
	            return;
	        this.globalXY["x"] = gx;
	        this.globalXY["y"] = gy;
	        this.checkInArea(gx, gy) == false;
	        Tool_Function.onRunFunction(this.FunMove, this.p1);
	        return this.p1.canMop;
	    }
	    destroyF() {
	        if (this.isDestroyed == true) {
	            return;
	        }
	        this.isDestroyed = true;
	        MyMouseManager.getInstance().removeListener(this);
	        if (MyMouseManager.getInstance().onlyMMO == this)
	            MyMouseManager.getInstance().onlyMMO = null;
	        this.Tar = null;
	        MyMouseManager.getInstance().removeListener(this);
	        this.FunDown = Tool_ObjUtils.destroyF_One(this.FunDown);
	        this.FunClick = Tool_ObjUtils.destroyF_One(this.FunClick);
	        this.FunUp = Tool_ObjUtils.destroyF_One(this.FunUp);
	        this.FunMove = Tool_ObjUtils.destroyF_One(this.FunMove);
	        this.FunNone = Tool_ObjUtils.destroyF_One(this.FunNone);
	    }
	}
	class MyMouseEvent {
	    constructor() {
	        this.p = new Laya.Point();
	        this.canMop = false;
	    }
	    get x() { return this.p.x; }
	    set x(v) { this.p.x = v; }
	    get y() { return this.p.y; }
	    set y(v) { this.p.y = v; }
	}

	var Handler$2 = Laya.Handler;
	class MyMouseEventStarling {
	    constructor(tar, index = -1) {
	        this.TouchID = 0;
	        this.startXY = {};
	        this.nowLocalXY = {};
	        this.worldXY = {};
	        this.isDown = false;
	        this.pausing = false;
	        this.stopEventMop = null;
	        this.LStartMove = 10;
	        this.MC = tar;
	        this.mm = new MyMouseManagerOne(this.MC, index);
	        this.mm.FunDown = this.onMMDown.bind(this);
	        this.mm.FunUp = this.onMMUp.bind(this);
	        this.mm.FunMove = this.onMMMove.bind(this);
	    }
	    get Index() { return this.mm.Index; }
	    ;
	    addLongTouchnHandler(val) {
	        this.Dic_LongTouch = { "fun": val, "need": Config.playSpeedTrue / 2 };
	    }
	    addDownHandler(val) {
	        this.setValue("down事件", val);
	    }
	    addClickHandler(val) {
	        this.setValue("点击", val);
	    }
	    addUpHandler(val) {
	        this.setValue("up事件", val);
	    }
	    addRollHandler(val) {
	        this.setValue("滚轮", val);
	    }
	    addSlideHandler(val) {
	        this.setValue("滑动", val);
	    }
	    addStageClickHandler(fun) {
	        if (fun == null) {
	            MyMouseManager.getInstance().onlyMMO = null;
	            this.mm.FunNone = Tool_ObjUtils.destroyF_One(this.mm.FunNone);
	            return;
	        }
	        MyMouseManager.getInstance().onlyMMO = this.mm;
	        this.mm.FunNone = fun;
	    }
	    setValue(str, val = null) {
	        switch (str) {
	            case "点击":
	                this.newDicClick();
	                this.Dic_Click["down"] = false;
	                if (val instanceof Handler$2)
	                    val.once = false;
	                this.Dic_Click["fun_click"] = val;
	                break;
	            case "down":
	            case "down事件":
	                this.newDicClick();
	                if (val instanceof Handler$2)
	                    val.once = false;
	                this.Dic_Click["fun_down"] = val;
	                break;
	            case "up":
	            case "up事件":
	                this.newDicClick();
	                if (val instanceof Handler$2)
	                    val.once = false;
	                this.Dic_Click["fun_up"] = val;
	                break;
	            case "滑动":
	                this.Dic_Slide = {};
	                this.Dic_Slide["初始x"] = this.MC.x;
	                this.Dic_Slide["初始y"] = this.MC.y;
	                this.Dic_Slide["上次x"] = this.MC.x;
	                this.Dic_Slide["上次y"] = this.MC.y;
	                this.Dic_Slide["开始移动"] = false;
	                this.Dic_Slide["开始移动距离"] = this.LStartMove;
	                this.Dic_Slide["满足滑动速度"] = 1;
	                if (val instanceof Handler$2)
	                    val.once = false;
	                this.Dic_Slide["fun"] = val;
	                break;
	            case "停止本次":
	                this.upF(null);
	                break;
	            case "暂停":
	                this.pausing = val;
	                break;
	            case "停止冒泡":
	                if (Tool_Function.isTypeOf(val, "str"))
	                    this.stopEventMop = val;
	                else if (val == true)
	                    this.stopEventMop = "all";
	                else
	                    this.stopEventMop = null;
	                break;
	            case "滚轮":
	                if (val == null) {
	                    if (this.Dic_roll) {
	                        Config.mStage.off(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	                        this.Dic_roll = Tool_ObjUtils.destroyF_One(this.Dic_roll);
	                    }
	                }
	                else {
	                    if (this.Dic_roll == null) {
	                        this.Dic_roll = {};
	                        Config.mStage.on(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	                    }
	                    else {
	                        this.Dic_roll["fun"] = Tool_ObjUtils.destroyF_One(this.Dic_roll["fun"]);
	                    }
	                    this.Dic_roll["fun"] = val;
	                }
	                break;
	            default:
	                this[str] = val;
	                break;
	        }
	    }
	    onMouseWheel(e) {
	        if (this.Dic_roll && this.Dic_roll["fun"]) {
	            var delta = e.delta;
	            if (delta > 0) {
	            }
	            Tool_Function.onRunFunction(this.Dic_roll["fun"], delta);
	        }
	        else {
	            Config.mStage.off(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	            this.Dic_roll = Tool_ObjUtils.destroyF_One(this.Dic_roll);
	        }
	    }
	    newDicClick() {
	        if (this.Dic_Click)
	            return;
	        this.Dic_Click = Tool_ObjUtils.getNewObjectFromPool();
	        this.Dic_Click["关闭点击距离"] = 20;
	    }
	    mouseHandler(e = null) {
	        if (this.MC == null)
	            return;
	        if (this.stopEventMop == "all") {
	            e.stopPropagation();
	        }
	        var p = {};
	        p["globalX"] = Laya.stage.mouseX;
	        p["globalY"] = Laya.stage.mouseY;
	        this.worldXY.x = p["globalX"];
	        this.worldXY.y = p["globalY"];
	        var pl = this.MC.globalToLocal(new Laya.Point(p["globalX"], p["globalY"]));
	        p["x"] = pl.x;
	        p["y"] = pl.y;
	        this.nowLocalXY.x = p.x;
	        this.nowLocalXY.y = p.y;
	        let result;
	        switch (e.type) {
	            case Laya.Event.MOUSE_DOWN:
	                result = this.downF(p);
	                break;
	            case Laya.Event.MOUSE_UP:
	                result = this.upF(p);
	                break;
	            case Laya.Event.MOUSE_MOVE:
	                result = this.moveF(p);
	                break;
	            case Laya.Event.MOUSE_OUT:
	                result = this.upF(p);
	                break;
	        }
	        if (result == true && this.stopEventMop != null && this.stopEventMop != "all") {
	            e.stopPropagation();
	        }
	        function isInsideF(spr, tar) {
	            for (let i = 0; i < spr.numChildren; i++) {
	                let onechild = spr.getChildAt(i);
	                if (onechild == tar)
	                    return true;
	                if (onechild instanceof Sprite) {
	                    if (isInsideF(onechild, tar) == true)
	                        return true;
	                }
	            }
	            return false;
	        }
	    }
	    onMMDown(p) {
	        this.nowLocalXY.x = p.x;
	        this.nowLocalXY.y = p.y;
	        this.worldXY.x = this.mm.globalXY["x"];
	        this.worldXY.y = this.mm.globalXY["y"];
	        this.downF(p);
	    }
	    downF(p) {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == true)
	            return;
	        this.isDown = true;
	        if (this.Dic_Click != null) {
	            this.Dic_Click["down"] = true;
	            if (this.mm) {
	                this.Dic_Click["起点x"] = p.x;
	                this.Dic_Click["起点y"] = p.y;
	            }
	            else {
	                this.Dic_Click["起点x"] = p.x;
	                this.Dic_Click["起点y"] = p.y;
	            }
	            if (this.Dic_Click["fun_down"] != null) {
	                let re = Tool_Function.onRunFunction(this.Dic_Click["fun_down"], p);
	                if (re == true) {
	                    p.canMop = true;
	                }
	            }
	        }
	        if (this.Dic_Slide != null) {
	            this.Dic_Slide["起点x"] = this.mm.globalXY["x"];
	            this.Dic_Slide["起点y"] = this.mm.globalXY["y"];
	            this.Dic_Slide["开始移动"] = false;
	            this.Dic_Slide["x速度"] = 0;
	            this.Dic_Slide["y速度"] = 0;
	            this.Dic_Slide["计时"] = 0;
	            if (this.FunEnter == null)
	                this.FunEnter = Handler$2.create(this, this.enterFrameF, null, false);
	            MainManager.getInstence().addEnterFrameFun(this.FunEnter);
	        }
	        if (this.Dic_LongTouch) {
	            this.Dic_LongTouch["start"] = MainManager.getInstence().Time;
	            if (this.FunEnterforLongTouch == null)
	                this.FunEnterforLongTouch = Handler$2.create(this, this.enterFrameF2, null, false);
	            MainManager.getInstence().addEnterFrameFun(this.FunEnterforLongTouch);
	        }
	    }
	    onMMUp(p) {
	        this.upF(p);
	    }
	    upF(p) {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false)
	            return;
	        if (this.FunEnterforLongTouch)
	            this.clearLongTouch();
	        if (p != null && this.Dic_Click != null) {
	            if ((this.Dic_Slide == null || this.Dic_Slide["开始移动"] == false) && this.Dic_Click["down"] == true && this.Dic_Click["开始移动"] != true) {
	                if (this.Dic_Click && this.Dic_Click["fun_click"] != null) {
	                    Tool_Function.onRunFunction(this.Dic_Click["fun_click"], p);
	                }
	            }
	            if (this.Dic_Click) {
	                this.Dic_Click["down"] = false;
	                this.Dic_Click["开始移动"] = false;
	                if (this.Dic_Click["fun_up"] != null) {
	                    Tool_Function.onRunFunction(this.Dic_Click["fun_up"], p);
	                }
	            }
	        }
	        if (this.Dic_Slide != null && this.Dic_Slide["开始移动"] == true && p != null) {
	            var lx = p.x;
	            var ly = p.y;
	            var dic = {};
	            dic["类型"] = "滑动";
	            if (Math.abs(this.Dic_Slide["x速度"]) >= this.Dic_Slide["满足滑动速度"])
	                dic["x"] = this.Dic_Slide["x速度"];
	            else
	                dic["x"] = 0;
	            if (Math.abs(this.Dic_Slide["y速度"]) >= this.Dic_Slide["满足滑动速度"])
	                dic["y"] = this.Dic_Slide["y速度"];
	            else
	                dic["y"] = 0;
	            dic["gx"] = this.mm.globalXY["x"];
	            dic["gy"] = this.mm.globalXY["y"];
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], dic);
	            if (this.FunEnter) {
	                MainManager.getInstence().removeEnterFrameFun(this.FunEnter);
	                this.FunEnter.clear();
	                this.FunEnter = null;
	            }
	        }
	        else if (this.Dic_Slide != null) {
	            dic = {};
	            dic["类型"] = "滑动";
	            dic["x"] = 0;
	            dic["y"] = 0;
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], dic);
	            if (this.FunEnter) {
	                MainManager.getInstence().removeEnterFrameFun(this.FunEnter);
	                this.FunEnter.clear();
	                this.FunEnter = null;
	            }
	        }
	        this.isDown = false;
	        return true;
	    }
	    onMMMove(p) {
	        this.moveF(p);
	        p.canMop = true;
	    }
	    moveF(p) {
	        if (this.pausing == true || p == null)
	            return;
	        if (this.isDown == false)
	            return;
	        var gx = this.mm.globalXY["x"];
	        var gy = this.mm.globalXY["y"];
	        this.worldXY.x = this.mm.globalXY["x"];
	        this.worldXY.y = this.mm.globalXY["y"];
	        this.nowLocalXY.x = p.x;
	        this.nowLocalXY.y = p.y;
	        if (this.Dic_Slide == null) {
	            if (this.Dic_Click && this.Dic_Click["开始移动"] != true) {
	                var L = (gx - this.Dic_Click["起点x"]) * (gx - this.Dic_Click["起点x"]) + (gy - this.Dic_Click["起点y"]) * (gy - this.Dic_Click["起点y"]);
	                L = Math.sqrt(L);
	                if (L > this.LStartMove)
	                    this.Dic_Click["开始移动"] = true;
	            }
	            return;
	        }
	        this.Dic_Slide["计时"] = 3;
	        if (this.dicSlide == null)
	            this.dicSlide = {};
	        if (this.Dic_Slide["开始移动"] == false) {
	            L = (gx - this.Dic_Slide["起点x"]) * (gx - this.Dic_Slide["起点x"]) + (gy - this.Dic_Slide["起点y"]) * (gy - this.Dic_Slide["起点y"]);
	            L = Math.sqrt(L);
	            if (L > this.Dic_Slide["开始移动距离"]) {
	                this.Dic_Slide["开始移动"] = true;
	                this.Dic_Slide["上次x"] = gx;
	                this.Dic_Slide["上次y"] = gy;
	                if (this.FunEnterforLongTouch) {
	                    this.clearLongTouch();
	                }
	            }
	        }
	        if (this.Dic_Slide["开始移动"] == true) {
	            if (this.Dic_Click != null && this.Dic_Click["down"] == true) {
	                L = (gx - this.Dic_Slide["起点x"]) * (gx - this.Dic_Slide["起点x"]) + (gy - this.Dic_Slide["起点y"]) * (gy - this.Dic_Slide["起点y"]);
	                L = Math.sqrt(L);
	                if (L > this.Dic_Click["关闭点击距离"]) {
	                    this.Dic_Click["down"] = false;
	                }
	            }
	            this.dicSlide["类型"] = "移动";
	            if (gx != 0)
	                this.dicSlide["x"] = gx - this.Dic_Slide["上次x"];
	            else
	                this.dicSlide["x"] = 0;
	            if (gy != 0)
	                this.dicSlide["y"] = gy - this.Dic_Slide["上次y"];
	            else
	                this.dicSlide["y"] = 0;
	            if (this.dicSlide["x"] == 0)
	                this.Dic_Slide["x速度"] = 0;
	            else if ((this.Dic_Slide["x速度"] > 0 && this.dicSlide["x"] > 0) || (this.Dic_Slide["x速度"] < 0 && this.dicSlide["x"] < 0))
	                this.Dic_Slide["x速度"] = (this.Dic_Slide["x速度"] + this.dicSlide["x"]) / 2;
	            else
	                this.Dic_Slide["x速度"] = this.dicSlide["x"] / 2;
	            if (this.dicSlide["y"] == 0)
	                this.Dic_Slide["y速度"] = 0;
	            else if ((this.Dic_Slide["y速度"] > 0 && this.dicSlide["y"] > 0) || (this.Dic_Slide["y速度"] < 0 && this.dicSlide["y"] < 0))
	                this.Dic_Slide["y速度"] = (this.Dic_Slide["y速度"] + this.dicSlide["y"]) / 2;
	            else
	                this.Dic_Slide["y速度"] = this.dicSlide["y"] / 2;
	            this.Dic_Slide["上次x"] = gx;
	            this.Dic_Slide["上次y"] = gy;
	            this.dicSlide["gx"] = gx;
	            this.dicSlide["gy"] = gy;
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], this.dicSlide);
	        }
	        return true;
	    }
	    enterFrameF() {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false)
	            return;
	        this.Dic_Slide["计时"] -= 1;
	        if (this.Dic_Slide["计时"] <= 0) {
	            this.Dic_Slide["计时"] = 0;
	            this.Dic_Slide["x速度"] = 0;
	            this.Dic_Slide["y速度"] = 0;
	        }
	    }
	    enterFrameF2() {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false) {
	            return;
	        }
	        if (MainManager.getInstence().Time - this.Dic_LongTouch["start"] > this.Dic_LongTouch["need"]) {
	            Tool_Function.onRunFunction(this.Dic_LongTouch["fun"]);
	            this.clearLongTouch();
	            return;
	        }
	    }
	    clearLongTouch() {
	        if (this.FunEnterforLongTouch) {
	            MainManager.getInstence().removeEnterFrameFun(this.FunEnterforLongTouch);
	            this.FunEnterforLongTouch.clear();
	            this.FunEnterforLongTouch = null;
	        }
	        this.Dic_LongTouch = Tool_ObjUtils.destroyF_One(this.Dic_LongTouch);
	    }
	    destroyF() {
	        if (this.MC) {
	            this.MC.off(Laya.Event.MOUSE_DOWN, this, this.mouseHandler);
	            this.MC.off(Laya.Event.MOUSE_UP, this, this.mouseHandler);
	            this.MC.off(Laya.Event.MOUSE_MOVE, this, this.mouseHandler);
	            this.MC.off(Laya.Event.MOUSE_OUT, this, this.mouseHandler);
	            this.MC = null;
	        }
	        if (this.FunEnter) {
	            MainManager.getInstence().removeEnterFrameFun(this.FunEnter);
	            this.FunEnter.clear();
	            this.FunEnter = null;
	        }
	        this.clearLongTouch();
	        this.Dic_Slide = Tool_ObjUtils.destroyF_One(this.Dic_Slide);
	        this.Dic_Click = Tool_ObjUtils.destroyF_One(this.Dic_Click);
	        if (this.Dic_roll) {
	            Config.mStage.off(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	            this.Dic_roll = Tool_ObjUtils.destroyF_One(this.Dic_roll);
	        }
	        this.mm = Tool_ObjUtils.destroyF_One(this.mm);
	    }
	}

	class BTN_Starling extends Sprite {
	    constructor(tar) {
	        super();
	        this.isDown = false;
	        this.autoChangeFrame = true;
	        this.MC = tar;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, this.MC);
	        this.isMc = this.MC instanceof SwfMovieClip;
	        this.onChangeFrame(BTN_Starling.Frame_Nor);
	        let rec = this.getBounds();
	        this.BtnVir = { "startX": rec.x, "startY": rec.y, "endX": rec.x + tar.width, "endY": rec.y + tar.height };
	    }
	    getChildByName(_name) {
	        let one = super.getChildByName(_name);
	        if (one == null) {
	            one = this.MC.getChildByName(_name);
	        }
	        return one;
	    }
	    initTouch(fd, fu, fc, val = null) {
	        this.FClick = fc;
	        if (this.FClick)
	            this.FClick.once = false;
	        this.FDown = fd;
	        if (this.FDown)
	            this.FDown.once = false;
	        this.FUp = fu;
	        if (this.FUp)
	            this.FUp.once = false;
	        this.ID = val;
	        if (this.mme == null) {
	            this.mme = new MyMouseEventStarling(this);
	            this.mme.setValue("down事件", Laya.Handler.create(this, this.onDownF));
	            this.mme.setValue("up事件", Laya.Handler.create(this, this.onUpF));
	            this.mme.setValue("点击", Laya.Handler.create(this, this.onClickF));
	        }
	    }
	    setEventStop(real = true) {
	        if (this.mme) {
	            this.mme.setValue("停止冒泡", real);
	        }
	    }
	    onDownF(p) {
	        if (this.pause)
	            return;
	        if (this.FDown) {
	            Tool_Function.onRunFunction(this.FDown, this.ID);
	        }
	        if (this.autoChangeFrame == true)
	            this.onChangeFrame(BTN_Starling.Frame_Down);
	    }
	    onUpF(p) {
	        if (this.autoChangeFrame == true)
	            this.onChangeFrame(BTN_Starling.Frame_Nor);
	        if (this.pause)
	            return;
	        if (this.FUp) {
	            if (this.ID == null)
	                this.FUp.run();
	            else
	                this.FUp.runWith(this.ID);
	        }
	    }
	    onClickF(p) {
	        if (this.FClick) {
	            if (this.ID == null)
	                this.FClick.run();
	            else
	                this.FClick.runWith(this.ID);
	        }
	    }
	    onChangeFrame(f) {
	        if (this.isMc == false || this.MC == null)
	            return;
	        if (this.MC.totalFrames <= f)
	            return;
	        this.MC.gotoAndStop(f, false);
	    }
	    checkIn(mx, my) {
	        if (this._pause || this.visible == false)
	            return false;
	        if (this.BtnVir == null)
	            return false;
	        var lx = mx;
	        var ly = my;
	        if (lx >= this.BtnVir.startX && lx <= this.BtnVir.endX && ly >= this.BtnVir.startY && ly <= this.BtnVir.endY)
	            return true;
	        return false;
	    }
	    get pause() {
	        return this._pause;
	    }
	    set pause(value) {
	        this._pause = value;
	        if (this.pause == true)
	            this.setFrame(BTN_Starling.FramePause);
	        else
	            this.setFrame(BTN_Starling.Frame_Nor);
	    }
	    setFrame(n) {
	        if (this.isMc == false)
	            return;
	        if (n == this.nowFrame)
	            return;
	        this.nowFrame = n;
	        if (this.MC != null) {
	            if (this.MC.totalFrames > n)
	                this.MC.gotoAndStop(n, false);
	        }
	    }
	    destroyF() {
	        this.removeFromParent();
	        this.mme = Tool_ObjUtils.destroyF_One(this.mme);
	        this.MC = null;
	        if (this.FClick) {
	            this.FClick.clear();
	            this.FClick = null;
	        }
	        if (this.FDown) {
	            this.FDown.clear();
	            this.FDown = null;
	        }
	        if (this.FUp) {
	            this.FUp.clear();
	            this.FUp = null;
	        }
	    }
	}
	BTN_Starling.Frame_Nor = 0;
	BTN_Starling.Frame_Down = 1;
	BTN_Starling.Frame_Selected = 2;
	BTN_Starling.FramePause = 3;

	class Tool_SpriteUtils {
	    static onGetImageNum(fName, spr, _url, _Parentswf = null) {
	        var tmp = spr.getChildByName(_url);
	        if (tmp == null)
	            return null;
	        var classLink = tmp.classLink;
	        var arr = classLink.split("_");
	        if (fName == null)
	            fName = arr[1];
	        if (_Parentswf == null) {
	            for (var i = 2; i < arr.length; i++) {
	                if (arr[i].indexOf("swf") == 0) {
	                    _Parentswf = arr[i].slice(3);
	                    if (_Parentswf == "SWF")
	                        _Parentswf += "_" + arr[i + 1];
	                    break;
	                }
	            }
	        }
	        var num = new ImageNum(fName, _Parentswf);
	        num.setValue("父对象", tmp);
	        if (arr.length > 2) {
	            var info = {};
	            for (var j = 2; j < arr.length; j++) {
	                var one = arr[j];
	                if (one.indexOf("颜色") == 0) {
	                    one = one.slice(2);
	                    if (one.indexOf("0x") != 0)
	                        one = "0x" + one;
	                    num.setValue("颜色", Tool_Function.onChangeInstance(one));
	                }
	                else if (one.indexOf("对齐") == 0) {
	                    num.setValue("对齐", one.slice(2));
	                }
	            }
	        }
	        return num;
	    }
	    static onGetAll_ImageNum(spr, fName = null, _Parentswf = null) {
	        var dic = {};
	        var arrChild = [];
	        for (var i = 0; i < spr.numChildren; i++) {
	            var c = spr.getChildAt(i);
	            var str = c.name;
	            if (str == null || str.indexOf("num_") != 0)
	                continue;
	            arrChild.push(str);
	        }
	        for (i = 0; i < arrChild.length; i++) {
	            str = arrChild[i];
	            var tmp = str.split("_");
	            var name = tmp[0] + "_" + tmp[1];
	            var info = { "fName": fName, "swf": _Parentswf };
	            for (var j = 2; j < tmp.length; j++) {
	                var one = tmp[j];
	                if (one.indexOf("字体") == 0)
	                    info["fName"] = one.slice(2);
	                else if (one.indexOf("swf") == 0) {
	                    var strSWF = "";
	                    j++;
	                    while (j < tmp.length) {
	                        if (strSWF.length > 0)
	                            strSWF += "_";
	                        strSWF += tmp[j++];
	                    }
	                    info["swf"] = strSWF;
	                    break;
	                }
	                else if (one.indexOf("颜色") == 0) {
	                    one = one.slice(2);
	                    if (one.indexOf("0x") != 0)
	                        one = "0x" + one;
	                    info["颜色"] = Tool_Function.onChangeInstance(one);
	                }
	                else if (one.indexOf("对齐") == 0)
	                    info["对齐"] = one.slice(2);
	            }
	            var num = this.onGetImageNum(info["fName"], spr, str, info["swf"]);
	            if (info["颜色"] != null)
	                num.setValue("颜色", info["颜色"]);
	            if (info["对齐"] != null)
	                num.setValue("对齐", info["对齐"]);
	            dic[name] = num;
	        }
	        return dic;
	    }
	    static cloneSprite(target) {
	        var numChilds = target.numChildren;
	        var sprite = new Sprite();
	        var child;
	        for (var i = 0; i < numChilds; i++) {
	            child = target.getChildAt(i);
	            if (child instanceof Image) {
	                sprite.addChild(Tool_ImgUtils.cloneImage(child));
	            }
	            else if (child instanceof Sprite) {
	                sprite.addChild(Tool_SpriteUtils.cloneSprite(child));
	            }
	        }
	        sprite.x = target.x;
	        sprite.y = target.y;
	        sprite.width = target.width;
	        sprite.height = target.height;
	        sprite.skewX = target.skewX;
	        sprite.skewY = target.skewY;
	        sprite.alpha = target.alpha;
	        return sprite;
	    }
	    static getMetaData_FromSPR(spr) {
	        var meta;
	        if (spr instanceof Sprite && spr.spriteData[0] instanceof Array == false)
	            meta = spr.spriteData[0];
	        return meta;
	    }
	    static getBtn(spr, url) {
	        var B;
	        var tmp = spr.getChildByName(url);
	        if (tmp == null)
	            return null;
	        B = new BTN_Starling(tmp);
	        B.name = url;
	        return B;
	    }
	    static onAddchild_ChangeParent(spr, child) {
	        spr.x = child.x;
	        spr.y = child.y;
	        if (child.parent) {
	            child.parent.addChildAt(spr, child.parent.getChildIndex(child));
	        }
	        spr.addChild(child);
	        child.x = child.y = 0;
	        spr.name = child.name;
	    }
	    static getBounds(tar, parent) {
	        if (tar instanceof SwfS9Image || tar instanceof Image) {
	            let rec = new Laya.Rectangle();
	            rec.x = tar.x;
	            rec.y = tar.y;
	            rec.width = tar.width;
	            rec.height = tar.height;
	            return rec;
	        }
	        if (tar instanceof Sprite) {
	            var x0 = 9999999;
	            var y0 = 9999999;
	            var x1 = -9999999;
	            var y1 = -9999999;
	            for (let i = 0; i < tar.numChildren; i++) {
	                var rec = Tool_SpriteUtils.getBounds(tar.getChildAt(i), null);
	                if (rec.x < x0) {
	                    x0 = rec.x;
	                }
	                if (rec.x + rec.width > x1) {
	                    x1 = rec.x + rec.width;
	                }
	                if (rec.y < y0) {
	                    y0 = rec.y;
	                }
	                if (rec.y + rec.height > y1) {
	                    y1 = rec.y + rec.height;
	                }
	            }
	            if (rec == null) {
	                rec = new Laya.Rectangle();
	                rec.x = tar.x;
	                rec.y = tar.y;
	                return rec;
	            }
	            rec.x = x0 + tar.x;
	            rec.y = y0 + tar.y;
	            rec.width = x1 - x0;
	            rec.height = y1 - y0;
	            return rec;
	        }
	        return tar.getBounds();
	    }
	    static removeChildren(spr, index = 0, end = -1, dispose = false) {
	        var count = end <= index ? 99999999 : end - index;
	        for (; count > 0 && spr.numChildren > index; count--) {
	            var one = spr.getChildAt(index);
	            if (dispose == true)
	                Tool_ObjUtils.destroyF_One(one);
	            else
	                spr.removeChild(one);
	        }
	    }
	    static replaceSprite(layer, nameTar, newChild) {
	        let tar = layer.getChildByName(nameTar);
	        if (tar && newChild) {
	            newChild.x = tar.x;
	            newChild.y = tar.y;
	            if (tar.classLink.indexOf("透明") == -1) {
	                newChild.scaleX = tar.scaleX;
	                newChild.scaleY = tar.scaleY;
	            }
	            layer.addChildAt(newChild, layer.getChildIndex(tar));
	            layer.removeChild(tar);
	        }
	    }
	    static sortChildren(tar, fun) {
	        let arr = [];
	        for (let i = 0; i < tar.numChildren; i++) {
	            arr[i] = tar.getChildAt(i);
	        }
	        tar.removeChildren();
	        arr.sort(fun);
	        for (let i = 0; i < arr.length; i++) {
	            tar.addChild(arr[i]);
	        }
	    }
	    static fitSpriteToRec(spr, rec, scale, typex, typey) {
	        if (spr == null || rec == null)
	            return;
	        let w = spr.width;
	        let h = spr.height;
	        if (scale == true) {
	            let sx = rec.width / w;
	            let sy = rec.height / h;
	            let s = sx <= sy ? sx : sy;
	            if (s > 1)
	                s = 1;
	            w = w * s;
	            h = h * s;
	            spr.scaleX *= s;
	            spr.scaleY *= s;
	        }
	        if (typex == "中") {
	            spr.x = rec.x + (rec.width - w) / 2;
	        }
	        else if (typex == "左") {
	            spr.x = rec.x;
	        }
	        else {
	            spr.x = rec.x + rec.width - w;
	        }
	        if (typey == "中") {
	            spr.y = rec.y + (rec.height - h) / 2;
	        }
	        else if (typey == "上") {
	            spr.y = rec.y;
	        }
	        else {
	            spr.y = rec.y0 + (rec.height - h);
	        }
	    }
	    static addRecMask(tar, showRec) {
	        tar.scrollRect = new Laya.Rectangle(showRec.x, showRec.y, showRec.width, showRec.height);
	    }
	    static onReplaceChild(mc, class0, FnewTar) {
	        if (mc == null) {
	            return;
	        }
	        var _displayObjects = mc._displayObjects;
	        var arr;
	        var l;
	        arr = _displayObjects[class0];
	        if (arr == null)
	            return;
	        l = arr.length;
	        for (let i = 0; i < l; i++) {
	            var old = arr[i];
	            if (FnewTar)
	                arr[i] = FnewTar();
	            else
	                arr[i] = null;
	            if (old)
	                old.removeFromParent(true);
	        }
	        var f = mc.currentFrame;
	        if (mc.isPlay == false)
	            mc.gotoAndStop(f);
	        else
	            mc.gotoAndPlay(f);
	    }
	}

	var Handler$3 = Laya.Handler;
	class TmpMovieClip extends Sprite {
	    constructor(mc, f = null, type = "nor") {
	        super();
	        this.Arr = [];
	        this.End = false;
	        this.Type = type;
	        if (this.Type == "循环点击") {
	            if (mc instanceof SwfMovieClip) {
	                mc.loop = true;
	                mc.play();
	            }
	        }
	        else if (mc instanceof SwfMovieClip) {
	            mc.completeFunction = Handler$3.create(this, this.overF);
	            mc.play();
	        }
	        else if (mc instanceof MyZMovieClip) {
	            mc.completeFunction = Handler$3.create(this, this.overF);
	            mc.play();
	        }
	        else if (mc instanceof Sprite) {
	        }
	        this.Fun = f;
	        if (mc == null) {
	            this.overF();
	        }
	        else {
	            this.addMC(mc);
	        }
	    }
	    addMC(mc) {
	        if (mc == null)
	            return;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, mc);
	        this.Arr.push(mc);
	    }
	    overF(val = null) {
	        this.stopF();
	        if (this.Type == "nor") {
	            this.onRunFunction();
	            this.destroyF();
	        }
	        else if (this.Type == "点击") {
	            this.addClickF();
	        }
	        else if (this.Type == "屏幕点击") {
	            this.addScreenClickF();
	        }
	        else if (this.Type == "手动") {
	            this.onRunFunction();
	        }
	    }
	    stageF(e) {
	        this.addClickF();
	    }
	    addClickF() {
	        if (this.mme == null) {
	            this.mme = new MyMouseEventStarling(this);
	            this.mme.addClickHandler(Laya.Handler.create(this, clickF, null, true));
	        }
	        function clickF(e) {
	            this.onRunFunction();
	            this.destroyF();
	        }
	    }
	    addScreenClickF() {
	        function clickF(e) {
	            this.onRunFunction();
	            this.destroyF();
	        }
	    }
	    stopF() {
	        for (var i = 0; i < this.Arr.length; i++) {
	            var mc = this.Arr[i];
	            if (mc)
	                mc.stop();
	        }
	    }
	    playF() {
	        for (var i = 0; i < this.Arr.length; i++) {
	            var mc = this.Arr[i];
	            if (mc)
	                mc.play();
	        }
	    }
	    addStageRemoveListener() {
	    }
	    onRemoveF(e) {
	        this.destroyF();
	    }
	    onRunFunction() {
	        if (this.Fun != null) {
	            var f = this.Fun;
	            this.Fun = null;
	            Tool_Function.onRunFunction(f);
	        }
	    }
	    destroyF() {
	        this.Fun = null;
	        this.End = true;
	        this.mme = Tool_ObjUtils.destroyF_One(this.mme);
	        this.Arr = Tool_ObjUtils.destroyF_One(this.Arr);
	        Tool_ObjUtils.destroyDisplayObj(this);
	    }
	}

	class Tool_ViewFit {
	    static get isFat() {
	        var isFat;
	        if (Config.stageScaleInfo["屏幕w"] / Config.stageScaleInfo["屏幕h"] >= Config.stageW / Config.stageH) {
	            isFat = true;
	        }
	        else {
	            isFat = false;
	        }
	        return isFat;
	    }
	    static onViewFitF(tar) {
	        if (Config.TypeFit != 2) {
	            return;
	        }
	        var w0 = 50;
	        var h0 = 50;
	        var infoFit;
	        var i;
	        var child;
	        var rec;
	        var rec2;
	        var dicNo = {};
	        var typex;
	        var typey;
	        var info;
	        var dicStatic = {};
	        var dicSuc = {};
	        var s0;
	        var meta;
	        if (tar instanceof Sprite)
	            meta = tar.metaData;
	        if (meta == null || meta["适配"] == null) {
	            infoFit = {};
	        }
	        else
	            infoFit = meta["适配"];
	        var arrWaiteRelationship = [];
	        for (var key in infoFit) {
	            if (infoFit[key]["类型"] == "关联") {
	                arrWaiteRelationship.push(key);
	                child = tar.getChildByName(key);
	                this.onSaveAssociated(infoFit[key], tar, child);
	            }
	        }
	        for (key in infoFit) {
	            if (key.indexOf("合并") != 0) {
	                continue;
	            }
	            var arrChilds = infoFit[key]["包含"];
	            var recCombo = null;
	            for (i = 0; i < arrChilds.length; i++) {
	                dicNo[arrChilds[i]] = true;
	                child = tar.getChildByName(arrChilds[i]);
	                rec = Tool_SpriteUtils.getBounds(child, tar);
	                if (recCombo == null) {
	                    recCombo = rec;
	                }
	                else {
	                    if (rec.x < recCombo.x) {
	                        recCombo.x = rec.x;
	                    }
	                    if (rec.y < recCombo.y) {
	                        recCombo.y = rec.y;
	                    }
	                    if (rec.right > recCombo.right) {
	                        recCombo.width += rec.right - recCombo.right;
	                    }
	                    if (rec.bottom > recCombo.bottom) {
	                        recCombo.height += rec.bottom - recCombo.bottom;
	                    }
	                }
	            }
	            info["sx"] = info["sy"] = null;
	            info = this.onFitOne(recCombo, infoFit[key], null);
	            for (i = 0; i < arrChilds.length; i++) {
	                child = tar.getChildByName(arrChilds[i]);
	                rec2 = Tool_SpriteUtils.getBounds(child, tar);
	                child.x += rec.x - recCombo.x;
	                child.x -= (rec2.x - recCombo.x) * (1 - info.s);
	                child.y += (rec.y - recCombo.y);
	                child.y -= (rec2.y - recCombo.y) * (1 - info.s);
	                if (info["sx"])
	                    child.scaleX = info["sx"];
	                else
	                    child.scaleX *= info.s;
	                if (info["sy"])
	                    child.scaleY = info["sy"];
	                else
	                    child.scaleY *= info.s;
	                dicStatic[arrChilds[i]] = info.s;
	                dicSuc[arrChilds[i]] = true;
	            }
	        }
	        for (i = 0; i < tar.numChildren; i++) {
	            child = tar.getChildAt(i);
	            if (child.name && (dicNo[child.name] == true || arrWaiteRelationship.indexOf(child.name) != -1)) {
	                continue;
	            }
	            rec = Tool_SpriteUtils.getBounds(child, tar);
	            if ((rec.width > 0) == false)
	                rec.width = 10;
	            if ((rec.height > 0) == false)
	                rec.height = 10;
	            info = this.onFitOne(rec, infoFit[child.name], child.classLink);
	            if (info["sx"] != null) {
	                child.scaleX *= info.sx;
	                child.scaleY *= info.sy;
	                child.x = info.x + (child.x - rec.x) * info.sx;
	                child.y = info.y + (child.y - rec.y) * info.sy;
	            }
	            else {
	                child.scaleX *= info.s;
	                child.scaleY *= info.s;
	                child.x = info.x + (child.x - rec.x) * info.s;
	                child.y = info.y + (child.y - rec.y) * info.s;
	            }
	            dicStatic[child.name] = info.s;
	            dicSuc[child.name] = true;
	        }
	        var L = arrWaiteRelationship.length;
	        while (true) {
	            for (i = 0; i < arrWaiteRelationship.length; i++) {
	                key = arrWaiteRelationship[i];
	                if (dicSuc[infoFit[key]["tar"]] != true) {
	                    continue;
	                }
	                dicSuc[key] = true;
	                child = tar.getChildByName(key);
	                if (child == null)
	                    continue;
	                var childTar = tar.getChildByName(infoFit[key]["tar"]);
	                var s = 1;
	                if (infoFit[key]["ss"] != "无" && dicStatic[infoFit[key]["tar"]] != null) {
	                    s = dicStatic[infoFit[key]["tar"]];
	                }
	                child.scaleX *= s;
	                child.scaleY *= s;
	                if (infoFit[key]["关联方式"] == null) {
	                    child.x = childTar.x + infoFit[key]["x"] * s;
	                    child.y = childTar.y + infoFit[key]["y"] * s;
	                }
	                else {
	                    rec = Tool_SpriteUtils.getBounds(child, tar);
	                    rec2 = Tool_SpriteUtils.getBounds(childTar, tar);
	                    if (infoFit[key]["关联方式"].charAt(0) == "左") {
	                        child.x = childTar.x + infoFit[key]["x"] * s;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(0) == "右") {
	                        child.x += rec2.right + infoFit[key]["x"] * s - rec.right;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(0) == "中") {
	                        child.x = rec2.x + rec2.width / 2 - rec.width / 2 + infoFit[key]["x"] * s;
	                    }
	                    if (infoFit[key]["关联方式"].charAt(1) == "上") {
	                        child.y = childTar.y + infoFit[key]["y"] * s;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(1) == "下") {
	                        child.y += rec2.bottom + infoFit[key]["y"] * s - rec.bottom;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(1) == "中") {
	                        child.y = rec2.y + rec2.height / 2 - rec.height / 2 + infoFit[key]["y"] * s;
	                    }
	                }
	                arrWaiteRelationship.splice(i--, 1);
	            }
	            if (L == arrWaiteRelationship.length || arrWaiteRelationship.length == 0) {
	                break;
	            }
	            L = arrWaiteRelationship.length;
	        }
	    }
	    static onSaveAssociated(infoSelf, tar, child) {
	        var childTar = tar.getChildByName(infoSelf["tar"]);
	        if (childTar == null || child == null) {
	            return;
	        }
	        if (infoSelf["关联方式"] == null) {
	            infoSelf["x"] = child.x - childTar.x;
	            infoSelf["y"] = child.y - childTar.y;
	        }
	        else {
	            var rec = Tool_SpriteUtils.getBounds(child, tar);
	            var rec2 = Tool_SpriteUtils.getBounds(childTar, tar);
	            if (infoSelf["关联方式"].charAt(0) == "左") {
	                infoSelf["x"] = child.x - childTar.x;
	            }
	            else if (infoSelf["关联方式"].charAt(0) == "右") {
	                infoSelf["x"] = rec.right - rec2.right;
	            }
	            else if (infoSelf["关联方式"].charAt(0) == "中") {
	                infoSelf["x"] = rec.x + rec.width / 2 - (rec2.x + rec2.width / 2);
	            }
	            if (infoSelf["关联方式"].charAt(1) == "上") {
	                infoSelf["y"] = child.y - childTar.y;
	            }
	            else if (infoSelf["关联方式"].charAt(1) == "下") {
	                infoSelf["y"] = rec.bottom - rec2.bottom;
	            }
	            else if (infoSelf["关联方式"].charAt(1) == "中") {
	                infoSelf["y"] = rec.y + rec.height / 2 - (rec2.y + rec2.height / 2);
	            }
	        }
	    }
	    static onFitOne(recCombo, infoSelf, _name) {
	        var w0 = 50;
	        var h0 = 50;
	        var typex;
	        var typey;
	        var info = {};
	        if (infoSelf && infoSelf["强制rec"] != null) {
	            if (infoSelf["强制rec"] instanceof Array) {
	                if (infoSelf["强制rec"][0] != null) {
	                    recCombo.x = infoSelf["强制rec"][0];
	                }
	                if (infoSelf["强制rec"][1] != null) {
	                    recCombo.y = infoSelf["强制rec"][1];
	                }
	                if (infoSelf["强制rec"][2] != null) {
	                    recCombo.width = infoSelf["强制rec"][2];
	                }
	                if (infoSelf["强制rec"][3] != null) {
	                    recCombo.height = infoSelf["强制rec"][3];
	                }
	            }
	            else {
	                recCombo = infoSelf["强制rec"];
	            }
	        }
	        if (recCombo.x < w0) {
	            info["左"] = true;
	        }
	        if (recCombo.x + recCombo.width > Config.stageW - w0) {
	            info["右"] = true;
	        }
	        if (recCombo.y < h0) {
	            info["上"] = true;
	        }
	        if (recCombo.y + recCombo.height > Config.stageH - h0) {
	            info["下"] = true;
	        }
	        if (info["左"] == true && info["右"] == true) {
	            typex = "中";
	        }
	        else if (info["左"] == true) {
	            typex = "左";
	        }
	        else if (info["右"] == true) {
	            typex = "右";
	        }
	        else {
	            typex = null;
	        }
	        if (info["上"] == true && info["下"] == true) {
	            typey = "中";
	        }
	        else if (info["上"] == true) {
	            typey = "上";
	        }
	        else if (info["下"] == true) {
	            typey = "下";
	        }
	        else {
	            typey = null;
	        }
	        if (infoSelf && infoSelf["x"] != null) {
	            typex = infoSelf["x"];
	        }
	        if (infoSelf && infoSelf["y"] != null) {
	            typey = infoSelf["y"];
	        }
	        if (infoSelf && infoSelf["类型"] != null) {
	            typex = infoSelf["类型"].charAt(0);
	            typey = infoSelf["类型"].charAt(1);
	        }
	        var sCombine = 1;
	        var rec = {};
	        if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "xy") {
	            let sx = (Config.stageScaleInfo["屏幕w"] / Config.stageW);
	            let sy = (Config.stageScaleInfo["屏幕h"] / Config.stageH);
	            if (recCombo.width >= Config.stageW && recCombo.height >= Config.stageH) {
	                sCombine = sx >= sy ? sx : sy;
	            }
	            else {
	                sCombine = sx <= sy ? sx : sy;
	            }
	        }
	        else if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "x") {
	            sCombine = (Config.stageScaleInfo["屏幕w"] / Config.stageW);
	        }
	        else if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "y") {
	            sCombine = (Config.stageScaleInfo["屏幕h"] / Config.stageH);
	        }
	        else {
	            let sx = (Config.stageScaleInfo["屏幕w"] / Config.stageW);
	            let sy = (Config.stageScaleInfo["屏幕h"] / Config.stageH);
	            if (recCombo.width >= Config.stageW - 1 && recCombo.height >= Config.stageH - 1) {
	                sCombine = sx >= sy ? sx : sy;
	            }
	            else if (_name && _name.indexOf("s9_") == 0 && recCombo.width >= Config.stageW - 1) {
	                sCombine = sx >= sy ? sx : sy;
	                rec["sx"] = sx;
	                rec["sy"] = sy;
	            }
	            else {
	                sCombine = sx <= sy ? sx : sy;
	            }
	        }
	        rec["s"] = sCombine;
	        if (typex == "左") {
	            rec.x = (recCombo.x / Config.stageW) * Config.stageScaleInfo["屏幕w"];
	        }
	        else if (typex == "右") {
	            rec.x = Config.stageScaleInfo["屏幕w"] - ((Config.stageW - recCombo.right) / Config.stageW) * Config.stageScaleInfo["屏幕w"] - recCombo.width * sCombine;
	        }
	        else if (typex == "中") {
	            rec.x = Config.stageScaleInfo["屏幕w"] / 2 + ((recCombo.x + recCombo.width / 2 - Config.stageW / 2) / Config.stageW) * Config.stageScaleInfo["屏幕w"] - recCombo.width * sCombine / 2;
	        }
	        else {
	            rec.x = ((recCombo.x + recCombo.width / 2) / Config.stageW) * Config.stageRealW - recCombo.width * sCombine / 2;
	        }
	        if (typey == "上") {
	            rec.y = (recCombo.y / Config.stageH) * Config.stageScaleInfo["屏幕h"];
	        }
	        else if (typey == "下") {
	            rec.y = Config.stageScaleInfo["屏幕h"] - ((Config.stageH - recCombo.bottom) / Config.stageH) * Config.stageScaleInfo["屏幕h"] - recCombo.height * sCombine;
	        }
	        else if (typey == "中") {
	            rec.y = Config.stageScaleInfo["屏幕h"] / 2 + ((recCombo.y + recCombo.height / 2 - Config.stageH / 2) / Config.stageH) * Config.stageScaleInfo["屏幕h"] - recCombo.height * sCombine / 2;
	        }
	        else {
	            rec.y = ((recCombo.y + recCombo.height / 2) / Config.stageH) * Config.stageRealH - recCombo.height * sCombine / 2;
	        }
	        return rec;
	    }
	    static onWindowFitScreen(sprWin, scaleType = 0) {
	        if (scaleType == 0) {
	            var rec = Tool_SpriteUtils.getBounds(sprWin, sprWin.parent);
	            var sw = rec.width / Config.stageW;
	            var sh = rec.height / Config.stageH;
	            var wNeed = Config.stageScaleInfo["屏幕w"] * sw;
	            var hNeed = Config.stageScaleInfo["屏幕h"] * sh;
	            sprWin.scaleX *= wNeed / rec.width;
	            sprWin.scaleY *= hNeed / rec.height;
	        }
	        else if (scaleType == 1) {
	            sw = Config.stageScaleInfo["屏幕w"] / Config.stageW;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	            sprWin.x = (Config.stageScaleInfo["屏幕w"] - sprWin.width) / 2;
	            sprWin.y = (Config.stageScaleInfo["屏幕h"] - sprWin.height) / 2;
	        }
	        else if (scaleType == 5) {
	            sw = Config.stageScaleInfo["屏幕w"] / Config.stageW;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	        }
	        else if (scaleType == 6) {
	            sw = Config.stageScaleInfo["屏幕h"] / Config.stageH;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	        }
	        else if (scaleType == 2) {
	            sprWin.x = Config.stageScaleInfo["屏幕w"] / 2;
	            sprWin.y = Config.stageScaleInfo["屏幕h"] / 2;
	        }
	        else if (scaleType == 3) {
	            sprWin.x = Config.stageScaleInfo["屏幕w"] / 2;
	            sprWin.y = Config.stageScaleInfo["屏幕h"] / 2;
	            rec = Tool_SpriteUtils.getBounds(sprWin, sprWin.parent);
	            sw = rec.width / Config.stageW;
	            sh = rec.height / Config.stageH;
	            wNeed = Config.stageScaleInfo["屏幕w"] * sw;
	            hNeed = Config.stageScaleInfo["屏幕h"] * sh;
	            var smin = wNeed / rec.width;
	            if (hNeed / rec.height < smin) {
	                smin = hNeed / rec.height;
	            }
	            sprWin.scaleX *= smin;
	            sprWin.scaleY *= smin;
	        }
	        else if (scaleType >= 4 && scaleType < 5) {
	            wNeed = Config.stageW - Config.stageScaleInfo["屏幕w"];
	            hNeed = Config.stageH - Config.stageScaleInfo["屏幕h"];
	            if (wNeed == hNeed) {
	                return;
	            }
	            if (wNeed > hNeed) {
	                sprWin.scaleX = (Config.stageW - wNeed) / Config.stageW;
	                sprWin.scaleY = sprWin.scaleX;
	                sprWin.y = (Config.stageH * (1 - sprWin.scaleY)) / 2;
	            }
	            else {
	                sprWin.scaleY = (Config.stageH - hNeed) / Config.stageH;
	                sprWin.scaleX = sprWin.scaleY;
	                sprWin.x = (Config.stageW * (1 - sprWin.scaleX)) / 2;
	            }
	            if (scaleType != 4.1) {
	                let b = sprWin.getChildAt(0);
	                if (wNeed > hNeed) {
	                    b.y = -sprWin.y / sprWin.scaleY;
	                    b.scaleY /= sprWin.scaleX;
	                }
	                else {
	                    b.x = -sprWin.x / sprWin.scaleX;
	                    b.scaleX /= sprWin.scaleY;
	                    if (scaleType == 4.2) {
	                        b.y -= (b.height - Config.stageScaleInfo["屏幕h"]) / (2 * sprWin.scaleY);
	                    }
	                }
	            }
	        }
	    }
	    static onMcFit(tar, needclearBG = true, type = 4) {
	        if (tar instanceof TmpMovieClip) {
	            var mc = tar.Arr[0];
	            if (needclearBG) {
	                let bimg = mc.getChildAt(0);
	                mc.removeChildFromAllFrame(bimg.classLink);
	                tar.addChildAt(bimg, 0);
	                if (type == 4)
	                    type = 4.1;
	            }
	            Tool_ViewFit.onWindowFitScreen(tar, type);
	        }
	        else if (tar instanceof SwfMovieClip) {
	            Tool_ViewFit.onWindowFitScreen(tar, type);
	        }
	        else if (tar instanceof MyMC) {
	            this.onMcFit(tar.MC, needclearBG, 4);
	        }
	    }
	}

	var Handler$4 = Laya.Handler;
	class MyMC extends Sprite {
	    constructor(mc) {
	        super();
	        this.TimeFrame = 1000 / Config.playSpeedTrue;
	        this.pastTime = 0;
	        this.needLast = 0;
	        this.MC = mc;
	        this.fps = SwfMovieClip.FPS_default;
	        this.loop = false;
	        this.autoStopChild = false;
	        this.isPlay = false;
	        MyMC.addF(this);
	        if (this.MC != null) {
	            this.addChild(this.MC);
	            this.MC.stop(this.autoStopChild);
	        }
	    }
	    static addF(mc) {
	        this.dicAll.set(mc, true);
	    }
	    static removeF(mc) {
	        this.dicAll.remove(mc);
	    }
	    static pauseF() {
	        for (var i = 0; i < this.dicAll.keys.length; i++) {
	            var mc = this.dicAll.keys[i];
	            if (mc) {
	                mc.pauseF();
	            }
	        }
	    }
	    static resumeF() {
	        for (var i = 0; i < this.dicAll.keys.length; i++) {
	            var mc = this.dicAll.keys[i];
	            if (mc) {
	                mc.resumeF();
	            }
	        }
	    }
	    static onNewByMc(mc) {
	        if (mc == null)
	            return null;
	        return new MyMC(mc);
	    }
	    enterPlayF() {
	        this.pastTime += this.TimeFrame;
	        var f2 = parseInt("" + this.pastTime / this.NeedTime);
	        while (f2 > 0) {
	            this.nextFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    enterPrePlayF() {
	        this.pastTime += this.TimeFrame;
	        var f2 = parseInt("" + this.pastTime / this.NeedTime);
	        while (f2 > 0) {
	            this.preFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    gotoF(f) {
	        if (this.MC != null) {
	            if (f > this.endFrame) {
	                if (this.isComplete() == true)
	                    return;
	                f = this.endFrame;
	            }
	            this.MC.gotoAndStop(f, this.autoStopChild);
	            if (this.frameFunction)
	                Tool_Function.onRunFunction(this.frameFunction);
	            if (this.compFun) {
	                if (this.isComplete() == true) {
	                    Tool_Function.onRunFunction(this.compFun);
	                }
	            }
	        }
	    }
	    nextFrame() {
	        if (this.MC != null) {
	            if (this.isComplete() == true) {
	                if (this.loop == true) {
	                    this.gotoF(this.startFrame);
	                }
	            }
	            else {
	                this.gotoF(this.MC.currentFrame + 1);
	            }
	        }
	    }
	    preFrame() {
	        if (this.MC != null) {
	            if (this.MC.currentFrame == this.startFrame) {
	                if (this.loop == true)
	                    this.gotoF(this.endFrame);
	            }
	            else {
	                this.gotoF(this.MC.currentFrame - 1);
	            }
	        }
	    }
	    play() {
	        if (this.mmo) {
	            return;
	        }
	        this.mmo = new MainManagerOne();
	        this.mmo.addEnterFrameFun(Handler$4.create(this, this.onEnterF, null, false));
	    }
	    onEnterF() {
	        this.enterPlayF();
	        if (this.MC == null) {
	            this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	            return;
	        }
	        if (this.loop == true)
	            return;
	        if (this.isComplete() == true) {
	            this.stop();
	        }
	    }
	    stop() {
	        this.pastTime = 0;
	        if (this.mmo) {
	            this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	        }
	    }
	    gotoAndStop(f) {
	        if (typeof f == "string") {
	            this.nowLabel = f;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStopLable(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	        }
	        else {
	            this.nowLabel = null;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	            this.gotoF(f);
	        }
	        this.stop();
	    }
	    gotoAndPlay(f) {
	        if (typeof f == "string") {
	            this.nowLabel = f;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStopLable(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	        }
	        else {
	            this.nowLabel = null;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	            this.gotoF(f);
	        }
	        this.pastTime = 0;
	        this.play();
	    }
	    pauseF() {
	        this.stop();
	    }
	    resumeF() {
	        this.play();
	    }
	    addEvent(fun) {
	        if (this.MC instanceof MyZMovieClip) {
	            this.MC.FunEvent = fun;
	        }
	        else if (this.MC instanceof SwfMovieClip) {
	            this.MC.eventFunction = fun;
	        }
	    }
	    destroyF() {
	        MyMC.removeF(this);
	        this.frameFunction = Tool_ObjUtils.destroyF_One(this.frameFunction);
	        this.compFun = null;
	        this.MC = Tool_ObjUtils.destroyF_One(this.MC);
	        this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	        super.destroyF();
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(value) {
	        this._loop = value;
	        if (this.MC instanceof MyZMovieClip) {
	            this.MC.loop = this._loop;
	        }
	    }
	    get fps() {
	        return this._fps;
	    }
	    set fps(value) {
	        this._fps = value;
	        this.NeedTime = 1000 / this._fps;
	    }
	    get currentFrame() {
	        return this.MC.currentFrame;
	    }
	    get totalFrames() {
	        return this.MC.totalFrames;
	    }
	    isComplete() {
	        if (this.MC == null)
	            return true;
	        if (this.MC instanceof SwfMovieClip) {
	            return this.MC.currentFrame >= this.endFrame;
	        }
	        if (this.MC instanceof MyZMovieClip) {
	            return this.MC.currentFrame >= this.endFrame;
	        }
	        return this.MC.currentFrame >= this.endFrame;
	    }
	    get autoStopChild() {
	        return this._autoStopChild;
	    }
	    set autoStopChild(value) {
	        this._autoStopChild = value;
	    }
	    onWindowFit(needclearBG = true, type = 4) {
	        if (this.MC instanceof SwfMovieClip) {
	            var mc = this.MC;
	            if (needclearBG) {
	                var bimg = mc.getChildAt(0);
	                mc.removeChildFromAllFrame(bimg.classLink);
	                this.addChildAt(bimg, 0);
	            }
	            Tool_ViewFit.onWindowFitScreen(this, type);
	        }
	    }
	}
	MyMC.dicAll = new Dictionary();

	class Tool_ImgUtils {
	    static onNoSmooth(obj, maxS = 3) {
	    }
	    static onAddSmooth(obj) {
	    }
	    static onCreateImageNum(fName, spr, _url, _swf = null) {
	        var tmp = spr.getChildByName(_url);
	        if (tmp == null)
	            return null;
	        var num = new ImageNum(fName, _swf);
	        num.setValue("父对象", tmp);
	        return num;
	    }
	    static cloneImage(target) {
	        var image = new Image(target.texture);
	        image.x = target.x;
	        image.y = target.y;
	        image.width = target.width;
	        image.height = target.height;
	        image.skewX = target.skewX;
	        image.skewY = target.skewY;
	        image.alpha = target.alpha;
	        return image;
	    }
	    static onNewImageFromTexture(t) {
	        if (t == null) {
	            return null;
	        }
	        var image = new Image(t);
	        return image;
	    }
	    static setMipMap(tar, value) {
	        if (tar == null)
	            return;
	        if (tar instanceof Image) {
	            tar.setMipMap(value);
	        }
	        else if (tar instanceof MyMC) {
	            this.setMipMap(tar.MC, value);
	        }
	        else if (tar instanceof SwfMovieClip) {
	            for (let k in tar._displayObjects) {
	                let arr = tar._displayObjects[k];
	                if (arr == null)
	                    continue;
	                let l = arr.length;
	                for (let i = 0; i < l; i++) {
	                    this.setMipMap(arr[i], value);
	                }
	            }
	        }
	        else if (tar instanceof Sprite) {
	            for (let i = 0; i < tar.numChildren; i++) {
	                this.setMipMap(tar.getChildAt(i), value);
	            }
	        }
	    }
	}

	class Tool_ObjUtils {
	    static destroyF_One(obj) {
	        if (obj == null)
	            return null;
	        if (obj instanceof Array) {
	            for (var i = 0; i < obj.length; i++) {
	                Tool_ObjUtils.destroyF_One(obj[i]);
	            }
	            obj.length = 0;
	            Tool_ArrayUtils.returnArrayToPool(obj);
	        }
	        else if (obj instanceof Dictionary) {
	            for (let i = 0; i < obj.keys.length; i++) {
	                this.destroyF_One(obj.get[obj.keys[i]]);
	                if (typeof obj.keys[i] === "string" || typeof obj.keys[i] === "number") { }
	                else
	                    this.destroyF_One(obj.keys[i]);
	                obj.keys.splice(i--, 1);
	            }
	        }
	        else if (typeof obj === "number") {
	            return 0;
	        }
	        else if (this.hasFunction(obj, "destroyF") == true) {
	            obj.destroyF();
	            if (obj instanceof Sprite) {
	                this.destroyDisplayObj(obj);
	            }
	        }
	        else if (obj instanceof Laya.Handler) {
	            obj.clear();
	        }
	        else if (obj instanceof Sprite) {
	            this.destroyDisplayObj(obj);
	        }
	        else if (obj instanceof Laya.Text) {
	            obj.removeSelf();
	        }
	        else if (obj instanceof Laya.Sprite) {
	            obj.removeSelf();
	        }
	        else if (obj instanceof Object) {
	            for (let n in obj) {
	                if (n == "__proto__")
	                    continue;
	                this.destroyF_One(obj[n]);
	                delete obj[n];
	            }
	            this.returnObjectToPool(obj);
	        }
	        else if (obj instanceof Map) {
	            obj.forEach((val, key, map) => {
	                Tool_ObjUtils.destroyF_One(val);
	                Tool_ObjUtils.destroyF_One(key);
	            });
	            obj.clear();
	        }
	        return null;
	    }
	    static destroyDisplayObj(obj) {
	        if (obj != null && obj.isDestroyed == false) {
	            obj.isDestroyed = true;
	            obj.customVariable = Tool_ObjUtils.destroyF_One(obj.customVariable);
	            while (obj.numChildren > 0) {
	                Tool_ObjUtils.destroyF_One(obj.getChildAt(0));
	            }
	            obj.removeFromParent(true);
	        }
	    }
	    static hasFunction(target, method) {
	        try {
	            return target[method] instanceof Function;
	        }
	        catch (e) {
	            return false;
	        }
	    }
	    static isEqual(obj1, obj2) {
	        if ((obj1 == null || obj1 == NaN) && (obj2 == null || obj2 == NaN))
	            return true;
	        if ((obj1 == null || obj1 == NaN) || (obj2 == null || obj2 == NaN))
	            return false;
	        if (obj1 instanceof Array)
	            return Tool_ArrayUtils.isEqual(obj1, obj2);
	        if (Tool_Function.isTypeOf(obj1, "str")) {
	            return obj1 == obj2;
	        }
	        if (Tool_Function.isTypeOf(obj1, "num") && Tool_Function.isTypeOf(obj2, "num")) {
	            let num1 = Tool_Function.onChangeInstance(obj1, "num");
	            let num2 = Tool_Function.onChangeInstance(obj2, "num");
	            if (Math.abs(num1 - num2) < 0.00001)
	                return true;
	            return false;
	        }
	        if (obj1 instanceof Object)
	            return Tool_ObjUtils.isEqualObj(obj1, obj2);
	        return obj1 == obj2;
	    }
	    static isEqualObj(dic1, dic2) {
	        var dicHave = {};
	        var haveWrong = false;
	        for (let n in dic1) {
	            dicHave[n] = true;
	            if (Tool_ObjUtils.isEqual(dic1[n], dic2[n]) == false) {
	                haveWrong = true;
	                break;
	            }
	        }
	        for (let n in dic2) {
	            if (dicHave[n] != true) {
	                haveWrong = true;
	                break;
	            }
	        }
	        if (haveWrong == true)
	            return false;
	        return true;
	    }
	    static CopyF(obj, noCopy = false) {
	        if (typeof obj === "string" || typeof obj === "number" || obj == null || typeof obj === "boolean")
	            return obj;
	        if (obj instanceof Array) {
	            if (noCopy)
	                return Tool_ArrayUtils.copyArr(obj);
	            return Tool_ArrayUtils.copyArr(obj);
	        }
	        if (obj instanceof Dictionary) {
	            var out = Tool_ObjUtils.getNewObjectFromPool();
	            for (let i = 0; i < obj.keys.length; i++) {
	                out[obj.keys[i]] = Tool_ObjUtils.CopyF(obj.values[i], noCopy);
	            }
	            return out;
	        }
	        if (obj instanceof Object) {
	            var out = Tool_ObjUtils.getNewObjectFromPool();
	            for (let key in obj) {
	                out[key] = Tool_ObjUtils.CopyF(obj[key], noCopy);
	            }
	            return out;
	        }
	        if (noCopy)
	            return obj;
	        if (obj instanceof Image) {
	            return Tool_ImgUtils.cloneImage(obj);
	        }
	        if (obj instanceof Sprite) {
	            return Tool_SpriteUtils.cloneSprite(obj);
	        }
	        return obj;
	    }
	    static onComboObject(obj, tar, copy, adNumber = true) {
	        if (obj == null || tar == null)
	            return;
	        for (let key in tar) {
	            if (tar[key] == null) {
	                continue;
	            }
	            if (obj[key] == null || Tool_Function.isTypeOf(obj[key], "num") == false || Tool_Function.isTypeOf(tar[key], "num") == false || adNumber == false) {
	                if (copy == false)
	                    obj[key] = tar[key];
	                else
	                    obj[key] = this.CopyF(tar[key]);
	            }
	            else {
	                obj[key] += tar[key];
	            }
	        }
	    }
	    static onClearObj(obj) {
	        if (obj == null)
	            return;
	        for (var key in obj) {
	            obj[key] = this.destroyF_One(obj[key]);
	            delete obj[key];
	        }
	    }
	    static getLengthOfObject(obj) {
	        if (obj == null)
	            return 0;
	        var count = 0;
	        for (let key in obj) {
	            count++;
	        }
	        return count;
	    }
	    static isChildOfObject(tar, obj) {
	        if (obj) {
	            for (let key in obj) {
	                if (obj[key] == tar)
	                    return key;
	            }
	        }
	        return null;
	    }
	    static getRandomKeyFromObject(obj) {
	        let arr = [];
	        for (let key in obj) {
	            arr.push(key);
	        }
	        if (arr.length == 0)
	            return null;
	        var index = Tool_Function.onForceConvertType(Math.random() * arr.length);
	        return arr[index];
	    }
	    static addColorFilter(tar, type, val) {
	        var fil;
	        var matrix;
	        if (type == this.ColorFilterType_gray) {
	            if (val == null)
	                val = 0;
	            matrix = [
	                0.3086, 0.6094, 0.0820, 0, val,
	                0.3086, 0.6094, 0.0820, 0, val,
	                0.3086, 0.6094, 0.0820, 0, val,
	                0, 0, 0, 1, 0
	            ];
	        }
	        else if (type == this.ColorFilterType_light) {
	            matrix = [
	                1, 0, 0, 0, val,
	                0, 1, 0, 0, val,
	                0, 0, 1, 0, val,
	                0, 0, 0, 1, 0
	            ];
	        }
	        else if (type == this.ColorFilterType_fanse) {
	            matrix = [
	                -1, 0, 0, 0, 255,
	                0, -1, 0, 0, 255,
	                0, 0, -1, 0, 255,
	                0, 0, 0, 1, 0
	            ];
	        }
	        else if (type == this.ColorFilterType_color) {
	            matrix = [
	                Color.getRed(val) / 255, 0.6094, 0.0820, 0, 0,
	                0.3086, Color.getGreen(val) / 255, 0.0820, 0, 0,
	                0.3086, 0.6094, Color.getBlue(val) / 255, 0, 0,
	                0, 0, 0, 1, 0
	            ];
	        }
	        else
	            return null;
	        if (tar.filters != null) {
	            for (let i = tar.filters.length; i >= 0; i--) {
	                if (tar.filters[i] instanceof Laya.ColorFilter) {
	                    tar.filters.splice(i--, 1);
	                }
	            }
	        }
	        fil = new Laya.ColorFilter(matrix);
	        if (tar.filters == null)
	            tar.filters = [fil];
	        else
	            tar.filters.push(fil);
	        return fil;
	    }
	    static getNewObjectFromPool(...arg) {
	        var obj;
	        if (MyPools.getInstance().hasPool("Object") == false) {
	            obj = MyPools.getInstance().getFromPool("Object");
	        }
	        if (obj == null) {
	            obj = {};
	        }
	        if (arg.length > 0) {
	            for (var i = 0; i < arg.length; i += 2) {
	                obj[arg[i]] = arg[i + 1];
	            }
	        }
	        return obj;
	    }
	    static returnObjectToPool(obj) {
	        if (MyPools.getInstance().hasPool("Object") == false) {
	            return;
	        }
	        MyPools.getInstance().returnToPool("Object", obj);
	    }
	    static getKeyOfValue(dic, value) {
	        for (let key in dic) {
	            if (dic[key] == value)
	                return key;
	        }
	        return null;
	    }
	}
	Tool_ObjUtils.Dic_HaveFun = new Object();
	Tool_ObjUtils.ColorFilterType_gray = "灰度";
	Tool_ObjUtils.ColorFilterType_light = "亮度";
	Tool_ObjUtils.ColorFilterType_fanse = "反色";
	Tool_ObjUtils.ColorFilterType_color = "颜色";

	class LayerManager extends Laya.Sprite {
	    constructor() {
	        super();
	        LayerManager.instance = this;
	        this.LayerView = new Sprite();
	        this.addChild(this.LayerView);
	        this.LayerTop = new Sprite();
	        this.addChild(this.LayerTop);
	    }
	    static getInstence() {
	        if (LayerManager.instance == null)
	            LayerManager.instance = new LayerManager();
	        return LayerManager.instance;
	    }
	    static get LayerTop() { return this.getInstence().LayerTop; }
	    static get LayerView() { return this.getInstence().LayerView; }
	    get Scene3D_bg() {
	        if (this._scene3d_bg == null && window["Laya3D"]) {
	            this._scene3d_bg = Laya.stage.addChild(new Laya.Scene3D());
	            var camera = new Laya.Camera(0, 0.1, 100);
	            this.Scene3D_bg.addChild(camera);
	            let r = Tool_Function.onColor_getRed(Config.stageBGColor);
	            let g = Tool_Function.onColor_getGreen(Config.stageBGColor);
	            let b = Tool_Function.onColor_getBlue(Config.stageBGColor);
	            camera.clearColor = new Laya.Vector4(r / 255, g / 255, b / 255, 1);
	            camera.transform.translate(new Laya.Vector3(0, 2, 0));
	            camera.transform.rotate(new Laya.Vector3(-90, 0, 0), true, false);
	        }
	        return this._scene3d_bg;
	    }
	    init(_stage) {
	        _stage.addChild(this);
	    }
	    destroyF() {
	        var child;
	        while (this.LayerView.numChildren > 0) {
	            child = this.LayerView.getChildAt(0);
	            this.LayerView.removeChild(child);
	            Tool_ObjUtils.destroyF_One(child);
	        }
	        while (this.LayerTop.numChildren > 0) {
	            child = this.LayerTop.getChildAt(0);
	            this.LayerTop.removeChild(child);
	            Tool_ObjUtils.destroyF_One(child);
	        }
	        for (let i = 0; i < this.numChildren; i++) {
	            child = this.getChildAt(i);
	            if (child == this.LayerView || child == this.LayerTop)
	                continue;
	            this.removeChild(child);
	            i--;
	            Tool_ObjUtils.destroyF_One(child);
	        }
	    }
	}

	class MyKeyboardManager {
	    constructor(s) {
	        this.Dic = {};
	        this.Funs_Down = {};
	        this.Funs_Up = {};
	        this.Dic_stopEvent = {};
	        this.pause = false;
	        this.noKeyEvent = false;
	        if (MyKeyboardManager.Arr_instance == null)
	            MyKeyboardManager.Arr_instance = [];
	        MyKeyboardManager.Arr_instance.push(this);
	        this.S = s;
	        this.Funs_Down["Every"] = [];
	        this.Funs_Up["Every"] = [];
	        if (this.S) {
	            this.S.on(Laya.Event.KEY_DOWN, this, this.keyDown);
	            this.S.on(Laya.Event.KEY_UP, this, this.keyUP);
	        }
	    }
	    onForceDown(key) {
	        if (this.pause)
	            return;
	        if (this.Dic[key] == null || this.Dic[key] == false) {
	            this.Dic[key] = true;
	            var arr = this.Funs_Down["Every"];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i], key);
	            }
	            if (this.Funs_Down[key] == null)
	                return;
	            arr = this.Funs_Down[key];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i]);
	            }
	        }
	    }
	    onForceUp(key) {
	        if (this.pause)
	            return;
	        if (this.Dic[key] == null || this.Dic[key] == true) {
	            this.Dic[key] = false;
	            var arr = this.Funs_Up["Every"];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i], key);
	            }
	            if (this.Funs_Up[key] == null)
	                return;
	            arr = this.Funs_Up[key];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i]);
	            }
	        }
	    }
	    onUpAll() {
	        for (var key in this.Dic) {
	            this.Dic[key] = false;
	        }
	    }
	    keyDown(e) {
	        if (this.noKeyEvent || e == null)
	            return;
	        var key = e.keyCode;
	        if (this.DicChangekeys && this.DicChangekeys[key])
	            key = this.DicChangekeys[key];
	        this.onForceDown(key);
	    }
	    keyUP(e) {
	        if (this.noKeyEvent || e == null)
	            return;
	        var key = e.keyCode;
	        if (this.DicChangekeys && this.DicChangekeys[key])
	            key = this.DicChangekeys[key];
	        this.onForceUp(key);
	    }
	    isDown(key) {
	        if (this.Dic[key] == null)
	            return false;
	        return this.Dic[key];
	    }
	    addChangeKey(keyCode, key) {
	        if (this.DicChangekeys == null)
	            this.DicChangekeys = {};
	        this.DicChangekeys[keyCode] = key;
	    }
	    addFunctionListener(key, f, type = "down") {
	        if (f == null) {
	            console.log("keybord错误的参数");
	            return;
	        }
	        var dic;
	        if (type == "down")
	            dic = this.Funs_Down;
	        else if (type == "up")
	            dic = this.Funs_Up;
	        else {
	            console.log("keybord错误的参数");
	            return;
	        }
	        if (key == null) {
	            var arr = dic["Every"];
	            if (arr.indexOf(f) == -1)
	                arr.push(f);
	            return;
	        }
	        if (dic[key] == null)
	            dic[key] = [];
	        arr = dic[key];
	        if (arr.indexOf(f) == -1)
	            arr.push(f);
	    }
	    removeFunctionListener(key, f, type = "down") {
	        if (f == null) {
	            console.log("keybord错误的参数");
	            return;
	        }
	        var dic;
	        if (type == "down")
	            dic = this.Funs_Down;
	        else if (type == "up")
	            dic = this.Funs_Up;
	        else {
	            console.log("错误的参数");
	            return;
	        }
	        if (key == null) {
	            var arr = dic["Every"];
	            var index = arr.indexOf(f);
	            if (index != -1)
	                arr.splice(index, 1);
	            return;
	        }
	        if (dic[key] == null)
	            return;
	        arr = dic[key];
	        index = arr.indexOf(f);
	        if (index != -1)
	            arr.splice(index, 1);
	    }
	    addComboKeysListener(keys, f) {
	        if (this.Funs_ComboKeys == null)
	            this.Funs_ComboKeys = {};
	        var lastKey = keys[keys.length - 1];
	        if (this.Funs_ComboKeys[lastKey] == null) {
	            this.Funs_ComboKeys[lastKey] = [];
	            this.addFunctionListener(lastKey, Laya.Handler.create(this, function () {
	                for (var i = 0; i < this.Funs_ComboKeys[lastKey].length; i++) {
	                    var keys = this.Funs_ComboKeys[lastKey][i][0];
	                    var f = this.Funs_ComboKeys[lastKey][i][1];
	                    var suc = true;
	                    for (var j = 0; j < keys.length - 1; j++) {
	                        if (keys[j] == "ctrl") {
	                            if (this.isDown(17) == false) {
	                                suc = false;
	                                break;
	                            }
	                        }
	                        else {
	                            var needKey = typeof keys[j] === "string" ? keys[j] : String.fromCharCode(keys[j]);
	                            if (this.isDown(keys[j]) == false) {
	                                suc = false;
	                                break;
	                            }
	                        }
	                    }
	                    if (suc)
	                        f.run();
	                }
	            }));
	        }
	        this.Funs_ComboKeys[lastKey].push([keys, f]);
	    }
	    stopEventMop(key, val = true) {
	        this.Dic_stopEvent[key] = val;
	    }
	    destroyF() {
	        if (this.S) {
	            this.S.off(Laya.Event.KEY_DOWN, this, this.keyDown);
	            this.S.off(Laya.Event.KEY_UP, this, this.keyUP);
	        }
	        if (MyKeyboardManager.Arr_instance) {
	            let index = MyKeyboardManager.Arr_instance.indexOf(this);
	            if (index >= 0) {
	                MyKeyboardManager.Arr_instance.splice(index, 1);
	            }
	        }
	    }
	}
	MyKeyboardManager.KEY_Back = "返回";

	class MainManager {
	    constructor() {
	        this.Time = 0;
	        this.pause = false;
	        this.inEnter = false;
	        this.Dic_enterFun = new Dictionary();
	        this.Arr_delay = [];
	    }
	    static getInstence() {
	        if (MainManager._instence == null)
	            MainManager._instence = new MainManager();
	        return MainManager._instence;
	    }
	    init(stage) {
	        this.MEM = new MyEventManager();
	        this.MTM = new MyTweenerManager();
	        if (this.mstage == null) {
	            this.mstage = stage;
	            Config.mStage = stage;
	            Config.initF();
	            LayerManager.getInstence().init(stage);
	            Laya.timer.frameLoop(1, this, this.enterF, null, false);
	        }
	    }
	    enterF(e) {
	        if (this.pause == true) {
	            return;
	        }
	        this.Time++;
	        this.inEnter = true;
	        for (var i = 0; i < this.Dic_enterFun.keys.length; i++) {
	            var f = this.Dic_enterFun.keys[i];
	            if (f != null) {
	                Tool_Function.onRunFunction(f);
	            }
	        }
	        for (i = 0; i < this.Arr_delay.length; i++) {
	            var arr = this.Arr_delay[i];
	            if (arr[1] <= 0) {
	                Tool_Function.onRunFunction(arr[0], arr[2]);
	                this.remove_delayFunction(arr[0]);
	                i--;
	            }
	            else {
	                arr[1] -= 1;
	            }
	        }
	        this.inEnter = false;
	        if (this.Arr_waiteRemove != null) {
	            for (i = 0; i < this.Arr_waiteRemove.length; i++) {
	                this.removeEnterFrameFun(this.Arr_waiteRemove[i]);
	            }
	            this.Arr_waiteRemove = null;
	        }
	    }
	    addEnterFrameFun(fun) {
	        if (fun == null)
	            return;
	        if (fun instanceof Laya.Handler && fun.once == true) {
	            fun.once = false;
	        }
	        this.Dic_enterFun.set(fun, true);
	    }
	    removeEnterFrameFun(fun) {
	        if (fun == null)
	            return;
	        if (this.inEnter == true) {
	            if (this.Arr_waiteRemove == null)
	                this.Arr_waiteRemove = [];
	            this.Arr_waiteRemove.push(fun);
	        }
	        else {
	            for (var i = 0; i < this.Dic_enterFun.keys.length; i++) {
	                var f = this.Dic_enterFun.keys[i];
	                if (Tool_Function.compareHandlers(fun, f) == true) {
	                    this.Dic_enterFun.remove(f);
	                    i--;
	                    if (f instanceof Laya.Handler)
	                        f.clear();
	                }
	            }
	        }
	        if (fun instanceof Laya.Handler) {
	            fun.clear();
	        }
	    }
	    add_delayFunction(fun, delay, val = null) {
	        if (fun instanceof Laya.Handler) {
	            fun.once = true;
	            if (fun.method == null) {
	                console.log("添加delayFun失败，method=null");
	                return;
	            }
	        }
	        if (delay <= 0) {
	            if (val == null)
	                Tool_Function.onRunFunction(fun);
	            else
	                Tool_Function.onRunFunction(fun, val);
	            return;
	        }
	        this.Arr_delay.push([fun, delay, val]);
	    }
	    remove_delayFunction(fun) {
	        for (var i = 0; i < this.Arr_delay.length; i++) {
	            var arr = this.Arr_delay[i];
	            var f = arr[0];
	            if (Tool_Function.compareHandlers(fun, f) == true || (f instanceof Laya.Handler && f.caller == null)) {
	                this.Arr_delay.splice(i, 1);
	                if (f instanceof Laya.Handler)
	                    f.clear();
	                break;
	            }
	        }
	        if (fun instanceof Laya.Handler) {
	            fun.clear();
	        }
	    }
	    destroyF() {
	        this.remove_delayFunction(null);
	        this.removeEnterFrameFun(null);
	        if (this.MTM) {
	            this.MTM.stopAll();
	            this.MTM = null;
	        }
	        if (this.MEM) {
	            this.MEM.destroyF();
	            this.MEM = null;
	        }
	        MyKeyboardManager.Arr_instance = Tool_ObjUtils.destroyF_One(MyKeyboardManager.Arr_instance);
	        Laya.timer.clear(this, this.enterF);
	        LayerManager.getInstence().destroyF();
	        MainManager._instence = null;
	    }
	    clearF() {
	        this.remove_delayFunction(null);
	        this.removeEnterFrameFun(null);
	        if (this.MTM) {
	            this.MTM.stopAll();
	            this.MTM = null;
	        }
	        this.MTM = new MyTweenerManager();
	        if (this.MEM) {
	            this.MEM.destroyF();
	            this.MEM = null;
	        }
	        MyKeyboardManager.Arr_instance = Tool_ObjUtils.destroyF_One(MyKeyboardManager.Arr_instance);
	        this.MEM = new MyEventManager();
	    }
	}

	var SoundManager = Laya.SoundManager;
	class SoundManagerMy {
	    constructor() {
	        this.pause = 0;
	        this.mute = false;
	        this.isLoading = false;
	        this.Dic_fullName = {};
	    }
	    static getInstance() {
	        if (SoundManagerMy.instance == null) {
	            SoundManagerMy.instance = new SoundManagerMy();
	        }
	        return SoundManagerMy.instance;
	    }
	    addSounds(arr, f) {
	        if (this.isLoading == true) {
	            if (this.Arr_waite == null)
	                this.Arr_waite = [];
	            this.Arr_waite.push({ "arr": arr, "f": f });
	            return;
	        }
	        this.Arr_now = arr;
	        this.Fun = f;
	        this.nextF(true);
	    }
	    nextF(first = false) {
	        this.isLoading = true;
	        if (first == false) {
	            this.Arr_now.shift();
	            if (this.Arr_now.length == 0) {
	                Tool_Function.onRunFunction(this.Fun);
	                this.Fun = null;
	                this.Arr_now = null;
	                if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	                    this.isLoading = false;
	                }
	                else {
	                    var waite = this.Arr_waite.shift();
	                    this.Arr_now = waite["arr"];
	                    this.Fun = waite["f"];
	                    this.nextF(true);
	                }
	                return;
	            }
	        }
	        if (this.Arr_now[0] == null) {
	            this.nextF();
	            return;
	        }
	        var name = this.Arr_now[0][0];
	        var fullName;
	        var dir = this.Arr_now[0][1];
	        if (dir == null || dir.length == 0) {
	            fullName = name;
	        }
	        else if (dir.charAt(dir.length - 1) == "/") {
	            fullName = dir + name;
	        }
	        else {
	            fullName = dir + "/" + name;
	        }
	        fullName = SoundManagerMy.DirPath + "/" + fullName + "." + this.Arr_now[0][2];
	        this.Dic_fullName[name] = fullName;
	        this.nextF();
	    }
	    getFullName(name) {
	        return this.Dic_fullName[name];
	    }
	    playMusic(name, vol = -1, stopF = null, sTime = 0, loop = 9999, fadeTime = 0) {
	        if (vol <= 0)
	            vol = SoundManagerMy.soundVal;
	        name = this.getFullName(name);
	        if (name) {
	            SoundManager.playMusic(name, 0, stopF);
	        }
	    }
	    playSound(name, vol = -1) {
	        if (vol <= 0)
	            vol = SoundManagerMy.soundVal;
	        if (vol <= 0)
	            return;
	        name = this.getFullName(name);
	        if (name) {
	            SoundManager.playSound(name);
	        }
	    }
	    setVol(vol, name = null) {
	        SoundManager.soundVolume = vol;
	    }
	    stopMusic(name, fadeTime = 0) {
	        SoundManager.stopMusic();
	    }
	    removeSound(id) {
	    }
	    stopAll() {
	        SoundManager.stopAll();
	    }
	    haveSound(name) {
	        return true;
	    }
	    destroyF() {
	        SoundManagerMy.instance = null;
	        SoundManagerMy.nowMic = null;
	        SoundManagerMy.countBackNum = 0;
	    }
	}
	SoundManagerMy.soundVal = 1;
	SoundManagerMy.DirPath = "sounds";

	class MySourceManager {
	    constructor() {
	        this.Dic_Num = {};
	        this.Dic_have = {};
	        this.Dic_swf = {};
	        this.Dic_json = {};
	        this.Dic_txt = {};
	        this.Arr_waiteArrs = [];
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new MySourceManager();
	        return this.instance;
	    }
	    addTexture(arr, f) {
	        for (var i = 0; i < arr.length; i++) {
	            if (arr[i] == null)
	                continue;
	            var name = arr[i][0];
	            if (name == null)
	                continue;
	            if (this.Dic_Num[name] == null)
	                this.Dic_Num[name] = 0;
	            this.Dic_Num[name]++;
	        }
	        if (this.Arr_waite) {
	            this.Arr_waiteArrs.push([arr, f]);
	            return;
	        }
	        this.FunLoadOver = f;
	        this.Arr_waite = [];
	        for (i = 0; i < arr.length; i++) {
	            if (arr[i] == null)
	                continue;
	            this.Arr_waite.push(arr[i]);
	        }
	        MainManager._instence.add_delayFunction(Laya.Handler.create(this, this.loadNextF), 2, true);
	    }
	    removeTexture(name) {
	        if (name == null)
	            return;
	        if (--this.Dic_Num[name] > 0)
	            return;
	        this.onRemoveF(name);
	    }
	    onRemoveF(name) {
	        var swf = this.Dic_swf[name];
	        if (swf)
	            swf.destroyF();
	        delete this.Dic_swf[name];
	        delete this.Dic_have[name];
	    }
	    removeTextures(arr) {
	        while (arr.length > 0) {
	            if (arr[0] != null)
	                this.removeTexture(arr[0][0]);
	            arr.shift();
	        }
	    }
	    loadNextF(first = false) {
	        if (first != true) {
	            if (this.Arr_waite == null || this.Arr_waite.length == 0)
	                console.error("loadNextF中Arr_waite = " + this.Arr_waite);
	            else
	                this.Arr_waite.shift();
	        }
	        if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	            var tmpF = this.FunLoadOver;
	            this.FunLoadOver = null;
	            Tool_Function.onRunFunction(tmpF);
	            if (this.Arr_waiteArrs.length > 0) {
	                this.Arr_waite = this.Arr_waiteArrs[0][0];
	                this.FunLoadOver = this.Arr_waiteArrs[0][1];
	                this.Arr_waiteArrs.shift();
	                if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	                    console.error("新添加的Arr_waite == null!");
	                }
	                this.loadNextF(true);
	            }
	            else {
	                this.Arr_waite = null;
	            }
	            return;
	        }
	        var name = this.Arr_waite[0][0];
	        if (this.Dic_have[name] != null) {
	            MainManager.getInstence().MEM.dispatchF("加载进度", []);
	            MainManager._instence.add_delayFunction(Laya.Handler.create(this, this.loadNextF), 2);
	            return;
	        }
	        var Type = this.Arr_waite[0][1];
	        if (Type == "png" || Type == "jpg" || Type == "net") {
	            if (this.Arr_waite[0][0] == null) {
	                this.onWantNextF("pic");
	                return;
	            }
	            if (this.Dic_pic == null) {
	                this.Dic_pic = {};
	            }
	            var Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "Pic/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onPicEnd, [Add]));
	        }
	        else if (Type == "swf") {
	            if (this.Arr_waite[0][0] == null) {
	                this.onWantNextF("swf");
	                return;
	            }
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = name + "/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            new SWF(Add + name + ".json", Laya.Handler.create(this, this.onSwfLoaded));
	        }
	        else if (Type == "zmc") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = name + "/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            new MyZMovieClip(Add + name + ".sk", Laya.Handler.create(this, this.onZMCLoaded, [name]));
	        }
	        else if (Type == "mp3" || Type == "wav" || Type == "snd") {
	            var sm = SoundManagerMy.getInstance();
	            Add = "";
	            if (this.Arr_waite[0].length == 3)
	                Add = this.Arr_waite[0][2];
	            if (MySourceManager.DirMain) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Type == "snd") {
	                if (window["conch"])
	                    Type = "wav";
	                else if (Config.isPC == true && Config.Platform == "default")
	                    Type = "ogg";
	                else
	                    Type = "mp3";
	            }
	            var arr = [this.Arr_waite[0][0], Add, Type];
	            sm.addSounds([arr], Laya.Handler.create(this, this.onWantNextF));
	        }
	        else if (Type == "json") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onJsonLoaded), null, Laya.Loader.TEXT);
	        }
	        else if (Type == "txt") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onTxtLoaded), null, Laya.Loader.TEXT);
	        }
	        else {
	            Config.LogF("暂时不支持加载的文件" + Type);
	        }
	    }
	    onPicEnd(Add) {
	        var t = Laya.loader.getRes(Add);
	        this.Dic_pic[this.Arr_waite[0][0]] = t;
	        Config.LogF("pic加载成功：" + this.Arr_waite[0][0] + "：" + t);
	        this.onWantNextF("swf");
	    }
	    onZMCLoaded(_name, zmc) {
	        if (zmc == null) {
	            Config.LogF("加载zmc：" + _name + "失败！");
	        }
	        else {
	            if (this.Dic_zmc == null) {
	                this.Dic_zmc = {};
	            }
	            this.Dic_zmc[_name] = zmc;
	            this.Dic_have[_name] = true;
	        }
	        this.onWantNextF("zmc");
	    }
	    onSwfLoaded(swf) {
	        if (swf != null) {
	            this.Dic_have[swf.SwfName] = true;
	            this.Dic_swf[swf.SwfName] = swf;
	        }
	        this.onWantNextF("swf");
	    }
	    onJsonLoaded(tar) {
	        var Add = this.Arr_waite[0][2];
	        if (tar != null) {
	            try {
	                var data = JSON.parse(tar);
	                this.Dic_json[this.Arr_waite[0][0]] = data;
	            }
	            catch (error) {
	                console.error("加载的Json文件[" + this.Arr_waite[0][0] + "]解析失败!");
	            }
	        }
	        this.onWantNextF("json");
	    }
	    onTxtLoaded(tar) {
	        var Add = this.Arr_waite[0][2];
	        if (tar != null) {
	            this.Dic_txt[this.Arr_waite[0][0]] = tar;
	        }
	        this.onWantNextF("txt");
	    }
	    onWantNextF(type) {
	        if (this.Arr_waite == null && this.FunLoadOver == null)
	            return;
	        MainManager.getInstence().MEM.dispatchF("加载进度", []);
	        MainManager._instence.add_delayFunction(this.loadNextF.bind(this), 1);
	    }
	    getSwf(name) {
	        return this.Dic_swf[name];
	    }
	    getTextureFromSWF(swfName, name) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            Config.LogF("没有SWF：" + swfName);
	            return null;
	        }
	        return swf.getTexture(name);
	    }
	    getImgFromSwf(swfName, imgname) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            Config.LogF("没有SWF：" + swfName);
	            return null;
	        }
	        return swf.getImage(imgname);
	    }
	    getS9FromSwf(swfName, s9name) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            return null;
	        }
	        return swf.getS9Image(s9name);
	    }
	    getSprFromSwf(swfName, sprname) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            console.error("没有swf：" + swfName);
	            return null;
	        }
	        return swf.getSprite(sprname);
	    }
	    getMcFromSwf(swfName, mcname) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            if (this.Dic_zmc != null && this.Dic_zmc[swfName]) {
	                var mc = this.Dic_zmc[swfName];
	                var nmc = new MyZMovieClip(mc.URL, null);
	                nmc.initF();
	                if (mcname)
	                    nmc.gotoAndPlayLable(mcname);
	                return nmc;
	            }
	            return null;
	        }
	        return swf.getMc(mcname);
	    }
	    getObjFromSwf(swfName, objName) {
	        if (swfName == null || objName == null)
	            return null;
	        if (objName.indexOf("img_") == 0)
	            return this.getImgFromSwf(swfName, objName);
	        if (objName.indexOf("spr_") == 0)
	            return this.getSprFromSwf(swfName, objName);
	        if (objName.indexOf("mc_") == 0)
	            return this.getMcFromSwf(swfName, objName);
	        if (objName.indexOf("s9_") == 0)
	            return this.getS9FromSwf(swfName, objName);
	        return null;
	    }
	    getJson(name) {
	        return this.Dic_json[name];
	    }
	    getTxt(name) {
	        return this.Dic_txt[name];
	    }
	    destroyF() {
	        for (var n in this.Dic_Num) {
	            this.removeTexture(n);
	        }
	        MySourceManager.instance = null;
	    }
	}

	class Config {
	    static get isPC() { return this.OS.indexOf("Win") != -1; }
	    static get isPAD() {
	        if ((Config.stageRealW > Config.stageRealH && Config.stageRealW / Config.stageRealH <= 4.1 / 3) ||
	            (Config.stageRealW < Config.stageRealH && Config.stageRealH / Config.stageRealW <= 4.1 / 3)) {
	            return true;
	        }
	        return false;
	    }
	    static initF() {
	        if (this.stageW < this.stageH) {
	            Laya.stage.screenMode = "vertical";
	        }
	        else {
	            Laya.stage.screenMode = "horizontal";
	        }
	        Laya.stage.bgColor = Tool_Function.getColorStrFromInt(this.stageBGColor);
	        MySourceManager.getInstance();
	        console.log("屏幕大小：" + Laya.Browser.width + ":" + Laya.Browser.height);
	        this.deviceWidth = Laya.Browser.width;
	        this.deviceHeight = Laya.Browser.height;
	        this.mStage.frameRate = this.playSpeedTrue == 60 ? Laya.Stage.FRAME_FAST : Laya.Stage.FRAME_SLOW;
	        ;
	        if (Config.OS == "Win32") {
	        }
	        this.frameMS = 1000 / this.playSpeedTrue;
	        var sx = this.deviceWidth / this.stageW;
	        var sy = this.deviceHeight / this.stageH;
	        if (this.stageW < this.stageH && this.deviceWidth > this.deviceHeight) {
	            this.TypeFit = 1;
	        }
	        else if (this.stageW > this.stageH && this.deviceWidth < this.deviceHeight) {
	            this.TypeFit = 1;
	        }
	        if (this.TypeFit == 2) {
	            if (sx < sy)
	                Laya.stage.scaleMode = Laya.Stage.SCALE_FIXED_HEIGHT;
	            else
	                Laya.stage.scaleMode = Laya.Stage.SCALE_FIXED_WIDTH;
	            this.stageScale = sx > sy ? sx : sy;
	            this.stageScaleInfo = { "屏幕w": this.deviceWidth / Config.stageScale, "屏幕h": this.deviceHeight / Config.stageScale };
	            this.LogF("屏幕宽" + this.deviceWidth + "屏幕高" + this.deviceHeight + "，缩放=" + this.stageScale + "，屏幕真实占用：", this.stageScaleInfo);
	        }
	        else {
	            this.stageScaleInfo = { "屏幕w": this.stageW, "屏幕h": this.stageH };
	            Laya.stage.scaleMode = Laya.Stage.SCALE_SHOWALL;
	        }
	        this.stageRealW = this.stageScaleInfo["屏幕w"];
	        this.stageRealH = this.stageScaleInfo["屏幕h"];
	        if (Config.isPC == true && Config.Platform == "default") {
	            Config.debug = true;
	            SoundManagerMy.DirPath = "sounds_ogg";
	        }
	        else if (window["conch"]) {
	            SoundManagerMy.DirPath = "sounds";
	        }
	        else {
	            SoundManagerMy.DirPath = "sounds_mp3";
	        }
	    }
	    static onThrowNewErrorF(mess) {
	        window.alert(mess);
	        throw mess;
	    }
	    static onCloseProgram() {
	        if (window["conch"]) {
	            window["conch"].exit();
	        }
	        else {
	            window.close();
	        }
	    }
	    static LogF(...arg) {
	        if (arg.length == 0)
	            return;
	        var str = "";
	        while (arg.length > 0) {
	            if (str.length > 0)
	                str += "  ";
	            str += this.toStringF(arg[0]);
	            arg.shift();
	        }
	        if (this.debug)
	            console.log(str);
	        if (this.FunLog) {
	            this.FunLog.runWith(str);
	        }
	    }
	    static LogError(...arg) {
	        if (arg.length == 0)
	            return;
	        var str = "";
	        while (arg.length > 0) {
	            if (str.length > 0)
	                str += "  ";
	            str += this.toStringF(arg[0]);
	            arg.shift();
	        }
	        console.error(str);
	        if (this.FunLog) {
	            this.FunLog.runWith(str);
	        }
	    }
	    static toStringF(obj) {
	        var str = "";
	        if (typeof obj === "string" || typeof obj === "number" || obj == null || typeof obj === "boolean")
	            str += obj;
	        else if (obj instanceof Array) {
	            str += "[";
	            for (var i = 0; i < obj.length; i++) {
	                if (i > 0)
	                    str += ", ";
	                str += this.toStringF(obj[i]);
	            }
	            str += "]";
	        }
	        else if (obj instanceof Laya.Rectangle) {
	            str += "RECT::{x=" + obj.x + ",y=" + obj.y + ",w=" + obj.width + ",h=" + obj.height + "}";
	        }
	        else if (obj instanceof Laya.Point) {
	            str += "POINT::{x=" + obj.x + ",y=" + obj.y + "}";
	        }
	        else if (obj instanceof Dictionary || obj instanceof Object) {
	            str += "{";
	            i = 0;
	            for (var key in obj) {
	                if (i > 0)
	                    str += ", ";
	                str += this.toStringF(key) + "::" + this.toStringF(obj[key]);
	                i++;
	            }
	            str += "}";
	        }
	        else {
	            if (Tool_ObjUtils.hasFunction(obj, "toString") == true) {
	                str += Tool_Function.getLastClassName(obj) + "->" + obj.toString();
	            }
	            else {
	                str += "实例::" + Tool_Function.getLastClassName(obj) + "-->" + obj;
	            }
	        }
	        return str;
	    }
	}
	Config.stageW = 1280;
	Config.stageH = 640;
	Config.playSpeedTrue = 60;
	Config.stageBGColor = 0;
	Config.swfFPS = 30;
	Config.SoundVol = 0.5;
	Config.MainDevice = "H5";
	Config.Platform = "default";
	Config.TypeFit = 2;
	Config.VerMain = 1.1;
	Config.VerSecond = 0;
	Config.Active = true;
	Config.debug = false;
	Config.Set_pauseCMDonBack = true;

	class PlayerMain {
	}
	PlayerMain.reopen = false;

	class MyDataStatistics {
	    static getDefaultCustomData() {
	        let dic = {};
	        dic["channel"] = Config.Platform;
	        dic["ver"] = Config.VerMain + "." + Config.VerSecond;
	        return dic;
	    }
	    static init(data, fun) {
	        if (this.inited == 1)
	            return;
	        if (MyDataStatistics.SDK == "极光") {
	            if (this.inited == 0) {
	                this.inited = 1;
	                MyDataStatistics.inited = 99;
	                MyDataStatistics.init(data, fun);
	                return;
	                dynamicLoadJs("https://web-stat.jiguang.cn/web-janalytics/scripts/janalytics-web.min.js", () => {
	                    MyDataStatistics.init(data, fun);
	                });
	                return;
	            }
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                MyDataStatistics.inited = 99;
	                return;
	            }
	            MyDataStatistics.inited = 100;
	            if (data.channel == null)
	                data.channel = Config.Platform;
	            window["JAnalyticsInterface"].init({
	                appkey: data.appkey,
	                debugMode: false,
	                channel: data.channel,
	                loc: true,
	                singlePage: false
	            });
	            Tool_Function.onRunFunction(fun);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            window["Platform"] = Config.Platform;
	            window["sequenceNumber"] = data.appkey;
	            window["appDisplayName"] = '';
	            window["DTGARequestUrl"] = 'http://tdwv1.talkingdata.com/g/h5/v1/receive';
	            dynamicLoadJs("libs2/talkingdata.js", () => {
	                console.info("初始化" + MyDataStatistics.SDK + "成功");
	                this.inited = 1;
	                Tool_Function.onRunFunction(fun);
	            });
	        }
	        function dynamicLoadJs(url, callback) {
	            var head = document.getElementsByTagName('head')[0];
	            var script = document.createElement('script');
	            script.type = 'text/javascript';
	            script.src = url;
	            script.onload = function () {
	                callback();
	                script.onload = null;
	            };
	            head.appendChild(script);
	        }
	    }
	    static login(suc, data) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	            var LoginEvent = window["JAnalyticsInterface"].Event.LoginEvent;
	            var lEvent = new LoginEvent(Config.Platform, suc);
	            if (data) {
	                for (let key in data) {
	                    lEvent.addKeyValue(key, data[key]);
	                }
	            }
	            window["JAnalyticsInterface"].onEvent(lEvent);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            let dic = { "accountType": 1 };
	            dic["accountId"] = data.uid + "";
	            if (data.lv != null)
	                dic["level"] = data.lv;
	            if (data.ser != null)
	                dic["gameServer"] = data.ser;
	            if (data.nick != null)
	                dic["accountName"] = data.nick;
	            window["TDGA"].Account(dic);
	            MyBridge.Log("成功调用登录统计！uid =" + data.uid);
	        }
	    }
	    static regist(suc, data) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	            var RegisterEvent = window["JAnalyticsInterface"].Event.RegisterEvent;
	            var rEvent = new RegisterEvent(Config.Platform, suc);
	            if (data) {
	                for (let key in data) {
	                    rEvent.addKeyValue(key, data[key]);
	                }
	            }
	            rEvent.addKeyValue("key1", "value1").addKeyValue("key2", "value2");
	            window["JAnalyticsInterface"].onEvent(rEvent);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            let dic = { "accountType": 1 };
	            dic["accountId"] = data.uid + "";
	            dic["level"] = 1;
	            if (data.ser != null)
	                dic["gameServer"] = data.ser;
	            if (data.nick != null)
	                dic["accountName"] = data.nick;
	            window["TDGA"].Account(dic);
	        }
	    }
	    static onLevelUp(lv) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].Account.setLevel(lv);
	        }
	    }
	    static changeGold(gold, reason) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onReward(gold, reason);
	        }
	    }
	    static onPaySuccess(orderId, goodName, money, goodNum, payment) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onChargeSuccess({
	                orderId: orderId, iapId: goodName, currencyAmount: money + '', currencyType: 'CNY', virtualCurrencyAmount: goodNum, paymentType: payment
	            });
	        }
	    }
	    static onShopping(_name, price, num = 1) {
	        if (price < 0)
	            price = -price;
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onItemPurchase({
	                item: _name,
	                itemNumber: num,
	                priceInVirtualCurrency: price
	            });
	        }
	    }
	    static useItem(_name, num) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onItemUse({
	                item: _name,
	                itemNumber: num
	            });
	        }
	    }
	    static onMissionStart(mid) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	            var CountEvent = window["JAnalyticsInterface"].Event.CountEvent;
	            var cEvent = new CountEvent("任务" + mid);
	            cEvent.addKeyValue("开始", 1);
	            window["JAnalyticsInterface"].onEvent(cEvent);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onMissionBegin(mid + "");
	            MyBridge.Log("成功统计开始任务：" + mid);
	        }
	    }
	    static onMissionComplete(mid) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	            var CountEvent = window["JAnalyticsInterface"].Event.CountEvent;
	            var cEvent = new CountEvent("任务" + mid);
	            cEvent.addKeyValue("成功", 1);
	            window["JAnalyticsInterface"].onEvent(cEvent);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onMissionCompleted(mid + "");
	        }
	    }
	    static onMissionFaild(mid, reason) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	            var CountEvent = window["JAnalyticsInterface"].Event.CountEvent;
	            var cEvent = new CountEvent("任务" + mid);
	            cEvent.addKeyValue("失败", 1);
	            window["JAnalyticsInterface"].onEvent(cEvent);
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onMissionFailed(mid + "", reason);
	        }
	    }
	    static onPlayerDead(data) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onEvent('死亡', data);
	        }
	    }
	    static onCustomEvent(ename, data) {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onEvent(ename, data);
	        }
	    }
	    static onLeaveGame() {
	        if (MyDataStatistics.SDK == "极光") {
	            if (window["JAnalyticsInterface"] == null) {
	                console.error("没有引入极光SDK");
	                return;
	            }
	        }
	        else if (MyDataStatistics.SDK == "TalkingData") {
	            if (window["TDGA"] == null) {
	                console.error("没有引入TalkingData的SDK");
	                return;
	            }
	            window["TDGA"].onPageLeave();
	        }
	    }
	}
	MyDataStatistics.inited = 0;

	class BGCon_Statistic {
	    constructor() {
	        MyDataStatistics.SDK = "TalkingData";
	        var appkey;
	        if (MyBridge.dicValue["appkey"] != null)
	            appkey = MyBridge.dicValue["appkey"];
	        else if (PlayerMain.Program == "火焰纹章")
	            appkey = "193B5EF71D904C8CA032076444C23507";
	        else if (PlayerMain.Program == "少儿情绪管理")
	            appkey = "D45C602E92744255B9CD5D4888169880";
	        else {
	            MyBridge.Log("没有appkey，也没有预设项目：" + PlayerMain.Program);
	            MyBridge.onLeave();
	            return;
	        }
	        MyDataStatistics.init({ "appkey": appkey, "channel": Config.Platform }, this.beginLoad.bind(this));
	    }
	    static get instance() {
	        if (this._instance == null)
	            this._instance = new BGCon_Statistic();
	        return this._instance;
	    }
	    beginLoad() {
	        if (MyBridge.dicValue["初始统计"] != null && PlayerMain.reopen == false) {
	            this.aslistener(MyBridge.dicValue["初始统计"]["con"], MyBridge.dicValue["初始统计"]["data"]);
	        }
	        if (this.Arr_waite && MyDataStatistics.inited == 1) {
	            for (let i = 0; i < this.Arr_waite.length; i++) {
	                this.onEventF(this.Arr_waite[i]);
	            }
	            this.Arr_waite = null;
	        }
	    }
	    onEventF(val) {
	        if (MyDataStatistics.inited == 0) {
	            if (this.Arr_waite == null)
	                this.Arr_waite = [];
	            this.Arr_waite.push(val);
	            return;
	        }
	        MyBridge.Log("开始统计：" + JSON.stringify(val));
	        try {
	            let con = val["con"];
	            let data = val["data"];
	            this.aslistener(con, data);
	        }
	        catch (error) {
	            MyBridge.Log("统计报错！" + error);
	        }
	    }
	    aslistener(con, data) {
	        if (data && Tool_Function.isTypeOf(data, "str") && (data.charAt(0) == "{" || data.charAt(0) == "[")) {
	            try {
	                data = JSON.parse(data);
	            }
	            catch (error) {
	                MyBridge.Log(con + "统计报错：json格式错误");
	            }
	        }
	        else if (data == null) {
	            data = {};
	        }
	        if (con == "注册") {
	            if (data["uid"] == null)
	                data["uid"] = PlayerMain.UID;
	            if (data["nick"] == null)
	                data["nick"] = PlayerMain.Nick;
	            MyDataStatistics.regist(true, data);
	        }
	        else if (con == "登陆" || con == "登录") {
	            if (data["uid"] == null)
	                data["uid"] = PlayerMain.UID;
	            if (data["nick"] == null)
	                data["nick"] = PlayerMain.Nick;
	            MyDataStatistics.login(true, data);
	        }
	        else if (con == "升级") {
	            MyDataStatistics.onLevelUp(Tool_Function.onForceConvertType(data["lv"]));
	        }
	        else if (con == "开始任务") {
	            MyDataStatistics.onMissionStart(data["mission"]);
	        }
	        else if (con == "任务完成") {
	            MyDataStatistics.onMissionComplete(data["mission"]);
	        }
	        else if (con == "任务失败") {
	            MyDataStatistics.onMissionFaild(data["mission"], data["reason"]);
	        }
	        else if (con == "获得金币") {
	            MyDataStatistics.changeGold(data["gold"], data["reason"]);
	        }
	        else if (con == "购买商品") {
	            MyDataStatistics.onShopping(data["商品"], data["价格"], data["数量"]);
	        }
	        else if (con == "充值") {
	            MyDataStatistics.onPaySuccess(data["订单号"], data["商品"], data["价格"], data["数量"], data["方式"]);
	        }
	        else if (con == "事件") {
	            MyDataStatistics.onCustomEvent(data["name"], data["value"]);
	        }
	        else if (con == "离开") {
	            MyDataStatistics.onLeaveGame();
	        }
	    }
	}

	var HttpRequest = Laya.HttpRequest;
	class Net_HttpRequest {
	    constructor(_ip, _data, f, post = false, _needBase64 = false, _needJson = false, timeout = -1, headers = null) {
	        this.mmo = new MainManagerOne();
	        this.isOver = false;
	        this.IP = _ip;
	        this.Data = _data;
	        this.Fun = f;
	        this.isPost = post;
	        this.needBase64 = _needBase64;
	        this.needJson = _needJson;
	        this.timeout = timeout;
	        if (this.timeout == -1)
	            this.timeout = Net_HttpRequest.timeOut;
	        if (this.timeout < 100)
	            this.timeout = this.timeout * 1000;
	        else if (this.timeout < 1000)
	            this.timeout = 1000;
	        this.needHeaders = headers;
	        this.connectF();
	    }
	    connectF() {
	        this.hr = new HttpRequest();
	        Net_HttpRequest.ArrHr.push(this.hr);
	        if (this.timeout == -1)
	            this.hr.http.timeout = Net_HttpRequest.timeOut;
	        else
	            this.hr.http.timeout = this.timeout;
	        this.mmo.add_delayFunction(Laya.Handler.create(this, this.onTimeOut), this.timeout * 0.001 * Config.playSpeedTrue, null);
	        this.hr.once(Laya.Event.COMPLETE, this, this.onHttpRequestComplete);
	        this.hr.once(Laya.Event.ERROR, this, this.onHttpRequestError);
	        if (this.isPost == false) {
	            let url = this.IP;
	            if (this.Data) {
	                if (url.indexOf("?") == -1)
	                    url += "?" + this.getDatasUrl();
	                else
	                    url += "&" + this.getDatasUrl();
	            }
	            this.hr.send(url, null, 'get', 'text');
	        }
	        else {
	            this.hr.http.open("post", this.IP);
	            if (this.needHeaders) {
	                for (let headername in this.needHeaders) {
	                    this.hr.http.setRequestHeader(headername, this.needHeaders[headername]);
	                }
	            }
	            if (Tool_Function.isTypeOf(this.Data, "str")) {
	                this.hr.send(this.IP, this.Data, 'post', 'text');
	            }
	            else {
	                this.hr.send(this.IP, JSON.stringify(this.Data), 'post', 'text');
	            }
	        }
	    }
	    getDatasUrl() {
	        if (Tool_Function.isTypeOf(this.Data, "str")) {
	            return this.Data;
	        }
	        let data = this.Data;
	        let url = "";
	        var first = true;
	        for (let key in data) {
	            if (first == false)
	                url += "&";
	            url += key + "=";
	            first = false;
	            if (this.Data[key] instanceof Array || this.Data[key] instanceof Object) {
	                url += JSON.stringify(this.Data[key]);
	            }
	            else {
	                url += this.Data[key];
	            }
	        }
	        return url;
	    }
	    onTimeOut() {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	        Tool_Function.onRunFunction(this.Fun, null);
	    }
	    onHttpRequestError(e) {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	        Tool_Function.onRunFunction(this.Fun, null);
	    }
	    onHttpRequestComplete(e) {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        this.mmo = Tool_ObjUtils.destroyF_One(this.mmo);
	        let data = null;
	        try {
	            if (this.needJson && this.hr.data) {
	                data = JSON.parse(this.hr.data);
	            }
	            else {
	                data = this.hr.data;
	            }
	        }
	        catch (error) {
	            console.error("收到网络信息报错：" + error);
	            Tool_Function.onRunFunction(this.Fun, null);
	            return;
	        }
	        let index = Net_HttpRequest.ArrHr.indexOf(this.hr);
	        if (index >= 0)
	            Net_HttpRequest.ArrHr.splice(index, 1);
	        Tool_Function.onRunFunction(this.Fun, data);
	    }
	}
	Net_HttpRequest.timeOut = 8000;
	Net_HttpRequest.ArrHr = [];

	class BGCon_Pay {
	    constructor(paytype, goods, message, price) {
	        if (paytype == null) {
	            MyBridge.onLeave("缺少type");
	            return;
	        }
	        if (goods == null) {
	            MyBridge.onLeave("缺少商品");
	            return;
	        }
	        if (message == null) {
	            MyBridge.onLeave("缺少描述");
	            return;
	        }
	        if (price == null) {
	            MyBridge.onLeave("缺少价格");
	            return;
	        }
	        if (PlayerMain.UID <= 0) {
	            MyBridge.onLeave("缺少uid");
	            return;
	        }
	        let paydata = { "type": paytype, "money": price, "商品": goods, "项目": PlayerMain.Program, "uid": 1, "dc": PlayerMain.DC, "描述": message };
	        new Net_HttpRequest("http://" + PlayerMain.ServerHttpURL + "/toGlobalPay", paydata, (v) => {
	            if (v == null) {
	                MyBridge.onLeave("联网失败！");
	                return;
	            }
	            if (v.indexOf("http") == -1 && v.includes("html") == false) {
	                MyBridge.onLeave(v);
	                return;
	            }
	            if (paytype == "微信") {
	                this.getWXInfo(v);
	            }
	            else {
	            }
	        });
	    }
	    getWXInfo(val) {
	        if (val.indexOf("http") == 0) {
	            new Net_HttpRequest(val, null, (v) => {
	                console.log(v);
	                window.location.href = v;
	            }, true, false, false, -1, { "Referer": "http://www.miracle-cn.com" });
	        }
	        else {
	            if (val.indexOf("<form") != -1 || val.includes("<html>")) {
	                let i = val.indexOf("url=\"weixin://wap/pay?prepayid");
	                if (i == -1) { }
	                else {
	                    let j = val.indexOf("\";", i);
	                    val = val.slice(i, j);
	                    val = val.substr(5);
	                    console.info(val);
	                    window.location.href = val;
	                }
	            }
	            else {
	                console.info(val);
	                window.location.href = val;
	            }
	        }
	    }
	}

	class MyBridge {
	    static onASEvent(con, data) {
	        if (MyBridge.dicValue == null) {
	            MyBridge.Log("没有dicValue！无法进行接受事件");
	            return;
	        }
	        if (con == "统计") {
	            BGCon_Statistic.instance.onEventF(data);
	        }
	        else if (con == "支付") {
	            if (PlayerMain.ServerHttpURL == null) {
	                MyBridge.onLeave("缺少IP");
	                return;
	            }
	            if (data == null) {
	                MyBridge.onLeave("缺少data");
	                return;
	            }
	            new BGCon_Pay(data["type"], data["商品"], data["描述"], data["money"]);
	        }
	    }
	    static initF() {
	        window["eventListener"] = this.onASEvent.bind(this);
	        var url0 = window.location.search;
	        let value = url0.substr(url0.indexOf("?") + 1);
	        value = decodeURIComponent(value);
	        this.valueStr = value;
	        if (value && value.length > 0) {
	            try {
	                this.dicValue = JSON.parse(value);
	                if (this.dicValue["测试"] == "ffx") {
	                    this.dicValue["uid"] = 1;
	                    this.dicValue["IP"] = "shaoerzzl506719169.miracle-cn.com:41001";
	                    this.dicValue["项目"] = "火焰纹章";
	                    this.dicValue["渠道"] = "Local";
	                }
	                if (this.dicValue["项目"] != null)
	                    PlayerMain.Program = this.dicValue["项目"];
	                if (this.dicValue["uid"] != null)
	                    PlayerMain.UID = this.dicValue["uid"];
	                if (this.dicValue["dc"] != null)
	                    PlayerMain.DC = this.dicValue["dc"];
	                if (this.dicValue["nick"] != null)
	                    PlayerMain.Nick = this.dicValue["nick"];
	                if (this.dicValue["IP"] != null)
	                    PlayerMain.ServerHttpURL = this.dicValue["IP"];
	                if (this.dicValue["重启"] != null)
	                    PlayerMain.reopen = true;
	                if (this.dicValue["统计appid"] != null)
	                    PlayerMain.TD_AppID = this.dicValue["统计appid"];
	                if (this.dicValue["渠道"])
	                    Config.Platform = this.dicValue["渠道"];
	            }
	            catch (e) {
	                this.onLeave("初始化error：" + e + "，参数=" + value);
	                return;
	            }
	            this.Log("收到初始化参数：" + value);
	            if (PlayerMain.Program == null) {
	                this.onLeave("缺少项目");
	                return;
	            }
	        }
	        if (this.dicValue["测试"] == "ffx") {
	            BGCon_Statistic.instance.onEventF({ "con": "登陆", "data": { "uid": 1 } });
	        }
	    }
	    static js_to_as(type, value) {
	        if (this.dicValue == null && type == "download") {
	            this.Log("开始下载" + value);
	            let str = value;
	            window["funDownload"](value, str.substr(str.lastIndexOf("/") + 1));
	            return;
	        }
	        if (this.dicValue)
	            console.log("执行js_to_as：" + type + "，" + value);
	        return;
	        if (this.dicValue && this.dicValue["airweb"]) {
	            if (type == "exit") {
	                window.location.href = "http://exit/" + JSON.stringify(value);
	            }
	            else if (type == "download") {
	                window.location.href = "http://download/" + value;
	            }
	            else if (type == "copy") {
	                window.location.href = "http://copy/" + value;
	            }
	            else if (type == "Log") {
	                window.location.href = "http://trace/" + value;
	            }
	            else if (type == "endUrlValue") {
	                window.location.href = "http://endUrlValue/" + value;
	            }
	            else {
	                window.location.href = "http://" + type + "/" + value;
	            }
	            return;
	        }
	        if (window["sendToANE"]) {
	            try {
	                window["sendToANE"](type, value);
	            }
	            catch (error) {
	                console.error("执行sendToANE报错：" + error);
	            }
	        }
	        else {
	            console.error("没有sendToANE方法");
	        }
	    }
	    static Log(str) {
	        console.log("log:::" + str);
	        this.js_to_as("Log", str);
	    }
	    static onLeave(mess = null) {
	        this.js_to_as("exit", mess);
	    }
	}

	class Main {
	    constructor() {
	        MainManager.getInstence().init(Laya.stage);
	        MyBridge.initF();
	    }
	}
	Config.OS = navigator.platform;
	console.log(navigator.platform + "   ，OS=" + Config.OS);
	Config.stageW = 1280;
	Config.stageH = 640;
	if (window["Laya3D"])
	    Laya3D.init(Config.stageW, Config.stageH);
	else
	    Laya.init(Config.stageW, Config.stageH, Laya["WebGL"]);
	new Main();

}());
